var _ = require('/lib/lodash')._ ,
	tools = require('/lib/tools');
	
var HTTPClient = (function(){
	var self = {};
	
	self.isAbsoluteURL = function(url){
		var pattern_absurl=/^https?:\/\//;
		return pattern_absurl.test(url);
	};
	self.buildQueryString = function(params){
		var qs=[];
		for(var key in params){
			qs.push(encodeURIComponent(key)+'='+encodeURIComponent(params[key]));
		}
		qs=qs.join('&');
		return qs;
	};
	
	self.remote = function(url,options,ctx){
		// by default cache all the GET requests
		if(!('cache' in options))
			options['cache'] = options.method === 'GET';
		if(!('cache_expiration' in options))
			options['cache_expiration'] = 3600;
		
		options = _.extend({method:"GET",data:{}},options||{});

		if(!self.isAbsoluteURL(url)){
			alert('error');
			return false;
		}

		if (options.cache) {
    		responseText = Ti.App.Cache.get(options.cache_key || url);
    		
    		if(responseText)
    		{
	    		Ti.API.info('-------------------------------------------------------------------------------');
				Ti.API.info(options.method + ' Read from cache:url: '+url);
				Ti.API.info('-------------------------------------------------------------------------------');
		        url.indexOf('maps.googleapis.com')===-1 && Ti.API.info(responseText);
	    		if(options.success){
			       	options.success.call(ctx||null,responseText);
		        }
		        return;
		    }
    	}

		var xhr = Ti.Network.createHTTPClient({
		    onload: function(e) {
		    	try {
		    		var parsed = JSON.parse(this.responseText);
			    	if (options.cache && !parsed.error) {
			    		Ti.App.Cache.put(options.cache_key || url, this.responseText, options.cache_expiration);
			    	}
		    	}
		    	catch(e) {
		    		Ti.API.info('HTTPClient.remote: RESPONSE IS NOT JSON - url: ' + url);
		    	} 

		        Ti.API.info('-------------------------------------------------------------------------------');
				Ti.API.info(options.method + ' HTTPClient.remote:url: '+url);
				Ti.API.info('-------------------------------------------------------------------------------');
				
		        url.indexOf('maps.googleapis.com')===-1 && Ti.API.info(this.responseText);
		        if(options.success){
		        	options.success.call(ctx||null,this.responseText);
		        }
		    },
		    onerror: function(e) {
				// this function is called when an error occurs, including a timeout
				Ti.API.info('-------------------------------------------------------------------------------');
				Ti.API.info(options.method + ' Error HTTPClient.remote:url: '+url);
				Ti.API.info('-------------------------------------------------------------------------------');
		        Ti.API.info(e.error);
		        if(options.error){
		        	options.error.call(ctx||null,e.error);
		        }
		    },
			ondatastream: function(e) {
				// function called as data is downloaded
				Ti.API.info('ondatastream called, readyState = '+this.readyState);
		    },
		    onsendstream: function(e) {
				// function called as data is uploaded
				Ti.API.info('onsendstream called, readyState = '+this.readyState);
		    },
		    timeout:options.timeout||20000  /* in milliseconds */
		});
		xhr.open((options.method).toUpperCase(), url);//options.async||false
		
		if(options.contentType){
			xhr.setRequestHeader("Content-Type",options.contentType);
		}
		//send
		if (options.method === 'GET') {
			xhr.send();	
		} else {
			xhr.send(options.data);
		}
		
		
		return xhr;
	};
	
	return self;

})();

module.exports = HTTPClient;