var	_ = require('/lib/lodash')._ ,
	tools = require('/lib/tools'),
	Config = require('/config/config'),
	IcabbiAPI = require('/core/IcabbiAPI'),
	ConnectPayAPI = require('/core/ConnectPayAPI'),
	Events = require('/lib/Events'),
	sqlite = require('/lib/SQLiteManager'),
	$$ = require('/core/ThemeManager'),
	Geolocation = require('/lib/Geolocation'),
	NavigationController = require('/lib/NavigationController');
	
//All application functionality is namespaced here
var ICABBI = (function() {
	var self = {
			version:Titanium.Platform.version,
			height:Titanium.Platform.displayCaps.platformHeight,
			width:Titanium.Platform.displayCaps.platformWidth,
		},
		osname = Titanium.Platform.osname,
		locale = Titanium.Platform.locale;
	
//	self.Version = '0.0.0.1';
	//considering tablet to have one dimension over 900px - this is imperfect, so you should feel free to decide
	//yourself what you consider a tablet form factor for android
	self.isTablet = osname === 'ipad' || (osname === 'android' && (self.width > 899 || self.height > 899));
	
	self.Config = Config;
	IcabbiAPI.initialize(self.Config.IcabbiApi);

	self.initializePayment = function(){
		ConnectPayAPI.initialize(self.Config.ConnectPayApi);
	};
	self.initialize = function(data){
		_.extend(ICABBI.Config, data);
		Ti.API.info(JSON.stringify(ICABBI.Config));
		ICABBI.Config.formatDate = 'Do MMMM, dddd';
		ICABBI.Config.formatTime = 'hh:mma';
		ICABBI.Config.formatShortDate = 'MMM Do';
		ICABBI.Config.formatShortTime = 'hh:mma';
		
		var formatProvider = null;

		if(ICABBI.Config['formatting'] && ICABBI.Config['formatting']['date'] && ICABBI.Config['formatting']['date']['providers'] &&
					ICABBI.Config['formatting']['date']['providers']['momentjs'])
			formatProvider = ICABBI.Config['formatting']['date']['providers']['momentjs'];
			
		if(formatProvider)
		{
			Ti.API.info('config fdates');
			ICABBI.Config.formatDate = formatProvider['long'].date;
			ICABBI.Config.formatTime = formatProvider['long'].time;
		
			ICABBI.Config.formatShortDate = formatProvider['short'].date;
			ICABBI.Config.formatShortTime = formatProvider['short'].time;
		}
	};
	self.start = function(restart) {
/*
		if (!Ti.Android) {
			var geolistener = function(e) {
				return true;
			}
			
			var firstSuccess = function(e) {
				Geolocation.stopTracking(geolistener);
			};
			
			Geolocation.init({
				firstSuccess:firstSuccess,
			});
			Geolocation.startTracking(false,geolistener);
		}
*/
		if(restart)
			ICABBI.SlidingMenu.restart();
		else
		{
			Ti.API.error('repornadsa');
			var	Window,
				path_appwindow = Ti.Android ? 'ui/handheld/android/ApplicationWindow':'ui/handheld/ApplicationWindow';
			Window = require(path_appwindow);

			new Window().open();
		}
	};
	
	return self;
})();

ICABBI.navController = new NavigationController();


//ICABBI.Controller
ICABBI.Controller = (function(){
	var self = {
		userSession:false,
		metadata:false,
		creatingBooking : false,
		DEVICE_LOCATION_LATLNG:{},
	};
	// _.extend(self.__proto__,Events);
	_.extend(self,Events);
	
	var user, meta, bookingNotes;
	var ChildWindow = require('/ui/common/ChildWindow'),
		LoginView = require('/ui/common/LoginView'),
		MetaModel = require('/model/Meta').Model;
		UserModel = require('/model/UserModel');

	self.initialize = function(restart){
		if(!restart)
		{
			meta = new MetaModel();
			user = new UserModel();
		
			user.bookings.on('sync:active',function(){
				self.trigger('change:active_bookings',{count:this.size()+user.bookingsUpcoming.size()});
			},user.bookings);
			user.bookingsUpcoming.on('sync:upcoming',function(){
				self.trigger('change:active_bookings',{count:this.size()+user.bookings.size()});
			},user.bookingsUpcoming);
		}
		else
		{
			meta.initialize(true);
		}
		self.refreshBookingNotes();
	};
	
	self.metReady = function (){
		// setTimeout(function(){
		//get user location upfront
		Geolocation.getCurrentPosition(function(e){
			if (e.success) {
				self.DEVICE_LOCATION_LATLNG = {latitude:e.coords.latitude,longitude:e.coords.longitude};
				// just to speed up first location fill
				Geolocation.reverseGeocoder(e.coords, function(e){
					self.DEVICE_LOCATION_ADDRESS = e;
				});
			}
		});
		IcabbiAPI.validateUserSession({
			success: function(session){
				setTimeout(function(){user.trigger('sessionvalidated');},13);
				if(session){
					if(session.phone){
						session.phone={code:'',number:session.phone};
					}
					//token is valid and user is logged in by persistence
					user.set({authenticated: true},{session:session});
					self.userSession = true;

					self.checkActiveBookings();
				}else{
					user.set({authenticated:false});
					self.userSession = true;
				}
			}
		});
	};
	self.checkActiveBookings = function(){
		user.bookings.once('sync:active',function(e){
			// show the active booking
			user.bookings.find(function(thisBooking){
				if (thisBooking.isActive()) {
					setTimeout(function(){
						if(!self.creatingBooking) {
							var BookingDetailView = require('/ui/common/BookingDetailView');
							var config = {
								content:BookingDetailView,
								title:'Details',
								data:{booking:thisBooking},
								notScrollable:true,
							};
							
							var win = new ChildWindow(config);
							ICABBI.navController.open(win);
						}
					}, 3000);
				
					return true;
				}
				return false;
			});
		});
	};
	self.start = function(){
		//[sonal] right now making it a sync call..will review later
		if(meta.isReady())
			self.metReady();
		else
			meta.on('ready',function(){self.metReady();});
// },13)
	};
	self.getBookingNotes = function(){
		return bookingNotes;
	};
	self.getMeta = function(){
		return meta;
	};
	self.getUser = function(){
		return user;
	};
	self.getSession = function(){
		return IcabbiAPI.session;
	};

	self.refreshBookingNotes = function() {
		bookingNotes = [];
		IcabbiAPI.getBookingNotes({
			success: function(notes){
				_.each(notes,function(note){
					bookingNotes.push(note.note);
				});
				self.trigger('booking_notes_changed');
			},
			error : function(e){}
		});
	};
	
	self.getCardTypes = function(options){
		IcabbiAPI.getCardTypes(options);
	};
	
	self.updateUser = function(data,callback, errorCallback){
		$$.ActivityIndicator.start('Updating your profile');
		 user.save(data,{
			patch:1,
		 	wait:1,
			url:'profile/update',
			emulateJSONKey:'user',
			success: function(model, resp, options){
				$$.ActivityIndicator.stop();
				if(resp.error)
					errorCallback && errorCallback(resp.message);
				else
					callback && callback();
				
			},
			error: function(model, resp, options){
				$$.ActivityIndicator.stop();
				errorCallback && errorCallback(resp.message);
			}
		 }); 
		 //if only send attr in data..thn mark patch=1
		 //user.save(data,{patch:1}); 
	};
	
	self.updateUserProfile = function(data, options){
		options = options || {};
		IcabbiAPI.updateUserProfile(data, options); 
	};
	
	self.resetPassword = function(data, options){
		$$.ActivityIndicator.start('Updating your password');
		IcabbiAPI.resetPassword(data,{
			success: function(){
				$$.ActivityIndicator.stop();
				options.success && options.success();
			},
			error: function(){
				$$.ActivityIndicator.stop();
			}
		}); 
	};
	
	self.forgotPassword = function(data,options){
		$$.ActivityIndicator.start('Please Wait');
		IcabbiAPI.forgotPassword(data,{
			success: function(){
				$$.ActivityIndicator.stop();
				options.success && options.success();
			},
			error: function(e){
				$$.ActivityIndicator.stop();
				options.error && options.error(e.message ? e.message : 'Error sending email!');
			}
		}); 
	};
	
	//login - sends back all user attr 
	// YES with authenticated=1
	// NO with authenticated=0
	self.register = function(data,options){
		Ti.API.info('##########self.register #######################');
		Ti.API.info(JSON.stringify(data));
		$$.ActivityIndicator.start();
		IcabbiAPI.register(data,{
			success: function(session,err){
				if(!session){
					$$.ActivityIndicator.stop();
					options.error && options.error(err);
					return;
				}
				IcabbiAPI.validateUserSession({
					session:session,
					success: function(session){
						$$.ActivityIndicator.stop();
						if(session){
							if(session.phone){
								session.phone={code:'',number:session.phone};
							}
							user.set({authenticated: true},{session:session});
						}
						options.success && options.success(user);
					}
				});
			},
			error:function(e){
				$$.ActivityIndicator.stop();
				var err=[];
				err.push(e.message+':');
				for (var key in e.body){
					if(e.body[key])
						err.push(e.body[key]);
				}
				options.error && options.error(err);
			}
		});
	};
	// self.isMetaReady = function() {
		// if (meta.vehicleTypes && meta.destinations) {
			// self.metadata = true;
		// }
	// },
	self.wait = function(args) {
			args.callback();
return;
		args.message = args.message || 'Getting Data';
		args.timeout = args.timeout || 1000;
		
		if (args.condition()) {
			$$.ActivityIndicator.stop();
			args.callback();
		} else {
			$$.ActivityIndicator.start(args.message);
			setTimeout(function(){
				self.wait(args)
			},args.timeout);
		}
	},
	self.authenticate = function(data,options){
		Ti.API.info('##########self.authenticate #######################');
		Ti.API.info(JSON.stringify(data));
/*		
		if (!self.userSession || !self.metadata) {
			self.wait({
				condition:function(){
					return (self.userSession && self.metadata);
				},
				callback:function(){
					self.authenticate(data,options);
				},
			});
			return;
		}
*/
		if(user.isAuthenticated()){
			setTimeout(function(){
				options.success && options.success(user);				
			},100);
			return;
		}
		$$.ActivityIndicator.start();
		IcabbiAPI.authenticate(data,{
			success: function(session,err){
				Ti.App.Properties.setString("LoginEmail", data.username);
				
				$$.ActivityIndicator.stop();
				if(!session){
					$$.ActivityIndicator.stop();
					options.error && options.error(err);
					return;
				}
				if(session.phone){
					session.phone={code:'',number:session.phone}
				}
				user.set({authenticated: true},{session:session});
				
				self.checkActiveBookings();
				
				options.success && options.success(user);
			},
			error:function(e){
				options.error && options.error(e.message);
			}
		});
	};
	self.deauthenticate = function(options){
		options=options||{};
		Ti.API.info('##########self.DEauthenticate #######################');

		if(!user.isAuthenticated()){
			setTimeout(function(){options.success && options.success(user);},100);
			return;
		}
		var _call = function(){
			IcabbiAPI.deauthenticate({
					success:function(e){
						user.set({authenticated: false});
						self.trigger('change:active_bookings',{count:0});
						options.success && options.success();
					},
					error:function(e){
						options.success && options.success(null);
					}
				});
		};
		if(options.alert){		
			var c = Ti.UI.createAlertDialog({
				title:'Confirm',
				message:'Are you sure you want to logout?',
				buttonNames:['Yes','No'],
				cancel:1,
			});
			c.addEventListener('click',function(e){
				if (e.index === 0) {
					_call();
				}
			});
			
			c.show();
		} else {
			_call();
		}
	};
	
	self.displayLogin= function(options){
		options = options||{};
		if(user.isAuthenticated()){
			setTimeout(function(){
				options.success && options.success(user,function(){return;});
				//further two trigger are fake BUT IMPORTANT..will think something better later
				user.creditcards.trigger('fetched');
				user.clientaccounts.trigger('fetched');
				user.bookings.trigger('fetched');
			},100);
			return;
		}
		var LoginView = require('/ui/common/LoginView');
		var modal = new ChildWindow({content:LoginView,title:'Login', notScrollable:false,
			onClose : function(){
				options.close && options.close();
			}
		});
		ICABBI.navController.open(modal);
		modal.addEventListener('loggedIn',function(){
	   		if(user.isAuthenticated()){
	   			if(user.config_fetched)
	   			{
					options.success && options.success(user,function(animated){
						animated = animated||false;
		   				ICABBI.navController.close(modal,{animated:animated});
		   			});	   					
	   			}
	   			else
	   			{
	   				ICABBI.Controller.on('user:config:fetched', function(){
						options.success && options.success(user,function(animated){
							animated = animated||false;
			   				ICABBI.navController.close(modal,{animated:animated});
			   			});	   					
	   				});
	   			}
	   		}
	  	});
	};
	
	self.registerCreditCard = function(creditcard, options){
		options = options||{};
		user.registerCreditCard(creditcard, options);
	};
	
	self.processPayment = function(payment, options){
		options = options||{};
		IcabbiAPI.processPayment(payment, options);
	};
	
	self.confirmBooking = function(booking,options){
		options = options||{};
		IcabbiAPI.confirmBooking(booking,options);	
	};

	self.registerBooking = function(booking,options){
		options = options||{};
		// Ti.API.info('###########registerBooking ######################');
		// Ti.API.info(JSON.stringify(booking.toJSON()));
		// Ti.API.info('#################################');
		user.registerBooking(booking,function(){
   			options.success && options.success(booking);
		},function(resp){
   			options.error && options.error(resp);
		});	
	};
	
	self.updateBooking = function(booking,options){
		options = options||{};
		Ti.API.info('###########registerBooking ######################');
		Ti.API.info(JSON.stringify(booking.toJSON()));
		Ti.API.info('#################################');
		user.updateBooking(booking,function(){
   			options.success && options.success(booking);
		},function(resp){
   			options.error && options.error(resp);
		});	
	};
	
	//bookings/cancel/242609
	self.cancelBooking = function(booking,options){
		options = options||{};
		IcabbiAPI.remote('bookings/cancel/'+booking.get('perma_id'),{
			cache : false,
			success: function(e){
   			options.success && options.success(JSON.parse(e));
			}
		},function(resp){
   			options.error && options.error(resp);
		});
	};
	
	self.deleteBooking = function(perma_id,options){
		options = options||{};
		IcabbiAPI.remote('bookings/delete/'+perma_id,{
			cache : false,
			success: function(e){
   			options.success && options.success(JSON.parse(e));
			}
		},function(resp){
   			options.error && options.error(resp);
		});
	};
	
	self.updateAppSetting = function(appsetting,options){
		options = options||{};
		Ti.API.info('###########updateAppSetting ######################');
		Ti.API.info(JSON.stringify(appsetting.toJSON()));
		Ti.API.info('#################################');
		user.updateAppSetting(appsetting,function(){
   			options.success && options.success(appsetting);
		});	
	};

	self.getETA = function(location){
		// var data = {lat:"53.387423725746345",lng:"-6.065311431884765",vehicle_type:4};//,hours:now.getHours(),minutes:now.getMinutes()}; //vehicle_type
		var data = {lat:location.latitude.toString(),lng:location.longitude.toString()};//,hours:now.getHours(),minutes:now.getMinutes()}; //vehicle_type
		IcabbiAPI.getETA(data,{
			success: function(eta){
				self.trigger("change:eta",{eta:eta});
			}
		});
	};

	self.rateDriver = function(rating,options){
		options = options||{};
		IcabbiAPI.rateDriver(rating,options);	
	};


	self.on('slidingmenu:toggle',function(args){
		ICABBI.SlidingMenu.fireEvent('toggle');
	});
	
	self.on('slidingmenu:buttonclick',function(args){
		if(args.rowData.logout){
			if(user.isAuthenticated())
				self.deauthenticate({success : function(){
					ICABBI.Controller.displayLogin({
						success:function(user,callback){
							ICABBI.Controller.trigger('mapview:reset');
							ICABBI.SlidingMenu.fireEvent('openview',{index:ICABBI.Config.show_map?0:1});
							callback(true);
						},
					});
				}});
			else
			{
				ICABBI.Controller.displayLogin({
					success:function(user,callback){
						ICABBI.Controller.trigger('mapview:reset');
						ICABBI.SlidingMenu.fireEvent('openview',{index:ICABBI.Config.show_map?0:1});
						callback(true);
					},
				});
			}
		}
	});
	
	self.on('slidingmenu:swapview',function(args){
		ICABBI.SlidingMenu.fireEvent('swapview',{index:args.index});
	});
	
	self.on('slidingmenu:switch',function(args){
		Ti.API.debug(JSON.stringify(args));
		var authenticate = args.rowData.authenticate || false;
		//if public view or user already in skip
		if(!authenticate || user.isAuthenticated()){
			//open corresponding view
			ICABBI.SlidingMenu.fireEvent('openview',{index:args.index});
			return;
		}
		//show login dialog
		setTimeout(function(){
				self.displayLogin({
					success:function(user,callback){
						ICABBI.SlidingMenu.fireEvent('openview',{index:args.index});
						callback(true);
					},
				});
			},13
		);
	});
	
	return self;
	
})();

module.exports = ICABBI;