var	ICABBI = require('/core/icabbi');

var tools = require('/lib/tools'),
	_ = require('/lib/lodash')._ ,
	Geolocation = require('/lib/Geolocation');

if (Ti.Android) {
	var MapModule = require('ti.map');
}

var roundOff=function(num){
	return Number(num.toFixed(6));
};

//ModalView Component Constructor
var MapControl = (function() {
	var self = {};
	self.count=0;
	var hasMap = true;
	var firstTime = true;
	var _options = {
			latitude: 0,
		    longitude: 0,
		    latitudeDelta: 0.01,
		    longitudeDelta: 0.01,
		    display: "map",
		    annotations:[],
		    top:0
		},
		_region= {center:{latitude:0,longitude:0},user:{latitude:0,longitude:0}},
		_annotations = {userLocation:null,driverLocations:[]},
		_trackETA=true,
		_pauseRegionChangedListener=false,
		_setAddressPauseRegionChangedListener=false;
	
	self.init = function(options,addressEdited){
		_options = _.extend(_options,options||{});
		_addressEdited = addressEdited;
		self.view = self.view || createView();
		
		var Anno = Ti.Android ? MapModule:Ti.Map;
		
		_annotations.userLocation =
				Anno.createAnnotation({
					latitude	: 0,
					longitude	: 0,
					title		: '',
					subtitle	: '',
					animate		: true,
					image		: '/images/map/user-location-pin.png',
					draggable	: false,
		}); 
		// self.startTracking();
		self.centerUser();
		setTimeout(function(){
				// self.updateMapLocation(_options);
				self.getCurrentPosition();
			},1000
		);

	};
	
	
	// mapview events = complete|error|loading|regionchanged
	var createView = function(){
		var view;
		if (Ti.Android) {
			// make sure google play is set up
			var rc = MapModule.isGooglePlayServicesAvailable();
			switch (rc) {
			    case MapModule.SUCCESS:
			        Ti.API.info('Google Play services is installed.');
			        break;
			    case MapModule.SERVICE_MISSING:
			    	hasMap = false;
			        ICABBI.navController.showAlert({
					    title:'Map error',
					    message:'Google Play services is missing. Please install Google Play services from the Google Play store.',
					    close:true
					});
			        return Ti.UI.createView({backgroundColor:'#ccc'});
			        break;
			    case MapModule.SERVICE_VERSION_UPDATE_REQUIRED:
			    	hasMap = false;
			        ICABBI.navController.showAlert({
					    title:'Map error',
					    message:'Google Play services is out of date. Please update Google Play services.',
					    close:true
					});
			        return Ti.UI.createView({backgroundColor:'#ccc'});
			        break;
			    case MapModule.SERVICE_DISABLED:
			    	hasMap = false;
			        ICABBI.navController.showAlert({
					    title:'Map error',
					    message:'Google Play services is disabled. Please enable Google Play services.',
					    close:true
					});
			        return Ti.UI.createView({backgroundColor:'#ccc'});
			        break;
			    case MapModule.SERVICE_INVALID:
			    	hasMap = false;
			        ICABBI.navController.showAlert({
					    title:'Map error',
					    message:'Google Play services cannot be authenticated. Reinstall Google Play services.',
					    close:true
					});
			        return Ti.UI.createView({backgroundColor:'#ccc'});
			        break;
			    default:
			    	hasMap = false;
			        ICABBI.navController.showAlert({
					    title:'Map error',
					    message:'Unknown Google Maps error.',
					    close:true
					});
			        return Ti.UI.createView({backgroundColor:'#ccc'});
			        break;
			}
		 	view = MapModule.createView({
				mapType: MapModule.NORMAL_TYPE,
				region: {
				    	latitude		: _options.latitude,
				    	longitude		: _options.longitude,
				        latitudeDelta	: _options.latitudeDelta,
				        longitudeDelta	: _options.longitudeDelta
				    },	
	    	    animate:true,
			    regionFit:true,
			    userLocation:false,
			    annotations:_options.annotations,
			    enableZoomControls:false,
			   	// left:0,
			  	// top:_options.top || 0
			});
			_options.annotations.length && view.selectAnnotation(_options.annotations[0]);
		} else {
		 	view = Titanium.Map.createView({
				mapType: _options.mapType || Titanium.Map.STANDARD_TYPE,
				region: {
				    	latitude		: _options.latitude,
				    	longitude		: _options.longitude,
				        latitudeDelta	: _options.latitudeDelta,
				        longitudeDelta	: _options.longitudeDelta
				    },	
	    	    animate:true,
			    regionFit:true,
			    userLocation:false,
			    annotations:_options.annotations,
			   	// left:0,
			  	// top:_options.top || 0
			});
			
			_options.annotations.length && view.selectAnnotation(_options.annotations[0]);
		}
		_region.center.latitude =  roundOff(_options.latitude);
		_region.center.longitude = roundOff(_options.longitude);
		
		return view;		
	};	

	self.getView = function(){
		return self.view;
	};

	self.setAddressEdited = function(address,trackETA) {
		_trackETA = trackETA===void 0?true:trackETA;
		_addressEdited = address;
		_setAddressPauseRegionChangedListener = true;
		// if(_.isNumber(address.get('lat')) && _.isNumber(address.get('lng'))){
		if(address.get('lat').length && address.get('lng').length){
			self.updateMapLocation({latitude:address.get('lat'),longitude:address.get('lng')},false);
		}
	};
	
	self.addDummyDrivers = function(locations){ return;
		var Anno = Ti.Android ? MapModule:Ti.Map;
		var annotation;
		_.each(locations,function(location){
			 var annotation =	Anno.createAnnotation({
						latitude	: location.latitude,
						longitude	: location.longitude,
						title		: 'ETA 12 min',
						animate		: true,
						image		: '/images/map/enroute.png',
						draggable	: false,
			}); 
			self.addAnnotation(annotation);
			//_annotations.driverLocations.push(annotation);
		});
	};

    self.addAnnotation = function (annotation,select) {
    	if(!hasMap) return;
		self.view.addAnnotation(annotation);
		select && self.selectAnnotation(annotation);
    };
    //deselectAnnotation,removeAllAnnotation,removeAnnotation
    self.selectAnnotation = function (annotation) {
    	if(!hasMap) return;
		self.view.selectAnnotation(annotation);
    };

    self.removeAnnotation = function (annotation) {
    	if(!hasMap) return;
		self.view.removeAnnotation(annotation);
    };
    self.removeAllAnnotations = function () {
    	if(!hasMap) return;
		self.view.removeAllAnnotations();
    };
    
	self.createAnnotation = function(options,addToMap){
		if(!hasMap) return;
		addToMap===void 0 && (addToMap=true);
		
		var Anno = Ti.Android ? MapModule:Ti.Map;
		
		var annotation =	Anno.createAnnotation(_.extend({
				title		: '',
				animate		: true,
				draggable	: false,
		},options)); 
        if (addToMap)
            self.view.addAnnotation(annotation);

		return annotation;
	};
	
	//removeRoute
	self.removeRoute = function (route) {
		if(!hasMap) return;
		if(!route) return;
		route.route && self.view.removeRoute(route.route);
		route.annotations && self.removeAllAnnotations();
	};
	
	self.addRoute = function (data,options) {
		if(!hasMap) return;
		self.removeAllAnnotations();
		options = options||{};
		var points = data.points||[];
		if(points.length<2){
			alert('invalid points in addRoute. returning');
			return;
		}
		 var route = {
            name: data.name||'',
            width: 4,
			color: '#f00',
            points: points,
        };
        self.view.addRoute(route);
		var annotations = [];
		
		var Anno = Ti.Android ? MapModule:Ti.Map;
		
		if(options.annotations){
			_.each(options.annotations,function(annotation){
				annotation = Anno.createAnnotation(_.extend({
						title		: '',
						animate		: true,
						draggable	: false,
				},annotation));
				self.addAnnotation(annotation);
				annotations.push(annotation);
			});
		}
		if(options.regionFit){
			self.fitRegion(points);
		}

		return {route:route,annotations:annotations};
  	};

	self.fitRegion = function (points) {
		if(!hasMap) return;
		var max=min={lat:null,lng:null};
		var xMean=yMean=xSquared=ySquared=0;
		_.each(points,function(point){
			xMean+=point.latitude;
			yMean+=point.longitude;
		});
		xMean=xMean/points.length;
		yMean=yMean/points.length;

		_.each(points,function(point){
			xSquared+=Math.pow(point.latitude-xMean,2);
			ySquared+=Math.pow(point.longitude-yMean,2);
		});
		var xSd=Math.sqrt(Math.sqrt(xSquared/(points.length))),
			ySd=Math.sqrt(Math.sqrt(ySquared/(points.length)));

        setTimeout(function () { 
        	//self.view.regionFit = true;
        	Ti.API.info('CHANGE LOCATION 2 $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$');
        	self.view.setLocation({
				animate:true,
				regionFit:true,
				latitude:xMean,
				longitude:yMean,
				// latitudeDelta:xSd<ySd?xSd:ySd,
				// longitudeDelta:xSd<ySd?xSd:ySd,
				latitudeDelta:xSd/2,
				longitudeDelta:ySd/2,
			}); 
        	}, 700
    	);
  	};

	self.pauseRegionChangedListener = function(pause){
		pause===void 0 && (pause=false);
		_pauseRegionChangedListener=pause;
	};
	
	self.centerUser = function(){
		if(!hasMap) return;
		var timer = 0,
			delta = 1000;
		self.view.addEventListener('regionchanged', function(e) {
			self.pinMoved--;

			// Ti.App.fireEvent('locating');
Ti.API.info('------------------------------------------------------');
Ti.API.info('>>>>>> REGIONCHANGED: map region change'+ JSON.stringify(e));
Ti.API.info('>>>>>> REGIONCHANGED: _region.center'+ JSON.stringify(_region.center));
			//{latitude: 37.389569,latitudeDelta: 0.1,longitude: -122.050212,longitudeDelta: 1,source: "[object TiMapView]",type: 'regionchanged'}

			e.latitude = roundOff(e.latitude);
			e.longitude = roundOff(e.longitude);
			_region.center.latitude =  e.latitude;
			_region.center.longitude =  e.longitude;
			
			var data = {latitude:e.latitude,longitude:e.longitude,latitudeDelta:e.latitudeDelta,longitudeDelta:e.longitudeDelta,type:'regionchanged'};

			if(_pauseRegionChangedListener)
				return;
			if(_setAddressPauseRegionChangedListener) {
				_setAddressPauseRegionChangedListener = false;
				return;
			}
			timer && clearTimeout(timer);
			timer = setTimeout(function(){
Ti.API.info('>>>>>> REGIONCHANGED: calling updateMapLocation: '+ JSON.stringify(data));
					self.updateMapLocation(data);
				},delta
			);
		});			

	};

	self.getCurrentPosition = function(){
		// self.stopTracking();
		Geolocation.getCurrentPosition(function(e) {
			Geolocation.setTrackingCallback(null);
			self.updateUserLocation(e);
		});
		//self.startTracking();
	};

	self.startTracking = function(){
		// Geolocation.startTracking(true,self.updateUserLocation);
		Geolocation.setTrackingCallback(self.updateUserLocation);
	};
	
	self.stopTracking = function(){
		// Geolocation.stopTracking(self.updateUserLocation);
		Geolocation.setTrackingCallback(null);
	};
	
	self.updateUserLocation = function(e){
   		Ti.API.info('updateUserLocation: '+JSON.stringify(e));
		if (!e.success || e.error) {
       		Ti.API.info('########################Geolocation.getCurrentPosition error:' + JSON.stringify(e.error));
			return;
		}
		e.coords.latitude = roundOff(e.coords.latitude);
		e.coords.longitude = roundOff(e.coords.longitude);
		
		_region.user.latitude = e.coords.latitude;
		_region.user.longitude = e.coords.longitude;
		
		//{"success":true,"provider":{"power":3,"accuracy":1,"name":"gps"},"coords":{"timestamp":1360242079000,"speed":0,"altitude":0,"longitude":-121.08329833333333,"latitude":37.41199833333334,"accuracy":0,"heading":0,"altitudeAccuracy":null}}
		self.updateMapLocation(e.coords);
		
		self.pinMoved = firstTime ? 1 : 2;
		firstTime = false;
	};
	self.hasPinMoved = function()
	{
		return self.pinMoved <= 0;
	};
	self.updateMapLocation = function(location,reverseGeocode){
Ti.API.info('#################################################################');
Ti.API.info('updateMapLocation: location ' + JSON.stringify(location));
Ti.API.info('updateMapLocation: _region.center' + JSON.stringify(_region.center));
Ti.API.info('updateMapLocation: _region.user' + JSON.stringify(_region.user));
Ti.API.info('updateMapLocation: _addressEdited' + JSON.stringify(_addressEdited.toJSON()));
Ti.API.info('#################################################################');
			reverseGeocode=reverseGeocode||true;
			self.currentLocation = location;
			if(!ICABBI.Config.app_disable_eta)
				ICABBI.Controller.getETA(location);

			//redundant as following if will be always false from regionchanged event
			var regionChanged = (location.type && location.type==='regionchanged');
			
			if(_region.center.latitude!==location.latitude
				&& _region.center.longitude!==location.longitude)
			{
				//{"timestamp":1357909204000,"speed":0,"altitude":0,"longitude":-122.084095,"latitude":37.422005,"accuracy":0,"heading":0,"altitudeAccuracy":null}
				Ti.API.info('updateMapLocation: center changed');

				//[sonal] 
				if(!regionChanged){
					setTimeout(function(){
						Ti.API.info('CHANGE LOCATION $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$');
						if(!hasMap) return;
						self.view.setLocation({
								latitude		: location.latitude,
								longitude		: location.longitude,
								latitudeDelta	: location.latitudeDelta||_options.latitudeDelta,
								longitudeDelta	: location.longitudeDelta||_options.longitudeDelta,
								animate			: true,
								regionFit		: true
						});
					}, 700);
				}
			}
/*
			_annotations.userLocation.applyProperties({
				latitude:location.latitude,
				longitude:location.longitude
			});
*/			
			//_annotations.userLocation.setTitle(String.format('(%s,%s)',location.latitude,location.longitude));
			// _annotations.userLocation.setSubtitle(subtitle);

			if(!_addressEdited.same(location)){
				_addressEdited.set({id:'',lat:location.latitude,lng:location.longitude});
				reverseGeocode && self.reverseGeocode(location);
			}else{
				_addressEdited.trigger('change');
			}
			
			self.removeAllAnnotations();		
			// (roundOff(_region.user.latitude)===roundOff(_region.center.latitude) 
				// && roundOff(_region.user.longitude)===roundOff(_region.center.longitude)) ?
				// self.startTracking() : self.stopTracking();
	};
	
	self.reverseGeocode = function(location){ 
		Geolocation.reverseGeocoder(location,function(e){
			if(e.success){
				_addressEdited.set({id:e.id,src:e.src,area:e.area,formatted:e.formatted,lat:e.lat,lng:e.lng});
			}else{
				_addressEdited.set({src:'',area:'',formatted:'Address not found', lat:location.latitude,lng:location.longitude});
			}	
		});
	};
	return self;
})();

module.exports = MapControl;
