var _ = require('/lib/lodash');
var ErrorModal=require('/ui/common/Modal');
var PickerModal=require('/ui/common/PickerModal');

var NavigationController = function() {
	this.windowStack = [];
	this.notificationContainer = null;
	this.notificationOptions = [];
	this.notificationError = null;
	this.notificationErrorClick = null;
	
	this.pickerContainer = null;
	this.pickerOptions = [];
	this.pickerModal = null;
	this.pickerClick = null;
};

NavigationController.prototype.opening = function(button) {
	if (button.opening === undefined) {
		button.opening = true;
		
		// set a timeout to reset the button, just in case we get a race condition
		setTimeout(function(){
			if (button) {
				button.opening = false;
			}
		},3000);

		return false;
	} else if (!button.opening) {
		button.opening = true;
		return false;
	} else {
		return true;
	}
};
NavigationController.prototype.open = function(/*Ti.UI.Window*/windowToOpen,openProperties,button, restart) {
	if (typeof(openProperties) === 'undefined') {
		openProperties = {};
	}
	
	//add the window to the stack of windows managed by the controller
	this.windowStack.push(windowToOpen);

	//grab a copy of the current nav controller for use in the callback
	var that = this;
	
	//hack - setting this property ensures the window is "heavyweight" (associated with an Android activity)
	windowToOpen.navBarHidden = windowToOpen.navBarHidden || false;
	var hasBackLisener = false;
	
	if (button) {
		windowToOpen.addEventListener('open',function(){
			setTimeout(function(){
				button.opening = false;	
			},500);
		});
	}
	
	// if(windowToOpen.overlayWnd==null)
	// {
		// overlay = Ti.UI.createView({
			// width:   Ti.UI.FILL,
        	// height:   Ti.UI.FILL,
	        // backgroundColor:  '#fff',
			// zIndex:10000,
	        // //opacity: 0.85,
	        // visible : false
		// });
		// windowToOpen.add(overlay);
		// windowToOpen.overlayWnd = overlay;
	// }
			
	//This is the first window, so just return the window
	if(restart)
	{
		this.windowStack.splice(1, this.windowStack - 1);
		return this.windowStack[0]; 
	}
	else if(this.windowStack.length === 1) {
		if(Ti.Android) {
			windowToOpen.exitOnClose = true;

			windowToOpen.addEventListener('androidback', function(e) {
				var handled = false;
				
				
				if(ICABBI.SlidingMenu) {
					var currentView = ICABBI.SlidingMenu.currentView();
					
					if (currentView.onBackPressed) {
						handled = !currentView.onBackPressed(e);
					}
				}
				
				if(!handled) {
					ICABBI.navController.showYesNo({
				    	title:'Confirm',
					    message:'Are you sure you want to Exit Application?',
					    click : function(e){
					    	if(e.index == 1) {
						    	windowToOpen.close({});
					    	}
					    }
					});
				}
			});
			
			return windowToOpen;
		} else {
			this.navGroup = Ti.UI.iPhone.createNavigationGroup({
				window : windowToOpen
			});

			var containerWindow = Ti.UI.createWindow({
				navBarHidden:true,
				backgroundColor:'#000000',
			});
			containerWindow.add(this.navGroup);
			return containerWindow;
		}
	}
	
	//All subsequent windows
	else {
		windowToOpen.tabBarHidden = true; // for iOS tabs
		windowToOpen.back = true; // for Android ActionBar
		windowToOpen.addEventListener(Ti.Android?'androidback':'close', function(e) {
			if(that.windowStack.length-2 >= 0) {
				if(that.windowStack[that.windowStack.length-2].mapControl) {
					that.windowStack[that.windowStack.length-2].mapControl.left = 0;
				}
				// if(that.windowStack[that.windowStack.length-2].overlayWnd)
				//that.windowStack[that.windowStack.length-2].overlayWnd.setVisible(false);
			}
			e.cancelBubble = true;
			var win;
			// remove from stack
			for (var w=0,l=that.windowStack.length;w<l;w++) {
				if (e.source === that.windowStack[w]) {
					win = that.windowStack.splice(w,1)[0];
					break;
				}
			}
			if (e.type == 'androidback') {
				win.close({});
			}
//			win.doRelease();
		});
		
		if(Ti.Android) {
			windowToOpen.open(openProperties);
		} else {
			this.navGroup.open(this.windowStack[this.windowStack.length-1],openProperties);
		}
	}
	
	if(this.windowStack.length-2 >= 0) {
		if(this.windowStack[this.windowStack.length-2].mapControl) {
			this.windowStack[this.windowStack.length-2].mapControl.left = Ti.Platform.displayCaps.platformWidth;
		}
		
		//if(this.windowStack[this.windowStack.length-2].overlayWnd)
		//this.windowStack[this.windowStack.length-2].overlayWnd.setVisible(true);
	}

	this.showNotification();
};

NavigationController.prototype.close = function(toClose,closeProperties) {
	closeProperties = closeProperties||{};
	if(this.windowStack.length-2 >= 0) {
		if(this.windowStack[this.windowStack.length-2].mapControl) {
			this.windowStack[this.windowStack.length-2].mapControl.left = 0;
		}
		//if(this.windowStack[this.windowStack.length-2].overlayWnd)
		//this.windowStack[this.windowStack.length-2].overlayWnd.setVisible(false);
	}
				
	if (!toClose) {
		toClose = this.windowStack[this.windowStack.length-1];
	}
	//most weird...review later
	if (!toClose) {
		return;
	}
			
	if (Ti.Android) {
		toClose.close(closeProperties);
		// remove from stack
		for (var w=0,l=this.windowStack.length;w<l;w++) {
			if (toClose === this.windowStack[w]) {
				this.windowStack.splice(w,1)[0];
				break;
			}
		}
//		toClose.doRelease();
	} else {
		this.navGroup.close(toClose,closeProperties);
	}
};

NavigationController.prototype.destroy = function() {
	var windows = this.windowStack.concat([]);
	for (var i=windows.length-1;i > -1;i--) {
		if (this.navGroup) {
			this.navGroup.close(windows[i],{animated:false});
		} else {
			windows[i].exitOnClose = false;
			windows[i].close({});
			// remove from stack
			for (var w1=0,l1=this.windowStack.length;w1<l1;w++) {
				if (windows[i] === this.windowStack[w1]) {
					this.windowStack.splice(w1,1);
					break;
				}
			}
//			windows[i].doRelease();
		}
	}
//	this.windowStack = []; //reset stack
	
	if(this.navGroup) {
		this.navGroup.close();
	}
};

NavigationController.prototype.exit = function() {
	if(Titanium.Android && Titanium.Android.currentActivity){
		var activity = Titanium.Android.currentActivity;
	    activity.finish();
   	}
   	else if(this.navGroup) {
		this.navGroup.close();
	}
	else if(this.notificationContainer)
	{
		this.notificationError.setVisible(false);
		this.notificationContainer.exitOnClose = true;
		this.notificationContainer.close();
	}
};
NavigationController.prototype.home = function(endPoint) {
	if (!endPoint) {
		endPoint = 0;
	}
	//store a copy of all the current windows on the stack
	var windows = this.windowStack.concat([]);
	for (var i=endPoint+1,l=windows.length;i < l;i++) {
		if (this.navGroup) {
			this.navGroup.close(windows[i],{animated:i===l-1});
		} else {
			windows[i].close({});
			// remove from stack
			for (var w1=0,l1=this.windowStack.length;w1<l1;w1++) {
				if (windows[i] === this.windowStack[w1]) {
					this.windowStack.splice(w1,1);
					break;
				}
			}
//			windows[i].doRelease();
		}
	}
};

NavigationController.prototype.hidePicker = function() {
	this.pickerOptions.pop();

	if(this.pickerOptions.length > 0){
		this.showPicker();
	}
	else {
		if(this.pickerContainer){
			this.pickerModal.setVisible(false);
			this.pickerContainer.close();
		}
	}
};
NavigationController.prototype.onPickerClick = function(e) {
	if(this.pickerOptions && this.pickerOptions.length > 0){
		var options = this.pickerOptions[this.pickerOptions.length - 1];
		if(options && options.click)
			options.click(e);			
		if(!e.cancel) {
			this.hidePicker();
		}		
	}
};

NavigationController.prototype.showPickerModal = function(options, noshow) {
	this.showPicker(options, noshow);
};

NavigationController.prototype.showPicker = function(options, noshow) {
	if(options)
		this.pickerOptions.push(options);
	
	if(!this.pickerClick) {
		this.pickerClick = _.bind(this.onPickerClick, this);
			
		this.pickerContainer = Ti.UI.createWindow({
			exitOnClose:false,
	        backgroundColor:  'transparent',
			zIndex:100000,
		});
		var self = this;
		self.pickerModal = new PickerModal();
		
		self.pickerModal.addEventListener('buttons_click',self.pickerClick);
		this.pickerContainer.add(self.pickerModal);
		
		this.pickerContainer.addEventListener('open',function(){
			self.pickerModal.setOptions(self.pickerOptions[self.pickerOptions.length - 1]);
			self.pickerModal.setVisible(true);
		});
	}
			
	if(this.pickerOptions.length > 0 && !this.pickerContainer.visible && !noshow) {
		this.pickerContainer.open();
	}
	else {
		this.pickerModal.setVisible(false);
		this.pickerContainer.close();
	}
};


NavigationController.prototype.hideNotification = function() {
	this.notificationOptions.pop();

	if(this.notificationOptions.length > 0){
		this.showNotification();
	}
	else {
		if(this.notificationContainer){
			this.notificationError.setVisible(false);
			this.notificationContainer.close();
		}
	}
};
NavigationController.prototype.onNotificationErrorClick = function(e) {
	if(this.notificationOptions && this.notificationOptions.length > 0){
		var options = this.notificationOptions[this.notificationOptions.length - 1];
		if(options && options.click)
			options.click(e);
			
		if(!e.cancel) {
			this.hideNotification();
		}		
	}
};

NavigationController.prototype.showYesNo = function(options, noshow) {
	this.showNotification(_.assign(options, {
			'buttons' : [
					{ lable : 'Yes', icon : '/images/buttons/icon_button_ok.png'},
					{ lable : 'No', icon : '/images/buttons/icon_button_cancel.png'}
				],
		}), noshow);
};
NavigationController.prototype.showError = function(options, noshow) {
	this.showNotification(_.assign(options, {
			'titleIcon' : '/images/headers/icon_error.png',
			'titleColor' : '#c00',
			'messageColor' : '#c00',
		}), noshow);
};
NavigationController.prototype.showAlert = function(options, noshow) {
	this.showNotification(_.assign(options, {
			buttons : ['OK']
		}), noshow);
};
NavigationController.prototype.showNotification = function(options, noshow) {
	if(options)
		this.notificationOptions.push(options);
	
	if(!this.notificationErrorClick) {
		this.notificationErrorClick = _.bind(this.onNotificationErrorClick, this);
			
		this.notificationContainer = Ti.UI.createWindow({
			backgroundImage: Ti.Android ? null : '/Default.png', 
			exitOnClose:false,
	        backgroundColor:  'transparent',
			zIndex:100000,
		});
		var self = this;
		self.notificationError = new ErrorModal();
		
		self.notificationError.addEventListener('buttons_click',self.notificationErrorClick);
		this.notificationContainer.add(self.notificationError);
		
		this.notificationContainer.addEventListener('open',function(){
			self.notificationError.setOptions(self.notificationOptions[self.notificationOptions.length - 1]);
			self.notificationError.setVisible(true);
		});
	}
			
	if(this.notificationOptions.length > 0 && !this.notificationContainer.visible && !noshow) {
		this.notificationContainer.open();
	}
	else {
		this.notificationError.setVisible(false);
		this.notificationContainer.close();
	}
};


module.exports = NavigationController;