var RemoteImage = (function() {
	var self = {
		downloading : false,
		downloads : {},
		downloadsQueue : [],
	};
	
	function getExtension(fn) {
	    var re = /(?:\.([^.]+))?$/;
	    var tmpext = re.exec(fn)[1];
	    return (tmpext) ? '.' + tmpext : '';
	};
	
	self.cacheLifetime = 86400000;
	self.requestTimeout = 30000;
	
	function downloadImage(imageUrl, cacheFilename, callback) {
		if(cacheFilename in self.downloads)
		{
			self.downloads[cacheFilename].Callbacks.push(callback);
		}
		else
		{
			self.downloads[cacheFilename] = {
				ImageUrl : imageUrl,
				CacheFilename : cacheFilename,
				Callbacks : [callback]
			};
			self.downloadsQueue.push(cacheFilename);
		}
		
		if(!self.downloading) {
			downloadImageProc();
		}
	}
	
	function downloadImageProc() {
		if(self.downloadsQueue.length == 0)
		{
			self.downloading = false;
			return;
		}
		Ti.API.info('downloadImageProc===============================');
		self.downloading = true;
		
		var cacheFilename = self.downloadsQueue.pop();
		var download = self.downloads[cacheFilename];
		delete self.downloads[cacheFilename];
		var cacheFile = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, cacheFilename);
		
		var imageUrl = download.ImageUrl;
		Ti.API.info('Start download image: ' + imageUrl);
	
		// create http client to download the image
		var xhr = Ti.Network.createHTTPClient();
		xhr.setTimeout(self.requestTimeout);
		// … and assign the local image after successful download		
		xhr.onload = function() {
			if (xhr.status == 200) {
				try {
					// cache the file
					cacheFile.write(xhr.responseData);
					Ti.API.info();
					// assign the image
					if(download.Callbacks) {
						for(var i = 0; i < download.Callbacks.length; i++)
						{
							var callback = download.Callbacks[i];
							if(callback)
								callback(cacheFile.nativePath);
						}
					}
				}
				catch (e) {
					// handle error case
				}
			};
			
			// load next
			downloadImageProc();
		};
		xhr.onerror = function(){
			// load next
			downloadImageProc();
		};
		// cache overwrite
		if ( imageUrl.indexOf('?') > 0 ) {
			imageUrl += '&' + new Date().getTime();
		}
		else {
			imageUrl += '?' + new Date().getTime();
		}
		// send the request
		xhr.open('GET', imageUrl);
		xhr.send();
	}
	
	self.createRemoteImageView = function(options) {
		// check for cached local file
		var imageUrl = options.image;
		var cacheFilename = Ti.Utils.md5HexDigest(imageUrl)+getExtension(imageUrl);
		var cacheFile = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, cacheFilename);
	
		var imageWidth = options.width ? options.width : false,
			imageHeight = options.height ? options.height : false;
	
		// create the basic object without image url
		delete(options.image);
		delete(options.width);
		delete(options.height);
		
		var remoteImage = Titanium.UI.createImageView(options);
	
		// check for cached version
		if (cacheFile.exists() && cacheFile.modificationTimestamp() > (new Date().getTime()-self.cacheLifetime)) {
			// remoteImage.setImage(cacheFile.resolve());
			remoteImage.setImage(cacheFile.nativePath);
			if(imageWidth)
				remoteImage.setWidth(imageWidth);
			if(imageHeight)
				remoteImage.setHeight(imageHeight);
		}
		else {
			remoteImage.setImage(options.defaultImage || '/images/empty.png');
			if(imageWidth)
				remoteImage.setWidth(imageWidth);
			if(imageHeight)
				remoteImage.setHeight(imageHeight);
			
			downloadImage(imageUrl, cacheFilename, function (filename) {
				Ti.API.info('Image downloaded ===============================' + filename);
				remoteImage.setImage(filename);
				if(imageWidth)
					remoteImage.setWidth(imageWidth);
				if(imageHeight)
					remoteImage.setHeight(imageHeight);
			});
		}	
		return remoteImage;
	};

	return self;
})();

module.exports = RemoteImage;
