var APP_SQLITE_SCHEMA='ICABBIDB';

var SQLiteManager = (function(){
	var self = {schema:'',fileName:''},
		_meta = {},
		_conn=null,
		_versionChecked=false;
		
	self.open = function(){
		!_conn && (_conn = Ti.Database.open(self.schema));
	}	
	self.close = function(){
		_conn && _conn.close();
		_conn = null;
	}
	
	self.initialize = function(schema){
		self.schema = schema||APP_SQLITE_SCHEMA;
		self.open();
		//_conn.file.setRemoteBackup(false);
		// _conn = Ti.Database.install('/mydata/weatherDB', APP_SQLITE_SCHEMA);
		_conn.execute('CREATE TABLE IF NOT EXISTS meta(id INTEGER PRIMARY KEY AUTOINCREMENT, module TEXT, key TEXT, value TEXT);');
		_conn.execute('CREATE  TABLE IF NOT EXISTS bookings(id INTEGER PRIMARY KEY AUTOINCREMENT,perma_id INTEGER NOT NULL  UNIQUE ,favourite INTEGER NOT NULL  DEFAULT 0);');
		_conn.execute(
			"CREATE TABLE IF NOT EXISTS session( \
			id INTEGER PRIMARY KEY AUTOINCREMENT, \
			user_id INTEGER, \
			token TEXT,  \
			expires INTEGER, \
			fetched INTEGER \
			);" 
		);
		
		self.close();
	};
	
	self.getMeta = function(module,key,force){
		force===void 0 && (force=false);
		if(!module || module.length===0){
			return undefined;
		}
		var retVal=null;
		if(!force && _meta[module]){
			retVal = _meta[module];
			if(key){ 
				retVal = typeof retVal[key]!=='undefined'? retVal[key] : undefined;
			}
			return retVal;
		}
		_meta[module] || (_meta[module]={});

		self.open();
		//fetch all and cache even if asked for 1..saves time for further
		var rs = _conn.execute('SELECT key,value FROM meta where module=?',module);
		while(rs.isValidRow()){
			_meta[module][rs.fieldByName('key')] = rs.fieldByName('value');
			rs.next();
		}
		retVal=key && typeof retVal[key]!=='undefined' ? _meta[module][key] : undefined;

		rs.close();
		self.close();
		return retVal;
	};
	
	self.getSchemaVersion = function(force){
		force===void 0 && (force=true);
		return self.getMeta('schema','version',force);
	};
	
	self.getSession = function(){
		self.open();
		var session={},
			rs = _conn.execute('SELECT user_id,token,expires FROM session LIMIT 1');
		if(0===rs.getRowCount()){
			rs.close();
			self.close();
			return null;
		}
		// while (rs.isValidRow()) {
			session.user_id = rs.fieldByName('user_id');
			session.token = rs.fieldByName('token');
			session.expires = rs.fieldByName('expires');
			// rs.next();
		// }
		rs.close();
		self.close();
		return session;
	};
	
	self.setSession = function(session){
		if(!session.user_id || !session.token) return false;
		session.expires===void 0 || (session.expires=0);
		self.deleteSession(session);
		
		self.open();
		_conn.execute('INSERT into session(user_id,token,expires) VALUES(?,?,?)',
			session.user_id,
			session.token,
			session.expires
		);
		self.close();
	};
	
	self.deleteSession = function(session){
		self.open();
		_conn.execute('DELETE FROM session where user_id=? and token=?',session.user_id,session.token);
		self.close();
	};
	
	self.deleteSessions = function(session){
		self.open();
		_conn.execute('DELETE FROM session where user_id=?',session.user_id);
		self.close();
	};
	
	/*
		DUMMY calls
	*/
	self.manageFavourite = function(perma_id,favourite){
		self.open();
		_conn.execute('INSERT OR REPLACE INTO bookings(perma_id,favourite) VALUES(?,?)',perma_id,(favourite?1:0));
		self.close();
	};
	self.getFavourites = function(){
		self.open();
		var rs = _conn.execute('SELECT perma_id FROM bookings where favourite=1');
		var bookings=[];
		while(rs.isValidRow()){
			bookings.push(rs.fieldByName('perma_id'));
			rs.next();
		}
		self.close();
		return bookings;
	};
	
	self.initialize();
	return self;
})();

module.exports = SQLiteManager;

