/**
 * @author Sonal
 */
var tools = require('/lib/tools'),
	_ = require('/lib/lodash')._ ,
	Events = require('/lib/Events');



var MemoryDataStore = (function(options){
	var self = {};
	self.defaults = {};
	self.options = _.extend(self.defaults,options||{});
	_.extend(self,Events);
	
	var events=[];
	var types =['bool','double','int','list','object','string'];
	
	_.each(types,function(type){
		// to maintain consistency with Ti.App.Properties
		var setter = 'set'+type.charAt(0).toUpperCase() + type.slice(1);
		self[setter]=(function(type){
			return function(key,value){
				Ti.App.Properties[setter](key,value);
				self.change(type,key,value);
			};
		})(type);

		var getter = 'get'+type.charAt(0).toUpperCase() + type.slice(1);
		self[getter]=(function(type){
			return function(key,def){
				var val=def;
				if(Ti.App.Properties.hasProperty(key))
					val = Ti.App.Properties[getter](key);
				return val;
			};
		})(type);
	});
	
	self.set = function(key,value){
		var setter = null;
		if(_.isString(value)) setter='setString'; 
		else if(_.isArray(value)) setter='setList'; 
		else if(_.isPlainObject(value)) setter='setObject'; 
		else if(_.isBoolean(value)) setter='setBool'; 
		else if(_.isNumber(value)){
			var isFloat = function(n){return n===+n && n!==(n|0);};
			var isInt= function(n){return n===+n && n===(n|0);};
			setter = isFloat(value) ? 'setDouble' : 'setInt';  
		} 
		if(setter!==null){
			return self[setter](key,value);
		}
	};
		

	self.change = function(type,key,value){
		var args = {};
		args[key] = value;
		self.trigger('change:'+type+':'+key,args);
	};   
	
	return self;
})();
module.exports = MemoryDataStore;


/*
	mds.on('change:string:buildingname',function(e){
		label.setText(label.getText()+' '+e.buildingname);		
	});	
*/		
