/**
 * @author Sonal
 */
(function(){
  var _ = require('/lib/lodash')._ ,
	  Model = require('/model/Model').Model,
	  Store = require('/lib/Store'),
  	  Address = require('/model/Address'),
  	  VehicleType = require('/model/VehicleType'),
	  IcabbiAPI = require('/core/IcabbiAPI');

  var MetaModel = Model.extend({

    // Default attributes
    defaults: {
      version:'',
    },

	urlRoot: 'meta',
	storage: new Store(Store.Adapter.AJAX),
	
	//child collection
	_ready: false,
	vehicleTypes: null,
	destinations: null,
	
	isReady : function(){
		return this._ready;
	},
	initialize: function(restart){
		var self=this,
			fetched={vehicleTypes:false,destinations:false};
		
		this._ready = false;
		
		// vehicle types
		if(!restart)
			this.vehicleTypes = new VehicleType.Collection();
		this.vehicleTypes.once('fetched',function(){
			fetched.vehicleTypes=true;
			self._ready = (fetched.vehicleTypes && fetched.destinations);
			if(self._ready) self.trigger('ready');
		});
		this.fetchVehicleTypes();

		// destinations
		if(!restart)
			this.destinations = new Address.Collection();
		this.destinations.once('fetched',function(){
			fetched.destinations=true;
			self._ready = (fetched.vehicleTypes && fetched.destinations);
			if(self._ready) self.trigger('ready');
		});
		this.fetchDestinations();
	},
	defaultDestionations : function(){
		if(ICABBI.Config.behaviours && ICABBI.Config.behaviours.fields && ICABBI.Config.behaviours.fields.destination &&
				ICABBI.Config.behaviours.fields.destination['default'] && 
				(ICABBI.Config.behaviours.fields.destination['default'].formatted || ICABBI.Config.behaviours.fields.destination['default'].id))
			{
				this.destinations.add(new Address.Model({
					id: ICABBI.Config.behaviours.fields.destination['default'].id,
					formatted: ICABBI.Config.behaviours.fields.destination['default'].formatted,
					area : '',
					lat: null,
					lng: null,
				}));
			}
			// ready?
			this.destinations.trigger('fetched');
	},
	fetchDestinations: function(){ 
		var self = this;
		IcabbiAPI.getDestinations('',{
			success: function(addresses){
				Ti.API.info('=fetchDestinations------------------------------------');
				if(addresses) {
					_.each(addresses,function(address){
						self.destinations.add(new Address.Model({
							id: address.id,
							src: address.source,
							area: address.area,
							formatted: address.address,
							lat: Number(address.lat),
							lng: Number(address.lng),
						}));
					});
				}
				self.defaultDestionations();
			},
			error : function(e){
				// TODO: treat error
				self.defaultDestionations();
			}
		});
	},
	
	fetchVehicleTypes: function(){
		var vehicleTypes = this.vehicleTypes;
		IcabbiAPI.getVehicles({
			success: function(vehicles){
				_.each(vehicles,function(vehicle){
					vehicleTypes.add(new VehicleType.Model({
				    	key:vehicle.key,
				    	title:vehicle.title,
				    	seats:Number(vehicle.seats),
				    	wheelchair:vehicle.wheelchair,
				    	executive:vehicle.executive,
				    	saloon:vehicle.saloon,
				    	luggage:vehicle.luggage,
				    	image:vehicle.image,
					}));
				});
				// ready?
				vehicleTypes.trigger('fetched');
				// ICABBI.Controller.isMetaReady();
			},
			error : function(e){
				vehicleTypes.trigger('fetched');
			}
		});

	},

  });
  
  module.exports.Model = MetaModel;
})();
