/**
 * @author Sonal
 */
(function(){
  var _ = require('/lib/lodash')._ ,
	  Model = require('/model/Model').Model,
	  Store = require('/lib/Store'),
  	  CreditCard = require('/model/CreditCard'),
	  Booking = require('/model/Booking'),
	  ClientAccount = require('/model/ClientAccount'),
	  IcabbiAPI = require('/core/IcabbiAPI');


  var SYNC_INTERVAL_BOOKINGS={
	active:15,
	upcoming:30,
	previous:120,
  };

  
  var UserModel = Model.extend({

	config_fetched:false,
    // Default attributes
    defaults: {
      id:'',
      name: '',
      email: '',
      phone: {code:'',number:''},
	  bookingreceipts: 0,
      authenticated:false,
	  token:'',
	  return_discount_type:'NONE',
    },
	urlRoot: 'profile',
	storage: new Store(Store.Adapter.AJAX),
	// storage: new Store('user',Store.Adapter.AJAX),
	
	//child collection
	creditcards: null,
	clientaccounts: null,
	bookings: null,
	bookingsUpcoming: null,
	bookingsPrevious: null,
	userprofile: null,
	
	destinations: null,

	parse: function(resp,options){	
		var user = (resp.hasOwnProperty('email') && resp.hasOwnProperty('phone'))?resp:resp.user;
		if(user.phone){
			user.phone={code:'',number:user.phone};
		}
		return user;
	},

	initialize: function(restart){
		//profile
		this.on('change:authenticated',this.setSession,this);

		//credit cards
		this.creditcards = new CreditCard.Collection();
		// this.on('change:authenticated',this.fetchCreditCards,this);
		
		//client accounts
		this.clientaccounts = new ClientAccount.Collection();
		// this.on('change:authenticated',this.fetchClientAccounts,this);

		//booking
		this.bookings = new Booking.Collection();

		//bookingsUpcoming
		this.bookingsUpcoming = new Booking.Collection();

		//bookingsPrevious
		this.bookingsPrevious = new Booking.Collection();
		
		this.on('change:authenticated',function(){
			this.syncBookings(false);
			ICABBI.Controller.refreshBookingNotes();
		},this);
	},

	isAuthenticated : function(){
		return this.get('authenticated');
	},
	
	setSession:function(){Ti.API.info('setSession@@@@@@@@@@@@@@@@@@@@@@@');
		Ti.App.Cache.clear();
		var user = this;
		if(user.isAuthenticated()){
			var session = arguments[2].session;
			user.set({
				id: session.user_id,
				name: session.name,
				email: session.email,
				phone: session.phone,
				bookingreceipts: session.bookingreceipts,
				token: session.token,
				return_discount_type: session.return_discount_type,
				userprofile: session.userprofile,
			});
			
			user.fetchClientAccounts(session.accounts);

			user.fetchCreditCards(session.creditcards);

			if(!user.config_fetched)
			{
				user.fetchConfig();
			}
		}else{
			user.set(user.defaults);
			user.id='';
		}
	},
	fetchConfig: function(){
		var user = this;
		IcabbiAPI.getConfig({
			success: function(data){
				Ti.API.info('Confi Log: ' + JSON.stringify(data));
				user.config_fetched = true;
				 _.extend(ICABBI.Config || {}, data);
				 ICABBI.Config.ConnectPayApi.RETRUNURL = /*'http://digiapi.icabbi.com/digi/payment/return?app_key=07638957f6d9be7fa01da16233babb08053d3a4f';*/ICABBI.Config.IcabbiApi.ENDPOINT + 'payment/return?app_key=' + ICABBI.Config.IcabbiApi.APP_KEY;
				 ICABBI.Config.currencySign = '';
				if(ICABBI.Config.currency.html){
					ICABBI.Config.currencySign = String.fromCharCode(ICABBI.Config.currency.html.substring(2, ICABBI.Config.currency.html.length - 1));
				}

				 ICABBI.initializePayment();
				 ICABBI.Controller.trigger('user:config:fetched');
			}
		});
	
	},
	fetchClientAccounts: function(accounts){
		this.clientaccounts.reset();
		var clientaccounts = this.clientaccounts;
		if(accounts) {
			_.each(accounts,function(account){
				var clientaccount = new ClientAccount.Model;
				clientaccount.set({
					id:account.id,
					type:account.type,
					name:account.name,
					fields:account.account_fields
				});		
				clientaccounts.add(clientaccount);			
			});
		}
	},
	
	syncBookings: function(once){
		once===void 0 && (once=true);
		this.syncBookingsActive(once);
		this.syncBookingsUpcoming(once);
		this.syncBookingsPrevious(once);
	},
	
	syncBookingsActive: function(once){
		once===void 0 && (once=true);
		var user = this;
		var bookings = this.bookings;
		if(!user.isAuthenticated()){
			bookings.each(function(booking){
				booking.off();
			});
			bookings.reset();
			bookings.trigger('cleared');
			return;
		}
		bookings.fetchActive({
			success: function(collection, resp){
				bookings.trigger('sync:active');
				bookings.trigger('fetched');
			},
			error: function(){
			}
		});
		user.isAuthenticated() && !once && setTimeout(function(){user.syncBookingsActive(once);},SYNC_INTERVAL_BOOKINGS.active*1000);
	},

	syncBookingsUpcoming: function(once){
		once===void 0 && (once=true);
		var user = this;
		var bookings = this.bookingsUpcoming;
		if(!user.isAuthenticated()){
			bookings.each(function(booking){
				booking.off();
			});
			bookings.reset();
			bookings.trigger('cleared');
			return;
		}
		bookings.fetchUpcoming({
			success: function(collection,resp){
				bookings.trigger('sync:upcoming');
				bookings.trigger('fetched');
			},
			error: function(){
			}
		});
		user.isAuthenticated() && !once && setTimeout(function(){user.syncBookingsUpcoming(once);},SYNC_INTERVAL_BOOKINGS.upcoming*1000);
	},
	syncBookingsPrevious: function(once){
		once===void 0 && (once=true);
		var user = this;
		var bookings = this.bookingsPrevious;
		if(!user.isAuthenticated()){
			bookings.each(function(booking){
				booking.off();
			});
			bookings.reset();
			bookings.trigger('cleared');
			return;
		}
		bookings.fetchPrevious({
			success: function(collection,resp){
				bookings.trigger('sync:previous');
				bookings.trigger('fetched');
			},
			error: function(){
			}
		});
		user.isAuthenticated() && !once && setTimeout(function(){user.syncBookingsPrevious(once);},SYNC_INTERVAL_BOOKINGS.previous*1000);
	},

	
	registerBooking: function(booking,callback,errorcallback){
		var user = this;
		booking.save(null,{
			url:'bookings/add',
			emulateJSONKey:'booking',
			success: function(model){
				user.syncBookings(true);
				callback && callback();
			},
			error: function(model, resp, options){
				errorcallback && errorcallback(resp);
			}
		});
	},
	
	updateBooking: function(booking,callback,errorcallback){
		var bookings = this.bookings;
		booking.save(null,{
			patch:1,
		 	wait:1,
			url:'bookings/update',
			emulateJSONKey:'booking',
			success: function(model){
				bookings.update(model,{merge:true,remove: false});
				callback && callback();
			},
			error: function(model, resp, options){
				errorcallback && errorcallback(resp);
			}
		});
	},
	
	
/*
	//bookings/cancel/242609
	cancelBooking: function(booking,callback,errorcallback){
		var bookings = this.bookings;
		if(booking.isNew()){
			booking.destroy();
		}else{
			booking.save(null,{
				success: function(model){
					bookings.update(model,{merge:true,remove: false});
					callback && callback();
				}
			});
		}
	},
*/	
	fetchCreditCards: function(cards){
		this.creditcards.reset();
		var creditcards = this.creditcards;
		if(cards) {
			_.each(cards, function(card){
				var creditcard = new CreditCard.Model;
				creditcard.set(card);		
				creditcards.add(creditcard);			
			});
		}
		creditcards.fetched=true;
		creditcards.trigger('fetched');
	},

	registerCreditCard: function(creditcard, options){
		options=options||{};
		var success = options.success,
			creditcards = this.creditcards;
		
		options.success = function(card) {
			var creditcard = new CreditCard.Model;
			creditcard.set(card);
			creditcards.add(creditcard);
				
			success && success(creditcard);
		};
		
		IcabbiAPI.registerCreditCard(creditcard, options);
	}
	
  });
  
  module.exports = UserModel;
})();
