var ICABBI = require('/core/icabbi');

var _ = require('/lib/lodash')._ ,
	$$ = require('/core/ThemeManager');

var Theme = require('/ui/Theme');
var FormTextfieldRow = require('/ui/common/FormTextfieldRow');
var FormCheckboxRow = require('/ui/common/FormCheckboxRow');

//AccountView Component Constructor
var AccountView = function(options) {
	//create object instance, a parasitic subclass of Observable
	var defaults = {
			title:'My Details',
		};
	options = _.extend(defaults,options.data||{});
	
	var scroll = options.__parent.scroll;
	
	var nameRow,emailRow,phoneRow,currentPasswordRow,passwordRow,password2Row;
	
	var createForm = function(){
		//create object instance, a parasitic subclass of Observable
		var view = Titanium.UI.createView({
			left:0,
			right:0,
	        height:Ti.UI.SIZE,
			layout:'vertical'
	    });
	    
	    var form={fields:{}};
	    /*
	     * Name
	     */
	    nameRow = new FormTextfieldRow({
   			label:'Name',
			autocapitalization:true,
			autocorrect:true,
			returnKeyType:Ti.UI.RETURNKEY_NEXT,
	    });
		form.fields.name = {
			title:'Name',
			required:true,
			getValue: function(){nameRow.doBlur();return nameRow.value;}
		};
		view.add(nameRow);

	    /*
	     * Email
	     */
	    emailRow = new FormTextfieldRow({
   			label:'Email',
   			keyboardType:Ti.UI.KEYBOARD_EMAIL,
			autocapitalization:false,
			autocorrect:true,
			returnKeyType:Ti.UI.RETURNKEY_NEXT,
			prevField:nameRow,
	    });
		form.fields.email = {
			title:'Email',
			required:true,
			getValue: function(){emailRow.doBlur();return emailRow.value.replace(/\s/g,'').toLowerCase();}
		};
		view.add(emailRow);

	    /*
	     * Phone
	     */
	    var flexSpace = Titanium.UI.createButton({
		    systemButton:Titanium.UI.iPhone.SystemButton.FLEXIBLE_SPACE
		});
		 
		
		var done = Titanium.UI.createButton({
		    systemButton:Titanium.UI.iPhone.SystemButton.DONE
		});
		 
		done.addEventListener('click',function(){
		    phoneRow.done();
		});
	    phoneRow = new FormTextfieldRow({
   			label:'Phone',
			keyboardType:Titanium.UI.KEYBOARD_NUMBER_PAD,
			autocapitalization:false,
			autocorrect:false,
			returnKeyType:Ti.UI.RETURNKEY_DONE,
			prevField:emailRow,
			keyboardToolbar:[flexSpace, done]
	    });
		form.fields.phoneno = {
			title:'Phone Number',
			required:true,
			getValue: function(){phoneRow.doBlur();return phoneRow.value.replace(/[^0-9]/g,'');}
		};
		view.add(phoneRow);

	    /*
	     * Receipts
	     */
	    var receiptLabel = Ti.UI.createView({
	    	top:4,
	    	bottom:4,
	    	left:0,
	    	right:0,
	    	height:Ti.UI.SIZE,
	    	layout:'horizontal'
	    });

		var receiptPart1 = Ti.UI.createLabel({
			left:0,
			right:0,
			height:Ti.UI.SIZE,
			color:Theme.form.smallColor,
			font:Theme.form.smallFont,
			text:'Send an email receipt for all bookings',
			touchEnabled:false,
		});
		receiptLabel.add(receiptPart1);

	    var receiptRow = new FormCheckboxRow({
	    	labelView:receiptLabel,
   			value:true,
   			top:3,
   			labelCheck:true
	    });
		form.fields.bookingreceipts = {
			title:'Recipts',
			required:false,
			getValue: function(){return receiptRow.value;}
		};
		view.add(receiptRow);
		
		var buttonsView = Ti.UI.createView({
			left:0,
			right:0,
	        height:Ti.UI.SIZE,
		});
		view.add(buttonsView);

		/*
		 * Submit button
		 */
		var btnSave = Ti.UI.createButton({
			top:3,
			bottom:15,
			right:16,
			width:110,
			height:42,
			backgroundColor:Theme.global.buttonBGColor,
			borderColor:Theme.global.highlightColor,
			backgroundFocusedColor:Theme.global.highlightColor,
			backgroundSelectedColor:Theme.global.highlightColor,
			borderWidth:1,
			borderRadius:8,
			color:Theme.global.fontColor,
			font:Theme.form.buttonFont,
			title:'Save',
	  	    style:Ti.UI.iPhone.SystemButtonStyle.PLAIN,
		});
		Theme.animateButton(btnSave);
		buttonsView.add(btnSave);

		btnSave.addEventListener('click',function(e){
			var error= '';
			var noError= true;
			
			var requiredFields = ['name','email','phoneno'];
			var validated = true;
			for(var i=0,count=requiredFields.length;i<count;i++){
				var fld = form.fields[requiredFields[i]];
				if(!fld.getValue().length){
					error+= '- '+fld.title+' is Required!\n';
					noError= false;
				}
			};
			
			if(!noError) {
				ICABBI.navController.showError({
				    title:'Error!',
				    message:error,
				    close:true
				});
				return;
			}
			
			var data = {
				name: form.fields.name.getValue().replace(/^\s*|\s*$/g,''),
				email: form.fields.email.getValue().replace(/\s/g,'').toLowerCase(),
				bookingreceipts: form.fields.bookingreceipts?1:0,
				phone: form.fields.phoneno.getValue().replace(/[^0-9]/g,''),
			};
			ICABBI.Controller.updateUser(data,function(){
				ICABBI.navController.showAlert({
				    title:'Saved!',
				    message:'- Account Saved!',
				    close:true
				});
			}, function(e){
				ICABBI.navController.showError({
				    title:'Error!',
				    message:e,
				    close:true
				});
			}); 
		});
		
		/*
		 * Edit profile button
		 */
		/*var btnProfile = Ti.UI.createButton({
			top:3,
			bottom:15,
			right:142,
			width:110,
			height:42,
			backgroundColor:Theme.global.buttonBGColor,
			borderColor:Theme.global.highlightColor,
			backgroundFocusedColor:Theme.global.highlightColor,
			backgroundSelectedColor:Theme.global.highlightColor,
			borderWidth:1,
			borderRadius:8,
			color:Theme.global.fontColor,
			font:Theme.form.buttonFont,
			title:'Edit Profile',
	  	    style:Ti.UI.iPhone.SystemButtonStyle.PLAIN,
		});
		Theme.animateButton(btnProfile);
		buttonsView.add(btnProfile);

		btnProfile.addEventListener('click',function(e){
			var UserProfileView = require('/ui/common/UserProfileView'),
				ChildWindow = require('/ui/common/ChildWindow');
				
			var config = {
				content: UserProfileView,
				title: 'My Profile',
				header:{visible:true, controls:{cancel:{visible:true}, submit:{visible:false}, close:{visible:false}}},
				data:{},
			};
			var win = new ChildWindow(config);
			ICABBI.navController.open(win);
		});*/
		
		var section = Ti.UI.createView({
			left:0,
			right:0,
			height: 46,
			backgroundColor:Theme.nav.backgroundColor,
		});
		
		var sectionLineTop = Ti.UI.createView({
			left:0,
			right:0,
			top:0,
			height:1,
			backgroundColor:Theme.nav.seperatorColor,
		});
		section.add(sectionLineTop);
		
		var sectionLineTopWhite = Ti.UI.createView({
			left:0,
			right:0,
			top:1,
			height:1,
			backgroundColor:Theme.global.bgColor,
		});
		section.add(sectionLineTopWhite);
		
		var sectionLineBottomWhite = Ti.UI.createView({
			left:0,
			right:0,
			bottom:1,
			height:1,
			backgroundColor:Theme.global.bgColor,
		});
		section.add(sectionLineBottomWhite);

		var sectionLineBottom = Ti.UI.createView({
			left:0,
			right:0,
			bottom:0,
			height:1,
			backgroundColor:Theme.nav.seperatorColor,
		});
		section.add(sectionLineBottom);
	
	
		var text = Ti.UI.createLabel({
			text: 'Edit Password',
			left: 12,
			color:Theme.nav.headerColor,
			font:Theme.nav.headerFont,
		});
		section.add(text);
   		view.add(section);
	        
	    /*
	     * Password
	     */
	    currentPasswordRow = new FormTextfieldRow({
   			label:'Current',
			autocapitalization:false,
			autocorrect:false,
			passwordMask:true,
			returnKeyType:Ti.UI.RETURNKEY_NEXT,
	    });
		form.fields.currentPassword = {
			title:'Current Password',
			required:true,
			getValue: function(){currentPasswordRow.doBlur();return currentPasswordRow.value;}
		};
		view.add(currentPasswordRow);

	    passwordRow = new FormTextfieldRow({
   			label:'New',
			autocapitalization:false,
			autocorrect:false,
			passwordMask:true,
			returnKeyType:Ti.UI.RETURNKEY_NEXT,
			prevField:currentPasswordRow,
	    });
		form.fields.password = {
			title:'New Password',
			required:true,
			getValue: function(){passwordRow.doBlur();return passwordRow.value;}
		};
		view.add(passwordRow);

	    password2Row = new FormTextfieldRow({
   			label:'Confirm',
			autocapitalization:false,
			autocorrect:false,
			passwordMask:true,
			returnKeyType:Ti.UI.RETURNKEY_DONE,
			prevField:passwordRow,
	    });
		form.fields.password2 = {
			title:'Confirm Password',
			required:true,
			getValue: function(){password2Row.doBlur();return password2Row.value;}
		};
		view.add(password2Row);
		
		/*
		 * Password button
		 */
		var btnSavePW = Ti.UI.createButton({
			top:10,
			bottom:5,
			right:16,
			width:110,
			height:42,
			backgroundColor:Theme.global.buttonBGColor,
			borderColor:Theme.global.highlightColor,
			backgroundFocusedColor:Theme.global.highlightColor,
			backgroundSelectedColor:Theme.global.highlightColor,
			borderWidth:1,
			borderRadius:8,
			color:Theme.global.fontColor,
			font:Theme.form.buttonFont,
			title:'Save',
	  	    style:Ti.UI.iPhone.SystemButtonStyle.PLAIN,
		});
		Theme.animateButton(btnSavePW);
		view.add(btnSavePW);

		btnSavePW.addEventListener('click',function(e){
			var error= '';
			var noError= true;
			
			var requiredFields = ['currentPassword','password','password2'];
			var validated = true;

			for(var i=0,count=requiredFields.length;i<count;i++){
				var fld = form.fields[requiredFields[i]];
				if(!fld.getValue().length){
					error+= '- '+fld.title+' is Required!\n';
					noError=false;	
				}
			};
			
			if (noError && (form.fields.password.getValue() !== form.fields.password2.getValue())) {
				error= '- Passwords do not match!\n';
				noError=false;
			}
			
			if(!noError) {
				ICABBI.navController.showError({
				    title:'Error!',
				    message:error,
				    close:true
				});
				return;
			}
			
			var data = {
				password: form.fields.password.getValue(),
				password_confirm: form.fields.password2.getValue(),
				password_current: form.fields.currentPassword.getValue(),
			};
			ICABBI.Controller.resetPassword(data,{
				success: function(){
					currentPasswordRow.putValue('');
					passwordRow.putValue('');
					password2Row.putValue('');
					
					ICABBI.navController.showAlert({
					    title:'Saved!',
					    message:'- Password Changed!',
					    close:true
					});
				}
			}); 
		});
	
		view.addEventListener('slidingmenu:view:load',function(){
			var setValues = function(user){
				nameRow.putValue(user.get('name'));
				emailRow.putValue(user.get('email'));
				phoneRow.putValue(user.get('phone').number);
				receiptRow.putValue(!!user.get('bookingreceipts'));
			};
			var user = ICABBI.Controller.getUser(); 
				setValues(user);
			currentPasswordRow.putValue('');
			passwordRow.putValue('');
			password2Row.putValue('');
			});
		return view;
	}; 
	
	var self = createForm();
	self.onClose = function() {
		if(nameRow)
			nameRow.doBlur();
		if(emailRow)
			emailRow.doBlur();
		if(phoneRow)
			phoneRow.doBlur();
		if(currentPasswordRow)
			currentPasswordRow.doBlur();
		if(passwordRow)
			passwordRow.doBlur();
		if(password2Row)
			password2Row.doBlur();
	};
	return self;
};

module.exports = AccountView;
