var tools = require('/lib/tools'),
	_ = require('/lib/lodash')._ ,
	$$ = require('/core/ThemeManager'),
	Theme = require('/ui/Theme'),
	FormTextfieldRow = require('/ui/common/FormTextfieldRow'),
	FormCheckboxRow = require('/ui/common/FormCheckboxRow'),
	Geolocation = require('/lib/Geolocation'),
	ChildWindow = require('/ui/common/ChildWindow'),
	ICABBI = require('/core/icabbi'),
	MapControl = require('/lib/MapControl');

//EditAddressView Component Constructor
var AddressSearchView = function(options) {
	options = options || {};

	var headerHeight = footerHeight = 48;
	var searchTimer = null;
	var searchIdentifier = 0;
	var textField, tableView, recentIcon, textFieldWrapper;
	var recentActivated = false;
	var poiCategoryId = null, btnPoiCategory, btnPoiCategoryText, updatepoiCategoryLayout;
	
	var getCurrentPosition = function(){
		Geolocation.getCurrentPosition(updateUserLocation);
		Geolocation.setTrackingCallback(updateUserLocation);
	};
	
	var roundOff=function(num){
		return Number(num.toFixed(6));
	};

	var updateUserLocation = function(e){
		Ti.API.info('updateUserLocation: '+JSON.stringify(e));
		if (!e.success || e.error) {
       		Ti.API.info('########################Geolocation.getCurrentPosition error:' + JSON.stringify(e.error));
			return;
		}
		
		e.coords.latitude = roundOff(e.coords.latitude);
		e.coords.longitude = roundOff(e.coords.longitude);
		
		reverseGeocode(e.coords);
	};
	
	var reverseGeocode = function(location){ 

		Geolocation.reverseGeocoder(location,function(e){
			if(e.success){
				Geolocation.setTrackingCallback(null);
				textField.setValue(e.formatted.trim());
				setRecent(false);
				
				clearPOI();
				
				searchIdentifier = (new Date()).valueOf();
				searchAddresses(e.formatted.trim(), searchIdentifier);
			}	
		});
	};
	
	var fillTable = function(addresses) {
		Ti.API.info('fill address');
		var tableData = [];
		if(addresses) {
			_.each(addresses, function(address){
				tableData.push(createAddressRow({
				  	title: address.formatted||'',
				  	subtitle:address.area||'',
				  	data:address
				}));	
			});
		}
		
		if(tableData.length==0 && textField.getValue().length>3 && !ICABBI.Config.allow_free_input_address){
			textField.blur();
			ICABBI.navController.showAlert({
			    title:'No adress found!',
			    message:' Address not found. Please call the operator.',
			    close:true
			});
		}
		tableView.setData([]);
		tableView.setData(tableData);
	};
	var searchAddresses = function(text, timer) {
		
		 var address_book = recentActivated;
		 var lastPosition = Geolocation.getLastLocation();
		 var location = options.data.location;
		 
		if(!location && lastPosition) {
			location = {
				lat : lastPosition.coords ? roundOff(lastPosition.coords.latitude) : 0,
				lng : lastPosition.coords ? roundOff(lastPosition.coords.longitude) : 0
			};
		}
			
		if(address_book)
		{
			Geolocation.searchAddressBook(text, options.data.destination, location, function(addresses){
					if(timer == searchIdentifier) {
						fillTable(addresses);
					}
				});
		}
		else
			Geolocation.searchAddress(text, poiCategoryId, options.data.destination, location, function(addresses){
					if(timer == searchIdentifier) {
						fillTable(addresses);
					}
				});
	};
	
	var setRecent = function(r){
			recentActivated = r;
		if(recentActivated) {
			recentIcon.setImage('/images/buttons/icon_recent_checked.png');
		}
		else {
			recentIcon.setImage('/images/buttons/icon_recent.png');
		}
	};
		
	var clearPOI = function(){
		poiCategoryId = null;
		btnPoiCategoryText.setText('');
		btnPoiCategory.visible = false;
		textField.setLeft(30);
	};
	var createContent = function(){
		var content = Titanium.UI.createView({
			top:0,
			bottom:footerHeight,
			left:0,
			right:0,
		    backgroundGradient: {
		        type: 'linear',
		        startPoint: { x: '50%', y: '0%' },
		        endPoint: { x: '50%', y: '100%' },
		        colors: [ { color: 'white', offset: 0.0}, { color: '#f1f1f1', offset: 1.0 } ],
		    },
	    });

		var contentTopBorder = Ti.UI.createView({
			top: 0,
			left: 0,
			right:0,
			height:1,
			backgroundColor:'#ffffff',
		});
		content.add(contentTopBorder);
		var contentBottomBorder = Ti.UI.createView({
			bottom: 0,
			left: 0,
			right:0,
			height:1,
			backgroundColor:Theme.global.altBGColor,
		});
		content.add(contentBottomBorder);

		var searchBar = Titanium.UI.createView({
		    height:53,
		    top:1,
		    left:0,
		    right:0,
		    backgroundColor: '#dddddd',
		});
		var searchBorder = Ti.UI.createView({
			bottom: 0,
			left: 0,
			right:0,
			height:1,
			backgroundColor:Theme.global.altBGColor,
		});
		searchBar.add(searchBorder);

		textFieldWrapper = Ti.UI.createView({
			borderWidth: 1,
			borderRadius: 12,
			borderColor: '#676767',
			color: '#336699',
			backgroundColor: '#fff',
			top: 7,
			left: 7,
			right: 7,
			height: 38,
			zIndex: 1,
		});

		var searchLocation = Ti.UI.createView({
			left:7,
			top:12,
			height:15,
			width:15
		});
		
		var searchIcon = Ti.UI.createImageView({
			height:15,
			width:15,
			touchEnabled:false,
			image:'/images/buttons/icon_search.png',
		});
		searchLocation.add(searchIcon);
		textFieldWrapper.add(searchLocation);

		btnPoiCategory = Ti.UI.createView({
			left:7,
			top:7,
			width:Ti.UI.SIZE,
			animateType:Theme.ANTIMATE_TYPE_OPACITY,
			visible:false,
			backgroundColor: '#e5e5e5',
			height: 24,
			borderColor:'#7f7f7f',
			borderWidth:1,
			borderRadius:5,
		});
		updatepoiCategoryLayout = function(e){
			Ti.API.info('POST LAYOUT');
			if(poiCategoryId != null) {
				var width = btnPoiCategory.rect.width;
				if(width > 90) {
					btnPoiCategory.setWidth(90);
				}
				var pad = Ti.Android ? 7 : 14;
				textField.setLeft(pad+btnPoiCategory.rect.width);
			}
		};
		btnPoiCategory.addEventListener('postlayout',updatepoiCategoryLayout);
		btnPoiCategoryText = Ti.UI.createLabel({
			color:Theme.global.fontColor,
			left:5,
			right:18,
			text:'',
			touchEnabled:false,
		    wordWrap:false,
		    ellipsize:true,
			width: Ti.UI.SIZE,
			font:Theme.global.smallFont,
		});
		btnPoiCategory.add(btnPoiCategoryText);
		var clearPOIIcon = Ti.UI.createImageView({
			height:10,
			width:10,
			right:5,
			touchEnabled:false,
			image:'/images/buttons/icon_clear.png',
		});
		btnPoiCategory.add(clearPOIIcon);
		btnPoiCategory.addEventListener('click',function(e){
			clearPOI();
			searchIdentifier = (new Date()).valueOf();
			searchAddresses(textField.getValue(), searchIdentifier);
		});
		
		// TODO add a close image
		textFieldWrapper.add(btnPoiCategory);
						
		textField = Ti.UI.createTextField({
		  hintText:'Search',
		  backgroundColor:'transparent',
		  top: Ti.Android ? 3 : 0,
		  left: 25,
		  right: 30,
		  height: 38,
		  returnKeyType : Ti.UI.RETURNKEY_DONE,
		  zIndex:2,
		  //softKeyboardOnFocus:Ti.UI.Android.SOFT_KEYBOARD_SHOW_ON_FOCUS,
		});
		textFieldWrapper.add(textField);
		textField.addEventListener('change',function(e){
			if(searchTimer != null)
				clearTimeout(searchTimer);
			searchIdentifier = (new Date()).valueOf();
			var timer = searchIdentifier;
			searchTimer = setTimeout(function(){
				searchAddresses(e.value, timer);
			}, 3000);
		});
		var btnClearSearch = Ti.UI.createView({
			right:7,
			top:9,
			height:20,
			width:20,
			animateType:Theme.ANTIMATE_TYPE_OPACITY,
		});
		
		var clearIcon = Ti.UI.createImageView({
			height:20,
			width:20,
			touchEnabled:false,
			image:'/images/buttons/icon_clear.png',
		});
		btnClearSearch.add(clearIcon);
		Theme.animateButton(btnClearSearch);

		btnClearSearch.addEventListener('click',function(e){
			textField.setValue("");
			searchIdentifier = (new Date()).valueOf();
			searchAddresses("", searchIdentifier);
		});
		textFieldWrapper.add(btnClearSearch);
		
		content.add(searchBar);		
		content.add(textFieldWrapper);

		// test tableview
		
		var tableData = [];
		// todo add a recent location search
		// for (var i=1; i<=20; i++){
// 		  
		  // tableData.push(createAddressRow({
		  	// title:i+' create Qpjulting engineers Dunston hall bunkers bar, ipswich road ',
		  	// subtitle:'nr'+i+'AERRTTA dunston, NPORorwich',
		  // }));
		// }
		
		tableView = Ti.UI.createTableView({
		  backgroundColor:'white',
		  data:tableData,
		  separatorStyle:Ti.UI.Android ? undefined : Ti.UI.iPhone.TableViewSeparatorStyle.NONE,
		  allowsSelection:Ti.Android,
		  top: 53,
		  bottom: 1,
		  left: 0,
		  right:0,
		  separatorColor: '#ffffff',
		});
		
		tableView.addEventListener('click', function(e) {
			options.parent.window.__retval = {address:e.row.data};
			ICABBI.navController.close(options.parent.window);
		});
		content.add(tableView);

		return content;
	}; 

	var createAddressRow = function(data) {
		var row = Ti.UI.createTableViewRow({
			height:48,
			width:'100%',
			backgroundColor:'transparent',
			selectedColor: 'transparent',
			backgroundFocusedColor:'transparent',
			backgroundSelectedColor:'transparent',
			selectionStyle:Ti.Android?undefined:Ti.UI.iPhone.TableViewCellSelectionStyle.NONE,
		});
		row.data = data.data;
		row.className = 'datarow';
		row.clickName = 'row';
		
		var container=Ti.UI.createView({
			right:0,
			height:48,
			left:Ti.UI.SIZE,
		});
		row.add(container);
		var title = Ti.UI.createLabel({
			color:Theme.global.fontColor,
			left:10,
			right:36,
			top:3,
			height:23,
			text:data.title,
			touchEnabled:false,
		    wordWrap:false,
		    ellipsize:true,
			width: Ti.UI.FILL,
			verticalAlign:'bottom',
			minimumFontSize:Theme.address.addressFont.fontSize,
			font:Theme.address.addressFont,
		});
		container.add(title);
		
		var subtitle = Ti.UI.createLabel({
			color:'#999999',
			left:10,
			right:36,
			top:23,
			height:17,
			text:data.subtitle,
			touchEnabled:false,
		    wordWrap:false,
		    ellipsize:true,
			font:Theme.address.subaddressFont,
		});
		container.add(subtitle);
		
		
		var arrowIcon = Ti.UI.createImageView({
		    height:11,
		    width:11,
		    right: 7,
		    top:18,
		    image:'/images/map/arrow.png',
		    touchEnabled:false,
		});
		container.add(arrowIcon);
		
		var rowTopLine = Ti.UI.createView({
			left:0,
			right:0,
			top:0,
			height:'1px',
			backgroundColor:'magenta',
			touchEnabled:false,
		});
		//row.add(rowTopLine);
		var rowBottomLine = Ti.UI.createView({
			left:0,
			right:0,
			bottom:0,
			height:'1px',
			backgroundColor:'#cacaca',
			touchEnabled:false,
		});
		row.add(rowBottomLine);
		Theme.animateButton(container);	
		return row;
	};
		  
	var createFooter = function(){
		var footer = Ti.UI.createView({
			bottom: 0,
			left: 0,
			right:0,
			height: footerHeight,
			backgroundColor: '#dddddd',
		});
		
		var footerBorder = Ti.UI.createView({
			top: 0,
			left: 0,
			right:0,
			height:1,
			backgroundColor:"#ffffff",
		});
		footer.add(footerBorder);

		var btnRecentAddresses = Ti.UI.createView({
			left:8,
			bottom:0,
			height:47,
			width:40,
			animateType:Theme.ANTIMATE_TYPE_OPACITY,
		});
		recentIcon = Ti.UI.createImageView({
			height:47,
			width:40,
			top: 0,
			touchEnabled:false,
			image:'/images/buttons/icon_recent.png',
		});
		btnRecentAddresses.add(recentIcon);
		var labelRecent = Ti.UI.createLabel({
			bottom:0,
			height:15,
			width:Ti.UI.FILL,
			color:Theme.global.fontColor,
			font:Theme.footer.footerFont,
			text:'Recent',
			textAlign:'center',
			touchEnabled:false,
		});
		btnRecentAddresses.add(labelRecent);
		
		Theme.animateButton(btnRecentAddresses);
		btnRecentAddresses.addEventListener('click',function(e){
			setRecent(!recentActivated);
			clearPOI();
			searchIdentifier = (new Date()).valueOf();
			searchAddresses(textField.getValue(), searchIdentifier);
		});
		footer.add(btnRecentAddresses);
		
		var btnPOIs = Ti.UI.createView({
			right:8,
			bottom:0,
			height:47,
			width:40,
			animateType:Theme.ANTIMATE_TYPE_OPACITY,
		});
		var poisIcon = Ti.UI.createImageView({
			height:47,
			width:40,
			top:0,
			touchEnabled:false,
			image:'/images/buttons/icon_pois.png',
		});
		btnPOIs.add(poisIcon);
		var labelPOIs = Ti.UI.createLabel({
			bottom:0,
			height:15,
			width:Ti.UI.FILL,
			color:Theme.global.fontColor,
			font:Theme.footer.footerFont,
			text:'POI’s',
			textAlign:'center',
			touchEnabled:false,
		});
		btnPOIs.add(labelPOIs);
		
		Theme.animateButton(btnPOIs);
		btnPOIs.addEventListener('click',function(e){
			if (ICABBI.navController.opening(e.source)) {
				return;
			}
			textField.blur();
			openPOIs(e.source);
		});
		footer.add(btnPOIs);

		return footer;
	};
	var openPOIs = function(source) {
		var POIsView = require('/ui/common/POIsView');
		var config = {content:POIsView,
				header:{visible:true,controls:{cancel:{visible:true}}},
				notScrollable : true,
				title:'Points of Interest',
		};			
	    var win = new ChildWindow(config);
	    ICABBI.navController.open(win,undefined,source);
	    win.addEventListener('close',function(e){
				if(win.__retval && _.isObject( win.__retval) && win.__retval.poi){
					poiCategoryId = win.__retval.poi.id;
					btnPoiCategory.setWidth(Ti.UI.SIZE);
					btnPoiCategoryText.setText(win.__retval.poi.name);
					btnPoiCategory.visible = true;
					
					if(!Ti.Android) {
						updatepoiCategoryLayout();
					}
					searchIdentifier = (new Date()).valueOf();
					searchAddresses(textField.getValue(), searchIdentifier);
				}
				else if(win.__retval && _.isObject( win.__retval) && win.__retval.recent){
					setRecent(true);
					clearPOI();
					textField.setValue('');
					
					searchIdentifier = (new Date()).valueOf();
					searchAddresses(textField.getValue(), searchIdentifier);
				}
				else {
					clearPOI();
					textField.setValue('');
					
					searchIdentifier = (new Date()).valueOf();
					searchAddresses(textField.getValue(), searchIdentifier);
				}
			});
	};
	var create = function(){
		var view = Titanium.UI.createView({
			top:0,
			left:0,
			right:0,
	        height:Ti.UI.SIZE,
			//layout:'vertical',
	    });
		
		content = createContent();
		footer = createFooter();
		view.add(content);
		view.add(footer);

		options.parent.window.addEventListener('open',function(){
			var addr = '';
			searchIdentifier = (new Date()).valueOf();
			searchAddresses(addr, searchIdentifier);
			textField.blur();
		});

		if(!Ti.Android) {
			Ti.App.addEventListener('keyboardframechanged', function(e){
				Ti.API.info(JSON.stringify(e));
				if(Ti.Platform.displayCaps.platformHeight == e['keyboardFrame']['y']) // off
				{
					view.setHeight(Ti.UI.SIZE);
				}
				else {
					view.setHeight(view.size.height - e['keyboardFrame']['height']);
				}
			});
		}
		return view;
	};
	
	Geolocation.setTrackingCallback(null);
	var self = create();
	self.createHeaderRightControls = function(){
		var buttons = [];
		
		var btnCurrentLocation = Ti.UI.createView({
			right:0,
			height:48,
			width:50,
			animateType:Theme.ANTIMATE_TYPE_OPACITY,
		});
		
		var locationIcon = Ti.UI.createImageView({
			height:25,
			width:25,
			right:0,
			touchEnabled:false,
			image:'/images/map/btn_location.png',
		});
		btnCurrentLocation.add(locationIcon);
		Theme.animateButton(btnCurrentLocation);

		btnCurrentLocation.addEventListener('click',function(e){
			getCurrentPosition();
		});
		buttons.push(btnCurrentLocation);	
		return buttons;	
	};
	self.onClose = function(){
		if(poiCategoryId != null) {
			clearPOI();
			
			openPOIs();
			return true;
		}
		
		clearPOI();
		
		return false;
	};
	return self;
};

module.exports = AddressSearchView;
