var ICABBI = require('/core/icabbi');
var tools = require('/lib/tools'),
	_ = require('/lib/lodash')._ ,
	$$ = require('/core/ThemeManager'),
	Booking = require('/model/Booking'),
	mds = require('/model/MemoryDataStore');

var Theme = require('/ui/Theme');
var ActionButton = require('/ui/common/ActionButton');

//BookingDetailView Component Constructor
var BookingDetailView = function(options) {
	options = options || {};
	var ICABBI = require('/core/icabbi'),
		Events = require('/lib/Events');
	var cancellProcess = false;
	// var booking = options.data && options.data.booking  ? options.data.booking.clone() : (new Booking.Model());
	var booking = options.data && options.data.booking  ? options.data.booking : (new Booking.Model());
	var lastStatus = booking.stage() + '';
	var platformWidth = ICABBI.width,
		platformHeight = ICABBI.height;

	var bookingsActive = ICABBI.Controller.getUser().bookings;

	var proxy={};
	_.extend(proxy,Events);

	var highlightBGColor = Theme.makeTranslucentColor(Theme.global.highlightColor);

	var footer;
	var footerHeight = 81;
	
	var contentView, priceMainView, scheduledRow;
	
	var createContentRow = function(args){
		var self = Ti.UI.createView({
			left:0,
			right:0,
			height:54,
			args:args,
			backgroundColor:'transparent',
			putText:function(args){
				label.text = args.label;
				title.text = args.title;		
				title.top = args.subTitle.length ? 4 : undefined;
				subTitle.text = args.subTitle;
			}
		});
		
		var label = Ti.UI.createLabel({
			left:0,
			height:21,
			width:52,
			color:Theme.form.labelColor,
			font:Theme.form.labelFont,
			text:args.label,
			textAlign:'right',
			touchEnabled:false,
		});
		self.add(label);
		
		var title = Ti.UI.createLabel({
			left:args.label.length?60:12,
			right:25,
			top: args.wrap ?  (args.title.length>35 ? 4:undefined)  :(args.subTitle.length ? 4:undefined),
			height: args.wrap? Ti.UI.SIZE :28,
			color:Theme.global.fontColor,
			font:Theme.menu.rowFont,
			minimumFontSize:Theme.menu.rowFont.fontSize,
			wordWrap: args.wrap,
			ellipsize: true,
			width: Ti.UI.FILL,
			text:args.title,
			touchEnabled:false,
		});
		self.add(title);
		
		var subTitle = Ti.UI.createLabel({
			left:args.label.length?60:12,
			right:25,
			bottom:4,
			height:24,
			color:Theme.form.smallColor,
			font:Theme.form.smallFont,
			text:args.subTitle,
			touchEnabled:false,
			wordWrap:false,
			ellipsize:true,
			width: Ti.UI.FILL,
		});
		self.add(subTitle);
		
		if(!args.lastItem) {
			var rowLine = Ti.UI.createView({
				left:args.label.length? 60 : 0,
				right:11,
				bottom:0,
				height:1,
				backgroundColor:Theme.global.listBottomLineColor,
				touchEnabled:false,
			});
			self.add(rowLine);
		}
		
		var rowLineTop = Ti.UI.createView({
			left:args.label.length?60:0,
			right:11,
			top:0,
			height:1,
			backgroundColor:Theme.global.listTopLineColor,
			visible:!args.firstItem,
			touchEnabled:false,
		});
		self.add(rowLineTop);
		return self;
	};
	
	var createTextContentRow = function(args){
		var self = Ti.UI.createView({
			left:0,
			right:0,
			height: 80,
			args:args,
			backgroundColor:'transparent',
			putText: function(args){
				label.setText(args.label);
				
				if(content1.getVisible()) {
					content2.setValue(args.text);
					content1.hide();
					content2.show();
				}
				else {
					content1.setValue(args.text);
					content2.hide();
					content1.show();
				}
				
				self.args.label = args.label;
				self.args.text = args.text;
			}
		});
		var label = Ti.UI.createLabel({
			left:0,
			height: Ti.UI.SIZE,
			width:52,
			color:Theme.form.labelColor,
			font:Theme.form.labelFont,
			text:args.label,
			textAlign:'right',
			touchEnabled:false,
		});
		self.add(label);
		
		var addContent = function(text, visible) {
			var content = Ti.UI.createTextArea({
				backgroundColor: 'transparent',
				editable: false,
				left: args.label.length ? 52 : 2,
				right: 11,
				height: Ti.UI.SIZE,
				width: Ti.UI.FILL,
				color: Theme.global.fontColor,
				font: Theme.menu.rowFont,
				value: text,
				touchEnabled: false,
				visible: visible
			});
			self.add(content);
			return content;
		};
		
		var content1 = addContent(args.text, true),
			content2 = addContent('', false);
		
		if(!args.lastItem) {
			var rowLine = Ti.UI.createView({
				left:args.label.length? 60 : 0,
				right:11,
				bottom:0,
				height:1,
				backgroundColor:Theme.global.listBottomLineColor,
				touchEnabled:false,
			});
			self.add(rowLine);
		}
		
		var rowLineTop = Ti.UI.createView({
			left:args.label.length?60:0,
			right:11,
			top:0,
			height:1,
			backgroundColor:Theme.global.listTopLineColor,
			visible:!args.firstItem,
			touchEnabled:false,
		});
		self.add(rowLineTop);
		return self;
	};
	var mapModal = null;
	var openMap = function(mode){
		if(!mapModal) {
			var ChildWindow = require('/ui/common/ChildWindow'),
						BookingMapView = require('/ui/common/BookingMapView'),
				params = {
					content	: BookingMapView,
					header:{
						visible:true,controls:{cancel:{visible:true},submit:{visible:false},close:{visible:true}},
						//font:Theme.bookingMapView.headerFont
					},
					data:{
						Mode:mode,
						booking:booking
					},
				};
		   mapModal = new ChildWindow(params);
		   mapModal.addEventListener('close', function(){
				mapModal = null;
		   });
		   ICABBI.navController.open(mapModal);
		}
	};
	
	var oldPrice = null;
	var createPriceView = function(displayBooking){
		if(!priceMainView)
		{
			priceMainView = Titanium.UI.createView({
   				top:0,
		        left:7,
		        right:15,
				height:0,
		    });
			contentView.add(priceMainView);
		}		
			
		if(!ICABBI.Config.hide_prices) {
   			
	   		var payment = displayBooking.get('payment');
	   		   		
	   		if(!displayBooking.wasCancelled() && payment && payment.price) {
	   			
	   			
			    priceMainView.height = 80;
	
	   			var hasDetails = payment.tip || payment.tolls || payment.extras || payment.service_charge || payment.discount || (payment.total != 0 && payment.total != payment.price);
	   			var newPrice = '';
	   			if(hasDetails)
	   			{
	   				if(payment.total != 0 && payment.total != payment.price)
						newPrice += 'Fare:' + ICABBI.Config.currencySign + payment.price.toFixed(2);
					
					if(payment.tip)
						newPrice += 'Tip:' + ICABBI.Config.currencySign + String.formatDecimal(payment.tip, 'en-US', '0.00');
						
					if(payment.tolls)
						newPrice += 'Tolls:' + ICABBI.Config.currencySign + String.formatDecimal(payment.tolls, 'en-US', '0.00');
	
					if(payment.extras)
						newPrice += 'Extras:' + ICABBI.Config.currencySign + String.formatDecimal(payment.extras, 'en-US', '0.00');
					
					if(payment.service_charge)
						newPrice += 'Fee:' + ICABBI.Config.currencySign + String.formatDecimal(payment.service_charge, 'en-US', '0.00') +
							((payment.service_charge_type == 'PERCENTAGE' && payment.service_charge > payment.min_service_charge) ? '(' + payment.processing_fee + '%)' : '');
						
					if(payment.discount)
						newPrice += 'Disc:' + payment.discount + '%';
	   			}
	   			else
	   				newPrice += displayBooking.isFinished() ? '(Fare)' : (payment.fixed ? '(Fixed Fare)' : '(Estimated)');
	   			
				newPrice += ICABBI.Config.currencySign + (payment.total != payment.price ? payment.total.toFixed(2) : payment.price.toFixed(2));
						
	   			if(oldPrice != newPrice) {
		   			oldPrice = newPrice;
		   			priceMainView.removeAllChildren();
		   			
		   			if(hasDetails) {
						var priceVertLine = Ti.UI.createView({
							left:'50%',
							top: 10,
							bottom:10,
							width:1,
							backgroundColor:Theme.global.listBottomLineColor,
							touchEnabled:false
						});
						priceMainView.add(priceVertLine);
						
		   				var priceDetailsView = Titanium.UI.createView({
			   				left: 0,
			   				right: '50%',
							layout:'vertical',
							height: Ti.UI.SIZE,
			   			});
						priceMainView.add(priceDetailsView);
						
						var newHeight = 0;
						
						var addSmallPrice = function(priceText, priceValue) {
			   				var priceSmallRow = Titanium.UI.createView({
				   				left: 0,
				   				right: 0,
				   				height: 22,
				   			});
				   			priceDetailsView.add(priceSmallRow);
				   			
							var priceLabel = Ti.UI.createLabel({
								left: 0,
								width: 45,
								color:Theme.form.labelColor,
								font:Theme.form.labelFont,
								text:priceText,
								touchEnabled:false,
								textAlign:Ti.UI.TEXT_ALIGNMENT_LEFT,
							});
							priceSmallRow.add(priceLabel);
				   			
							var priceValLabel = Ti.UI.createLabel({
								left: 45,
								right: 5,
								color:Theme.childwindow.headerColor,
								font:Theme.form.labelFont,
								text:priceValue,
								touchEnabled:false,
								textAlign:Ti.UI.TEXT_ALIGNMENT_LEFT,
							});
							priceSmallRow.add(priceValLabel);
							
							newHeight += 30;
						};
										
						if(payment.total != 0 && payment.total != payment.price)
							addSmallPrice('Fare:', ICABBI.Config.currencySign + payment.price.toFixed(2));
						
						if(payment.tip)
							addSmallPrice('Tip:', ICABBI.Config.currencySign + String.formatDecimal(payment.tip, 'en-US', '0.00'));
							
						if(payment.tolls)
							addSmallPrice('Tolls:', ICABBI.Config.currencySign + String.formatDecimal(payment.tolls, 'en-US', '0.00'));
		
						if(payment.extras)
							addSmallPrice('Extras:', ICABBI.Config.currencySign + String.formatDecimal(payment.extras, 'en-US', '0.00'));
						
						if(payment.service_charge)
							addSmallPrice('Fee:', ICABBI.Config.currencySign + String.formatDecimal(payment.service_charge, 'en-US', '0.00') +
							((payment.service_charge_type == 'PERCENTAGE' && payment.service_charge > payment.min_service_charge) ? '(' + payment.processing_fee + '%)' : ''));
							
						if(payment.discount)
							addSmallPrice('Disc:', payment.discount + '%');
							
						if(newHeight > 80) {
							priceMainView.setHeight(newHeight);
						}
		   			}
		   			
		   			var priceTotalView = Titanium.UI.createView({
		   				left: hasDetails ? '50%' : 0,
		   				right: 0,
						layout:'vertical',
						height: Ti.UI.SIZE,
		   			});
		   			priceMainView.add(priceTotalView);
		   			
					var farePrice = Ti.UI.createLabel({
						right:0,
						left:0,
						color:Theme.childwindow.headerColor,
						font:Theme.bookingReview.totalPrice,
						text:ICABBI.Config.currencySign + (payment.total != payment.price ? payment.total.toFixed(2) : payment.price.toFixed(2)),
						touchEnabled:false,
						textAlign:Ti.UI.TEXT_ALIGNMENT_CENTER,
					});
					priceTotalView.add(farePrice);
		   			
					if(!hasDetails) {
						var farePriceLabel = Ti.UI.createLabel({
							right:0,
							left:0,
							top:-10,
							bottom:0,
							color:Theme.childwindow.headerColor,
							font:Theme.bookingReview.totalPriceLabel,
							text: displayBooking.isFinished() ? '(Fare)' : (payment.fixed ? '(Fixed Fare)' : '(Estimated)'),
							touchEnabled: false,
							textAlign:Ti.UI.TEXT_ALIGNMENT_CENTER,
						});
						priceTotalView.add(farePriceLabel);
					}
					   			  			
					var priceBottomLine = Ti.UI.createView({
						left:0,
						right:0,
						bottom:0,
						height:1,
						backgroundColor:Theme.global.listBottomLineColor,
						touchEnabled:false
					});
					priceMainView.add(priceBottomLine);
				}
	   		}
	   		else
   				priceMainView.height = 0;
   		}
   		else
   			priceMainView.height = 0;
	};
	
	var createContent = function(){
		//create object instance, a parasitic subclass of Observable
		mainview = Titanium.UI.createView({
			top:0,
	        left:0,
	        right:0,
	        bottom:0,
		    /*backgroundGradient: {
		        type: 'linear',
		        startPoint: { x: '50%', y: '0%' },
		        endPoint: { x: '50%', y: '100%' },
		        colors: [ { color: 'white', offset: 0.0}, { color: '#f1f1f1', offset: 1.0 } ],
		    },
			*/
			backgroundColor:'#f1f1f1',
	    });

		scrollVView =  Ti.UI.createScrollView({
				scrollType:'vertical',
		        bottom:footerHeight,
		        top:0,
			});
			
	    mainview.add(scrollVView);

	    contentView = Ti.UI.createView({
			top:0,
	        left:8,
	        right:0,
			layout:'vertical',
	    });
	    scrollVView.add(contentView);
		
		/*
   		 * Price
   		 */
   		createPriceView(booking);
		
	    // if this is active, display a special row
	    if (booking.isScheduled() || booking.isActive()) {
	    	var currentETA = booking.get('live_eta') || booking.get('eta') || 'NA';
	    	var oldETA = 525600; // moments so dear
	    	
	    	var etaRow = Ti.UI.createView({
	    		left:0,
	    		right:0,
	    		height:currentETA?100:0,
	    		layout:'horizontal',
	    	});
		
			var lblEstimate = Ti.UI.createLabel({
				width:60,
				height:34,
				color:Theme.form.smallColor,
				font:Theme.global.smallFont,
				textAlign:'right',
				text:'Estimated\nArrival',
			});
			etaRow.add(lblEstimate);
	
			var timeBox = Ti.UI.createView({
				left:10,
				height:70,
				width:Ti.UI.SIZE,
			});
			etaRow.add(timeBox);
		
			// add booking ref number
			
		    var refRow = createContentRow({
		    	label:'Ref',
		    	title:booking.perma_id||booking.id,
		    	subTitle:'',
		    	wrap: false,
		    });
	    	contentView.add(refRow);
			var doTime = function() {
				if (oldETA === currentETA) {
					return;
				} else {
					oldETA = currentETA;
				}
				
				currentETA = booking.get('live_eta') || booking.get('eta') || 'NA';
				
				etaRow.height = currentETA?0:100;
				
				if (timeBox.children && timeBox.children.length) {
					timeBox.removeAllChildren();
				}
				
				var etaArray = currentETA.toString().split('');
				var thisLeft = 0;
				
				for (var i in etaArray) {
					var box = Ti.UI.createView({
						height:34,
						width:31,
						left:thisLeft,
						backgroundImage:'/images/map/background_minute.png'
					});
					
					var digit = Ti.UI.createLabel({
						height:45,
						width:31,
						color:Theme.map.timeColor,
						font:Theme.map.timeFont,
						text:etaArray[i],
						textAlign:'center',
					});
					box.add(digit);
					
					timeBox.add(box);
					
					thisLeft += 32;
				}
			};
			
			doTime();
			
			var lblMins = Ti.UI.createLabel({
			    left:10,
				height:34,
			    width:70,
			    color:Theme.global.fontColor,
			    font:Theme.map.infoFont,
			    text:'Minutes',
			});
			etaRow.add(lblMins);

			proxy.listenTo(bookingsActive,'sync:active',function(){
				var syncedBooking;
				if(syncedBooking = bookingsActive.get(booking.get('perma_id'))){
					doTime();
				}
			});
			
			contentView.add(etaRow);

			var bookingDate = booking.formatDateLocaleDisplay();
						
		    scheduledRow = createContentRow({
		    	label:'When',
			    title: bookingDate.date,
		    	subTitle: bookingDate.time,
		    	firstItem:true,
		    	wrap: false,
		    });
		    scheduledRow.top = 10;
		    contentView.add(scheduledRow);

		} else {
			var bookingDate = booking.formatDateLocaleDisplay();
		    var bookedRow = createContentRow({
		    	label:'Booked',
			    title: bookingDate.date,
		    	subTitle: bookingDate.time,
		    	firstItem:true,
		    	wrap: false,
		    });
		    bookedRow.top = 10;
		    contentView.add(bookedRow);
		}
	    
	    // common stuff
		var arrFrom = booking.get('address');		
	    var fromRow = createContentRow({
	    	label:'From',
	    	title:arrFrom.formatted,
	    	subTitle:arrFrom.area||'',
	    	wrap: false,
	    });
	    //fromRow.top = bookedRow?0:10;
	    contentView.add(fromRow);

		var arrTo = booking.get('destination');
		
	    var toRow = createContentRow({
	    	label:'To',
	    	title:arrTo.formatted,
	    	subTitle:arrTo.area||'',
	    	wrap: false,
	    });
	    
	    contentView.add(toRow);
		
		var vias = booking.get('vias');
		var viaRows = [];
		if(vias && vias.length) {
			for(var idx in vias) {				
			    var viaRow = createContentRow({
			    	label : 'Via ' + (parseInt(idx) + 1),
			    	title : vias[idx].formatted,
			    	subTitle : vias[idx].area || '',
			    	wrap: false,
			    });
			    contentView.add(viaRow);
			    viaRows.push(viaRow);				
			}
		}
		var paymentType = booking.get('payment_type');
		if(paymentType == booking.PAYMENT_MODES.ACCOUNT) {
			var account_id = booking.get('account_id');
			
			var clientaccounts = ICABBI.Controller.getUser().clientaccounts;
			clientaccounts.each(function(clientaccount){
				Ti.API.info(JSON.stringify(clientaccount.toJSON()));
				if(clientaccount.get('id') == account_id)
				{
					paymentType = clientaccount.get('name') || 'Account';
				}
			});
			 
		}
		 var typeRow = createContentRow({
	    	label:'Type',
	    	title:paymentType,
	    	subTitle:'',
	    	wrap: false,
	    });
	    
	    contentView.add(typeRow);
	    
		var driver = booking.get('driver');
		var showDriverRow = (driver.id && !(booking.isDispatched() || booking.isScheduled()));
	    var driverRow = createContentRow({
	    	label:'Driver',
	    	title: driver.name+'/'+driver.vehicle.make+' '+driver.vehicle.model,
	    	subTitle:'Vehicle Reg: ' + driver.vehicle.reg,
	    	lastItem:false,
	    	wrap: false,
	    });
		if(showDriverRow){
			driverRow.height = 54;
		}else{
			driverRow.height = 0;
		}
	    contentView.add(driverRow);
	    
	    var statusRow = createTextContentRow({
	    	label: 'Status',
	    	text: booking.isDispatched() ? 'Assigning Driver, Please Wait' : booking.get('status_text'),
	    	lastItem:true,
	    });

	    contentView.add(statusRow);
		
			
	    var //statusWaitText = 'Thank you for your patience, please wait',
	    	statusText = statusRow.args.text, 
	    	oldStatusText = statusRow.args.text;
	    	
	    	
	    if(statusText === '') {
	    	statusText = booking.isDispatched() ? 'Assigning Driver, Please Wait' : 'Your taxi is scheduled for ' + bookingDate.date + ' ' + bookingDate.time;
	    } //hack
	    
	    var updateBookingStatus = function(sync) {
	    	var syncedBooking = bookingsActive.get(booking.get('perma_id'));
	    	
	    	if(syncedBooking != null)
	    		createPriceView(syncedBooking);
	    	
	    	// if(!sync && (syncedBooking == null || syncedBooking.isDispatched())) {
	    		// statusText = statusRow.args.text == statusWaitText ? 
					// (oldStatusText ? oldStatusText : 'Assigning Driver, Please Wait') :
	    			// statusWaitText;
			// }
			
			if (statusRow.args.text != statusText) {
				statusRow.animate({backgroundColor:highlightBGColor,duration:1000,autoreverse:true});
				statusRow.putText({
			    	label: 'Status',
			    	text: statusText,
			    });
			}
	    };
	    
	    if(!booking.isFinished() && !booking.wasCancelled()) {
		    setInterval(function(){ updateBookingStatus(false); }, 5000);
	    }
	    
	    // listen for data updates
		proxy.listenTo(bookingsActive,'sync:active',function(){
			var syncedBooking;
			if(syncedBooking = bookingsActive.get(booking.get('perma_id'))){
				
				if(syncedBooking.get('status_text'))
				{
					statusText = syncedBooking.get('status_text');
					oldStatusText = statusText;
				}
				
				updateBookingStatus(true);

				var driver = syncedBooking.get('driver');
				var showDriverRow = (driver.id && !(syncedBooking.isDispatched() || syncedBooking.isScheduled()));
				// if(!driverRow.height && driver.name){
				if(showDriverRow) {
					driverRow.putText({
						label:'Driver',
				    	title: driver.name+'/'+driver.vehicle.make+' '+driver.vehicle.model,
				    	subTitle:'Vehicle Reg: ' + driver.vehicle.reg,
					});
					driverRow.height = 54;
					if(!cancellProcess)
						openMap(3);
				}
				else {
					driverRow.height = 0;
				}
				
				var paymentType = syncedBooking.get('payment_type');
				if(paymentType == syncedBooking.PAYMENT_MODES.ACCOUNT) {
					var account_id = syncedBooking.get('account_id');
					
					var clientaccounts = ICABBI.Controller.getUser().clientaccounts;
					clientaccounts.each(function(clientaccount){
						Ti.API.info(JSON.stringify(clientaccount.toJSON()));
						if(clientaccount.get('id') == account_id)
						{
							paymentType = clientaccount.get('name') || 'Account';
						}
					});
					 
				}
				
				if(scheduledRow)
				{
					var bookingDate = syncedBooking.formatDateLocaleDisplay();
					scheduledRow.putText({
						label:'When',
			    		title: bookingDate.date,
			    		subTitle:bookingDate.time,
					});
				}
				
				var arrFrom = syncedBooking.get('address');		
			   	fromRow.putText({
					label:'From',
			    	title:arrFrom.formatted,
			    	subTitle:arrFrom.area||'',
				});
		
				var arrTo = syncedBooking.get('destination');
				
				toRow.putText({
					label:'To',
			    	title: arrTo.formatted,
			    	subTitle:arrTo.area||'',
				});
			    
			    var vias = booking.get('vias');
				if(vias && vias.length) {
					for(var idx in vias) {
						if(idx < viaRows.length) {
							viaRows[idx].putText({
								label : 'Via ' + (parseInt(idx) + 1),
						    	title : vias[idx].formatted,
						    	subTitle : vias[idx].area || '',
							});
						}
						else
						{
							var viaRow = createContentRow({
						    	label : 'Via ' + (parseInt(idx) + 1),
						    	title : vias[idx].formatted,
						    	subTitle : vias[idx].area || '',
						    	wrap: false,
						    });
						    contentView.add(viaRow);
						    viaRows.push(viaRow);
						}
					}

					// remove vias 
					if(vias.length < viaRows.length) {
						for(var i = vias.length; i < viaRows.length; i++) {
							contentView.remove(viaRows[i]);
						}
						viaRows = viaRows.splice(vias.length, viaRows.length - vias.length);
					}
				}
				else if(viaRows.length > 0){
					viaRows.forEach(function(viaRow){contentView.remove(viaRow);});
					viaRows =[];
				}

			    typeRow.putText({
					label:'Type',
			    	title: paymentType,
			    	subTitle:'',
				});
			}
		});
		
		ICABBI.Controller.on('booking:complete',bookingComplete);

		var controlBar = Titanium.UI.createView({
	        height:65,
	        width:Ti.UI.SIZE,
	        layout:'horizontal',
	        top: 9,
	    });

		mainview.createFooter = function(parent) {
			Ti.API.info('createFooter======================');
			var footerBg = Ti.UI.createView({
				bottom:0,
				left:0,
				right:0,
				height: footerHeight,
				backgroundColor: '#000000',
				opacity: 0.10,
			});
	
			parent.add(footerBg);
	
			var footer = Ti.UI.createView({
				bottom:0,
				left:0,
				right:0,
				height: footerHeight,
			});
	
			var footerBorderGray = Ti.UI.createView({
				top: 0,
				left: 0,
				right:0,
				height:1,
				backgroundColor:Theme.global.listBottomLineColor,
			});
			footer.add(footerBorderGray);
			
			var footerBorderWhite = Ti.UI.createView({
				top: 1,
				left: 0,
				right:0,
				height:1,
				backgroundColor:Theme.global.listTopLineColor,
			});
			footer.add(footerBorderWhite);
	
		    footer.add(controlBar);
		    
			parent.add(footer);
		};
	
	    var buttons = [];
	    
	    if (driver.name && !(booking.isDispatched() || booking.isScheduled()) && !booking.wasCancelled() && !booking.isFinished()) {
			var btnTrack = new ActionButton({title:'TRACK',icon:'/images/buttons/icon_track.png'});
			btnTrack.addEventListener('click',function(){
				openMap(3);
			});
			buttons.push(btnTrack);
	    }
	    
	    if (booking.isFinished() && !booking.wasCancelled()) {
			var btnMap = new ActionButton({title:'MAP',icon:'/images/buttons/icon_map.png'});
			btnMap.addEventListener('click',function(){
				//ICABBI.SlidingMenu.fireEvent('openview',{index:0});
				booking.syncStatus({
					locations:1,
					success: function(){
						var ChildWindow = require('/ui/common/ChildWindow'),
						BookingMapView = require('/ui/common/BookingMapView'),
						params = {
							content	: BookingMapView,
							header:{
									visible:true,controls:{cancel:{visible:true},submit:{visible:false},close:{visible:true}},
									//font:Theme.bookingMapView.headerFont
								},
							data:{
								Mode:2,
								booking:booking
							},
						};
				   		var modal = new ChildWindow(params);
				   		ICABBI.navController.open(modal);
					}
				});
				//ICABBI.navController.close(options.window);
			});
			buttons.push(btnMap);
	    }
	    
	    //if (!driver || !driver.name) 
	    {
			var btnPhone = new ActionButton({title:'PHONE',icon:'/images/buttons/icon_phone.png'});
			btnPhone.addEventListener('click',function(){
				Ti.Platform.openURL('tel:'+ICABBI.Config.phone.replace(/[^\d^+]/g, ''));
			});
			buttons.push(btnPhone);
	    }
		
		btnBook = new ActionButton({title:'NEW',icon:'/images/buttons/icon_add.png'});
		btnBook.addEventListener('click',function(){
				ICABBI.Controller.trigger('mapview:reset');
				ICABBI.SlidingMenu.fireEvent('openview',{index:ICABBI.Config.show_map?0:1});
				
				ICABBI.navController.close(options.window);
		});
		buttons.push(btnBook);

/*		
		if (!booking.get('favourite')) {
			btnFav = new ActionButton({title:'FAVOURITE',icon:'/images/buttons/icon_fav.png'});
			btnFav.addEventListener('click',function(){
				$$.ActivityIndicator.start('Adding to Favourites');
				ICABBI.Controller.updateBooking(booking,{
					success:function(){
						controlBar.remove(btnFav);
						booking.manageFavourite();
						$$.ActivityIndicator.stop();
						Ti.UI.createAlertDialog({
							title:'Saved',
							message:'This booking has been added to your favourites.',
							buttonNames:['OK'],
						}).show();
					},
					error:function(){
						$$.ActivityIndicator.stop();
						Ti.UI.createAlertDialog({
							title:'Error',
							message:'This booking could not be added to your favourites.',
							buttonNames:['OK'],
						}).show();
						booking.manageFavourite();
					}
				})
			});
			buttons.push(btnFav);
		}
*/
		
		if (!booking.isFinished()) 
		{
			btnCancel = new ActionButton({title:'CANCEL',icon:'/images/buttons/icon_cancel.png'});
			btnCancel.addEventListener('click',function(){
				if(!ICABBI.Config.suppress_cancel_confirmation)
				{
					ICABBI.navController.showYesNo({
					    title:'Booking Cancel!',
					    message:'Are you sure you want to cancel this booking?',
					    click : function(e){
					    	if(e.index == 1)
					    		cancelBooking();
					    }
					});
				}
				else
				{
					cancelBooking();
				}
			});
			buttons.push(btnCancel);
		}
	    
	    var buttonCount = 0;
	    for (var i in buttons) {
	    	if (buttonCount > 0) {
	    		buttons[i].left = 16;
	    	}
			controlBar.add(buttons[i]);
	    	buttonCount++;
	    }

		options.parent.window.addEventListener('open',function(){
			mds.setObject('booking_details',{perma_id:booking.get('perma_id')});
		});
		options.parent.window.addEventListener('close',function(){
			mds.setObject('booking_details',{perma_id:0});
			proxy.stopListening();
			ICABBI.Controller.off('booking:complete',bookingComplete);
		});
		
		return mainview;
	}; 
	var cancelBooking = function(){
		cancellProcess = true;
		ICABBI.Controller.cancelBooking(booking,{
			success: function(data){
				var content = [];
				if(data.error) {
					content.push(data.message || 'Error canceling booking!');
				}
				else {
					bookingsActive.addCancelledBooking(booking.get('perma_id'));
					
					var msg = 'Your booking from ' + booking.get('address').formatted;
					
					if(booking.get('destination').formatted) {
						msg += ' to ' + booking.get('destination').formatted;
					}
					if(booking.get('now')) {
						msg += ' scheduled now';
					}
					else {
						var bookingDate = booking.formatDateLocaleDisplay();
						msg += ' scheduled on ' + bookingDate.date;
					}							
					msg += ', has been cancelled';
					
					content.push(msg);
				}
				
				
				var ChildWindow = require('/ui/common/ChildWindow'),
					PromptView = require('/ui/common/PromptView'),
					params = {
						content	: PromptView,
						header	: {visible:true,controls:{close:{visible:true}}},
						title:'Booking Cancelled',
						data:{
							message:content,
							modal:true,
						},
					};
			   var modal = new ChildWindow(params);
				modal.addEventListener('open',function(e){
					ICABBI.Controller.trigger('mapview:reset');
					ICABBI.SlidingMenu.fireEvent('openview',{index:ICABBI.Config.show_map?0:1});
					// refresh bookings
					ICABBI.Controller.getUser().syncBookings(true);
					// ICABBI.Controller.getUser().syncBookingsActive(true);
					// ICABBI.Controller.getUser().syncBookingsUpcoming(true);
					//show prompt
					if(!data.error)
						ICABBI.navController.close(options.parent.window,{animated:false});
				});
				modal.addEventListener('close',function(e){
					cancellProcess = false;
				});
			   ICABBI.navController.open(modal);
			}
		});
	};
	var bookingComplete = function(data){
		if((booking.get('perma_id') && booking.get('perma_id') == data.perma_id) || 
			booking.get('id') == data.id){
			setTimeout(function(e){
				if(mapModal){
					ICABBI.navController.close(mapModal, {animated:false});
				}
				ICABBI.navController.close(options.parent.window,{animated:false});
			},500);
		};
	};	
	var self = createContent();
	self.onClose = function(){
				
	};
	return self;
};

module.exports = BookingDetailView;
