var	ICABBI = require('/core/icabbi'),
	ChildWindow = require('/ui/common/ChildWindow');

var tools = require('/lib/tools'),
	_ = require('/lib/lodash')._ ,
	$$ = require('/core/ThemeManager'),
	Booking = require('/model/Booking'),
	Address = require('/model/Address'),
	VehicleType = require('/model/VehicleType');

var Theme = require('/ui/Theme');
var ActionButton = require('/ui/common/ActionButton');
var FormCheckboxRow = require('/ui/common/FormCheckboxRow');

//BookingReviewView Component Constructor
var BookingReviewView = function(options) {
	options = options || {};
	var bookingData = options.data && options.data.bookingData  ? options.data.bookingData : (new Booking.Model());
	
	var booking = bookingData.booking.clone().set(options.data && options.data.booking  ? options.data.booking : (new Booking.Model()));
	
	var bookingDataBooking = bookingData.booking;
	
	var estimate = options.data && options.data.estimate  ? options.data.estimate : null;
	var price = options.data && options.data.price  ? options.data.price : null;
	var nameRow, phoneRow;

	var footer,
		footerBg,
		view,
		fillerControlBar;
	var headerHeight = 48,
		footerHeight = 81;
				
	var createRow = function(args){
		
		var self = Ti.UI.createView({
			left:0,
			right:0,
			height:54,
			putText:function(args){
				label.text = args.label;
				title.text = args.title;
				title.top = args.subTitle.length ? 4:undefined;
				subTitle.text = args.subTitle;
			},
			value:args.value,
		});
		
		var label = Ti.UI.createLabel({
			left:0,
			height:21,
			width:45,
			color:Theme.form.labelColor,
			font:Theme.form.labelFont,
			text:args.label,
			textAlign:'right',
			touchEnabled:false,
		});
		self.add(label);
		
		if(args.inline) {
			var input = Ti.UI.createTextField({
				left:args.label.length?70:22,
				right:12,
				backgroundColor:'transparent',
				borderColor:'transparent',
				color:Theme.global.fontColor,
				font:Theme.menu.rowFont,
				paddingLeft:10,
				paddingRight:10,
				hintText:args.hint,
				value:self.value,
				// keyboard stuff
				clearButtonMode:args.clearButtonMode,
				clearOnEdit:args.clearOnEdit,
				keyboardType:args.keyboardType,
				keyboardToolbar:args.keyboardToolbar,
				passwordMask:args.passwordMask,
				returnKeyType:args.returnKeyType,
				autocapitalization:args.autocapitalization,
				autocorrect:args.autocorrect,
			});
			input.addEventListener('change',function(e){
				self.value = e.value;
			});
			self.add(input);
			var pencilIcon = Ti.UI.createImageView({
			    height:15,
			    width:15,
			    left: 60,
			    image:'/images/map/btn_pencil_noborder.png',
			    touchEnabled:false,
			});
			self.add(pencilIcon);
			self.done = function(){
				input.blur();
			};
		} else {
			var title = Ti.UI.createLabel({
				left:args.label.length?60:12,
				right:25,
				top:args.subTitle && args.subTitle.length ? 4:undefined,
				height:28,
				color:args.note && args.note ? Theme.form.smallColor : Theme.global.fontColor,
				font:args.note && args.note ? Theme.menu.rowFontSmall : Theme.menu.rowFont,
				minimumFontSize:args.note && args.note ? Theme.menu.rowFontSmall.fontSize : Theme.menu.rowFont.fontSize,
				wordWrap:false,
				ellipsize:true,
				width: Ti.UI.FILL,
				text:args.title,
				touchEnabled:false,

			});
			self.add(title);
			
			var subTitle = Ti.UI.createLabel({
				left:args.label.length?60:12,
				right:25,
				bottom:4,
				height:24,
				color:Theme.form.smallColor,
				font:Theme.form.smallFont,
				text:args.subTitle,
				touchEnabled:false,
				wordWrap:false,
				ellipsize:true,
				width: Ti.UI.FILL,
			});
			self.add(subTitle);
		}
		
		if(!args.readonly && !args.inline) {
			var detail = Ti.UI.createImageView({
				right:12,
				width:9,
				height:15,
				image:'/images/menu/arrow.png',
				touchEnabled:false,
			});
			self.add(detail);
		}
			
		var rowLine = Ti.UI.createView({
			left:args.label.length?60:0,
			right:11,
			bottom:0,
			height:1,
			backgroundColor:Theme.global.listBottomLineColor,
			visible:!args.lastItem,
			touchEnabled:false,
		});
		self.add(rowLine);
		
		var rowLineTop = Ti.UI.createView({
			left:args.label.length?60:0,
			right:11,
			top:0,
			height:1,
			backgroundColor:Theme.global.listTopLineColor,
			visible:!args.firstItem,
			touchEnabled:false,
		});
		self.add(rowLineTop);
		
		if(!args.readonly && !args.inline)
			Theme.animateButton(self);
		
		return self;
	};

	var createForm = function(){
		//create object instance, a parasitic subclass of Observable

		mainview = Titanium.UI.createView({
			top:0,
	        left:0,
	        right:0,
	        bottom:0,
		    // backgroundGradient: {
		        // type: 'linear',
		        // startPoint: { x: '50%', y: '0%' },
		        // endPoint: { x: '50%', y: '100%' },
		        // colors: [ { color: 'white', offset: 0.0}, { color: '#f1f1f1', offset: 1.0 } ],
		    // },
		    backgroundColor:'#f1f1f1',
	    });

		scrollVView =  Ti.UI.createScrollView({
				scrollType:'vertical',
		        bottom:footerHeight,
		        top:0,
			});
			
	    mainview.add(scrollVView);

	    var view = Ti.UI.createView({
			top:0,
	        left:8,
	        right:0,
			layout:'vertical',
	    });
	    scrollVView.add(view);
	    	    
		
   		/*
   		 * Price
   		 */
   		if(!ICABBI.Config.hide_prices) {
   			
	   		var payment = booking.get('payment'),
	   			returnPayment = booking.get('return_payment'),
	   			returnPaymentTotal = returnPayment && returnPayment.total ? returnPayment.total : 0;
	   		
	   		var extra = '';
   			if(returnPayment && (returnPayment.return_price_discount + payment.return_price_discount) > 0)
   			{
   				extra = ', you save ' + ICABBI.Config.currencySign + String.formatDecimal(returnPayment.return_price_discount + payment.return_price_discount, 'en-US', '0.00');
   			}
	   			
	   		var priceMainView;
	   		
	   		if((payment && payment.price) || estimate) {
	   			var priceContainerView = Titanium.UI.createView({
	   				top:0,
			        left:7,
			        right:15,
					height:Ti.UI.SIZE,
					layout:'vertical',
			    });
			    
	   			priceMainView = Titanium.UI.createView({
			        width:Ti.UI.FILL,
					height:Ti.UI.SIZE,
					layout:'horizontal',
			    });
			    priceContainerView.add(priceMainView);
			    
			    var priceBottomLine = Ti.UI.createView({
					left:0,
					right:0,
					height:1,
					backgroundColor:Theme.global.listBottomLineColor,
					touchEnabled:false
				});
				priceContainerView.add(priceBottomLine);
				
			    view.add(priceContainerView);
	   		}
	 
	   		if(payment && payment.price) {
	
				var clientaccounts = ICABBI.Controller.getUser().clientaccounts;
	   			
	   			var hasDetails = payment.tip || payment.tolls || payment.extras || payment.service_charge || payment.discount || returnPaymentTotal || (payment.total != 0 && payment.total != payment.price);
	   			
	   			if(hasDetails) {
	   				var priceDetailsView = Titanium.UI.createView({
		   				width: '49%',
						layout:'vertical',
						height: Ti.UI.SIZE,
		   			});
					priceMainView.add(priceDetailsView);
					
					var priceVertLine = Ti.UI.createView({
						top: 10,
						bottom: 10,
						width:1,
						height: 1,
						backgroundColor:Theme.global.listBottomLineColor,
						touchEnabled:false
					});
					priceMainView.add(priceVertLine);
					priceDetailsView.addEventListener('postlayout', function(){
						priceVertLine.height = priceDetailsView.rect.height;
					});
					
					var addSmallPrice = function(priceText, priceValue) {
		   				var priceSmallRow = Titanium.UI.createView({
			   				left: 0,
			   				right: 0,
			   				height: Ti.UI.SIZE
			   			});
			   			priceDetailsView.add(priceSmallRow);
			   			
						var priceLabel = Ti.UI.createLabel({
							left: 0,
							width: 47,
							color:Theme.form.labelColor,
							font:Theme.form.labelFont,
							text:priceText,
							touchEnabled:false,
							textAlign:Ti.UI.TEXT_ALIGNMENT_LEFT,
						});
						priceSmallRow.add(priceLabel);
			   			
						var priceValLabel = Ti.UI.createLabel({
							left: 47,
							right: 5,
							color:Theme.childwindow.headerColor,
							font:Theme.form.labelFont,
							text:priceValue,
							touchEnabled:false,
							textAlign:Ti.UI.TEXT_ALIGNMENT_LEFT
						});
						priceSmallRow.add(priceValLabel);
					};
									
					if(payment.total != 0 && payment.total != payment.price)
						addSmallPrice('Fare:', ICABBI.Config.currencySign + payment.price.toFixed(2));
					
					if(payment.tip)
						addSmallPrice('Tip:', ICABBI.Config.currencySign + String.formatDecimal(payment.tip, 'en-US', '0.00'));
						
					if(payment.tolls)
						addSmallPrice('Tolls:', ICABBI.Config.currencySign + String.formatDecimal(payment.tolls, 'en-US', '0.00'));
	
					if(payment.extras)
						addSmallPrice('Extras:', ICABBI.Config.currencySign + String.formatDecimal(payment.extras, 'en-US', '0.00'));
					
					var account_id = booking.get('account_id');
					if(account_id) {
						var clientaccounts = ICABBI.Controller.getUser().clientaccounts,
							account_name = false;
							
						clientaccounts.each(function(clientaccount) {
							if(account_id == clientaccount.get('id')) {
								account_name = clientaccount.get('name');
							}
						});
						if(account_name)
							addSmallPrice('A/C:', account_name);
					}
					
					if(payment.service_charge)
						addSmallPrice('Fee:' , ICABBI.Config.currencySign + String.formatDecimal(payment.service_charge, 'en-US', '0.00') + 
						((payment.service_charge_type == 'PERCENTAGE' && payment.service_charge > payment.min_service_charge) ? '(' + payment.processing_fee + '%)' : ''));
						
					if(payment.discount)
						addSmallPrice('Disc:', payment.discount + '%');
	
					if(returnPaymentTotal)
						addSmallPrice('Return:', ICABBI.Config.currencySign + String.formatDecimal(returnPaymentTotal, 'en-US', '0.00'));
	   			}
	   			
	   			var priceTotalView = Titanium.UI.createView({
	   				width: hasDetails ? '50%' : '100%',
					layout:'vertical',
					height: Ti.UI.SIZE,
	   			});
	   			priceMainView.add(priceTotalView);
	   			
				var farePrice = Ti.UI.createLabel({
					right:0,
					left:0,
					color:Theme.childwindow.headerColor,
					font:Theme.bookingReview.totalPrice,
					text:ICABBI.Config.currencySign + (payment.total != payment.price || returnPaymentTotal ? String.formatDecimal(payment.total + returnPaymentTotal, 'en-US', '0.00') : String.formatDecimal(payment.price, 'en-US', '0.00')),
					touchEnabled:false,
					textAlign:Ti.UI.TEXT_ALIGNMENT_CENTER,
				});
				priceTotalView.add(farePrice);
	   			
				var farePriceLabel = Ti.UI.createLabel({
					right:0,
					left:0,
					top:-10,
					height:Ti.UI.SIZE,
					color:Theme.childwindow.headerColor,
					font:hasDetails ? Theme.bookingReview.totalPriceSmallLabel : Theme.bookingReview.totalPriceLabel,
					text: payment.fixed ? '(Fixed Fare' + extra + ')' : '(Estimate' + extra + ')',
					touchEnabled: false,
					textAlign:Ti.UI.TEXT_ALIGNMENT_CENTER,
				});
				priceTotalView.add(farePriceLabel);

	   		}
	   		else if(estimate) {
	
	   			var priceTotalView = Titanium.UI.createView({
	   				left: 0,
	   				right: 0,
					layout:'vertical',
					height: Ti.UI.SIZE,
	   			});
	   			priceMainView.add(priceTotalView);
	   			
				var estimatedPrice = Ti.UI.createLabel({
					right:0,
					left:0,
					color:Theme.childwindow.headerColor,
					font:Theme.bookingReview.totalPrice,
					text:ICABBI.Config.currencySign + estimate.value,
					touchEnabled:false,
					textAlign:Ti.UI.TEXT_ALIGNMENT_CENTER,
				});
				priceTotalView.add(estimatedPrice);
	   			
				var estimatedPriceLabel = Ti.UI.createLabel({
					right:0,
					left:0,
					top:-10,
					height:Ti.UI.SIZE,
					color:Theme.childwindow.headerColor,
					font:Theme.bookingReview.totalPriceLabel,
					text:'(Estimate' + extra + ')',
					touchEnabled:false,
					textAlign:Ti.UI.TEXT_ALIGNMENT_CENTER,
				});
				priceTotalView.add(estimatedPriceLabel);
			}
		}

   		/*
   		 * DateTime
   		 */
   		var timeRow = createRow({
   			label:'When',
   			title:bookingData.objDateTime.now ? 'Now' : bookingData.objDateTime.formatted.date,
   			subTitle:bookingData.objDateTime.now ? '': bookingData.objDateTime.formatted.time,
   			readonly:true,
   			firstItem:true,
   		});
   		timeRow.top = 10;
		view.add(timeRow);

   		/*
   		 * From
   		 */
   		var fromRow = createRow({
   			label:'From',
			title:bookingData.addressFrom.formatDisplay(),
   			subTitle:bookingData.addressFrom.get('area'),
   			readonly:true,
   		});
   		view.add(fromRow);
   		
   		/*
   		 * To
   		 */
   		if(!bookingData.addressTo.isEmpty())
   		{
	   		var toRow = createRow({
	   			label:'To',
	   			title:bookingData.addressTo.formatDisplay(),
	   			subTitle:bookingData.addressTo.get('area'),
	   			readonly:true,
   			});
   			view.add(toRow);
   		}
   		/*
   		 * Vias
   		 */
		var vias = bookingDataBooking.get('vias');
		if(vias) {
			for(var idx in vias) {
				var address = new Address.Model();
				address.set(vias[idx]);
				
		   		var viaRow = createRow({
		   			label:'Via ' + (parseInt(idx) + 1),
		   			title: address.isEmpty() ? '' : address.formatDisplay(),
		   			subTitle: address.isEmpty() ? 'optional' : address.get('area'),
		   			readonly:true,
		   		});
		   		view.add(viaRow);
			}
		}
		
   		/*
   		 * Return DateTime
   		 */
   		if(bookingData.returnJourneyChecked) {
	   		var returnTimeRow = createRow({
	   			label:'Return',
	   			title:bookingData.objReturnDateTime.now ? 'Return Date & Time' : bookingData.objReturnDateTime.formatted.date,
	   			subTitle:bookingData.objReturnDateTime.now ? '': bookingData.objReturnDateTime.formatted.time,
	   			readonly:true,
	   		});
			view.add(returnTimeRow);
		}

   		/*
   		 * VehicleType
   		 */
   		var	vehicleType = ICABBI.Controller.getMeta().vehicleTypes.get(bookingDataBooking.get('vehicle_type'));

		if(vehicleType) {
	   		var typeRow = createRow({
	   			label:'Type',
	   			title:vehicleType ? vehicleType.get('title') : '',
	   			subTitle:'',
	   			readonly:true,
	   		});
	   		view.add(typeRow);
	   	}
		
   		/*
   		 * CustomerName
   		 */
   		nameRow = createRow({
   			label:'Name',
   			title:booking.get('name'),
   			subTitle:'',
   			readonly:true,
	    });
   		view.add(nameRow);

   		/*
   		 * TelephoneNumber
   		 */
   		phoneRow = createRow({
   			label:'Phone',
   			title:booking.get('phone'),
   			subTitle:'',
   			readonly:true,
	    });
   		view.add(phoneRow);
		
   		/*
   		 * DriverNote
   		 */
   		if(booking.get('instructions')){
   		var noteRow = createRow({
   			label:'Note',
   			title:booking.get('instructions'),
   			subTitle:'',
   			lastItem:true,
   			note: true,
   			readonly:true,
   		});
   		view.add(noteRow);
		}
		
		var controlBar = Titanium.UI.createView({
	        height:65,
	        width:Ti.UI.SIZE,
	        layout:'horizontal',
	        top: 9,
	    });

	    
		mainview.createFooter = function(parent) {
			footerBg = Ti.UI.createView({
				bottom:0,
				left:0,
				right:0,
				height: footerHeight,
				backgroundColor: '#000000',
				opacity: 0.10,
			});
	
			parent.add(footerBg);
	
			footer = Ti.UI.createView({
				bottom:0,
				left:0,
				right:0,
				height: footerHeight,
			});
	
			var footerBorderGray = Ti.UI.createView({
				top: 0,
				left: 0,
				right:0,
				height:1,
				backgroundColor:Theme.global.listBottomLineColor,
			});
			footer.add(footerBorderGray);
			
			var footerBorderWhite = Ti.UI.createView({
				top: 1,
				left: 0,
				right:0,
				height:1,
				backgroundColor:Theme.global.listTopLineColor,
			});
			footer.add(footerBorderWhite);
	
		    footer.add(controlBar);
		    
			parent.add(footer);
		};
	
		var processPayment = function(e, order_id) {
			var payment = booking.get('payment'),
				returnPayment = booking.get('return_payment');
				
			if(e.success) {

				payment.card_id = bookingData.booking.get('creditcard_id');
				payment.status = 'PROCESSED';
				payment.order_id = order_id;
				bookingData.booking.set('payment', payment);
				
				if(returnPayment) {
					returnPayment.card_id = bookingData.booking.get('creditcard_id');
					returnPayment.status = 'PROCESSED';
					returnPayment.order_id = order_id;
				}

				ICABBI.navController.showAlert({
				    title:'Payment Successful',
				    message:'- Your payment was successful.',
				    close: false,
				    click: function(e){
				    	$$.ActivityIndicator.start('Confirm booking');
						_registerBooking(order_id);
				    }
				});
			}
			else {
				ICABBI.navController.showError({
				    title: 'Error!',
				    message: e.message,
				    close: true
				});
			}
		};
		
		var confirmBooking = function() {
			var payment = booking.get('payment'),
				returnPayment = booking.get('return_payment'),
				returnPaymentTotal = returnPayment && returnPayment.total ? returnPayment.total : 0;
			
			if(ICABBI.Config.card_prepaid_payment 
					&& bookingData.booking.get('payment_type') == booking.PAYMENT_MODES.CREDITCARD 
				 // && bookingData.booking.get('account_id') 
					&& payment && payment.total) {
						
				$$.ActivityIndicator.start('Processing payment');
				
				var order_id = ICABBI.Controller.getUser().get('id') + '_' + new Date().getTime();
				
				var payment_data = {
					cvv: bookingData.booking.get('card_cvv') || '',
					card_id: bookingData.booking.get('creditcard_id'),
					amount: payment.total + returnPaymentTotal,
					order_id: order_id,
				};
				
				ICABBI.Controller.processPayment(payment_data, 
					{
						success: function(e) {
							$$.ActivityIndicator.stop();

							// 3D Secure
							if(e['3DUrl'])
							{
								var paymentOptions = {
									url: e['3DUrl'],
									PaReq: e['PaReq'],
									TermUrl: ICABBI.Config.IcabbiApi.ENDPOINT + 'payment/continue?app_key=' + ICABBI.Config.IcabbiApi.APP_KEY + '&id=' + bookingData.booking.get('creditcard_id'),
									provider: ICABBI.Config.card_provider,
									MD: e['MD'],
								};
								
								winOptions = {
									content:require('/ui/common/Card3DPaymentView'),
									title:'3D Secure Payment',
									data:{
										payment: paymentOptions
									},
								};
								var modal = new ChildWindow(winOptions);
								ICABBI.navController.open(modal);
								modal.addEventListener('close',function(e){
									if(modal.__retval && _.isObject( modal.__retval))
										processPayment(modal.__retval, order_id);
								});
							}
							else
								processPayment(e, order_id);
						},
						
						error: function(e) {
							$$.ActivityIndicator.stop();
							
							ICABBI.navController.showError({
							    title: 'Error!',
							    message: e.message,
							    close: true
							});
						}
					});
			}
			else
			{
				$$.ActivityIndicator.start('Confirm booking');
				
				if(bookingData.booking.get('payment_type') == booking.PAYMENT_MODES.CREDITCARD){

					payment.card_id = bookingData.booking.get('creditcard_id');
					payment.status = 'NEW';
					bookingData.booking.set('payment', payment);
					
					if(returnPayment) {
						returnPayment.card_id = bookingData.booking.get('creditcard_id');
						returnPayment.status = 'NEW';
						bookingData.booking.set('return_payment', returnPayment);
					}
				}
				
				_registerBooking();
			}
		};
		
		var confirmAction = function(e){
			if (ICABBI.navController.opening(e.source)) {
				return;
			}

			if(bookingData.booking.get('payment_type') == booking.PAYMENT_MODES.CREDITCARD)
			{
				if(bookingData.booking.get('creditcard_id')) // card was selected ask for cvv
				{
					var creditcards = ICABBI.Controller.getUser().creditcards,
							requiresCVV = ICABBI.Config.card_provider == 'sagepay';
					if(requiresCVV) {
						var modalOptions = {
							content:require('/ui/common/CreditCardPaymentView'),
							title:'Confirm Card Payment',
							data	: {booking:bookingData.booking, creditcards:creditcards, requiresCVV:requiresCVV, creditcard_id : bookingData.booking.get('creditcard_id')},
						};
						var modal = new ChildWindow(modalOptions);
						ICABBI.navController.open(modal,undefined,self);
						modal.addEventListener('close',function(e){
							if(modal.__retval)
								confirmBooking();						
						});
					}
					else {
						confirmBooking();
					}
				}
				else // new card
				{
					var modalOptions = {
						content:require('/ui/common/CreditCardRegistrationView'),
						title:'Confirm Card Payment',
						data:{booking:bookingData.booking, accountcard_id : bookingData.booking.get('accountcard_id')},
					};
					var modal = new ChildWindow(modalOptions);
					ICABBI.navController.open(modal,undefined,self);
					modal.addEventListener('close',function(e){
						if(modal.__retval)
							confirmBooking();
					});
				}
			}
			else
			{
				confirmBooking();
			}
			
			e.source.opening = false;
		};
	
		var cancelAction = function(){
			ICABBI.navController.showYesNo({
			    title:'Booking Cancel!',
			    message:'Are you sure you want to cancel this booking?',
			    click : function(e){
			    	if(e.index == 1) {
			    		ICABBI.SlidingMenu.fireEvent('openview',{index:ICABBI.Config.show_map?0:1});
						ICABBI.navController.close(options.window);
						// reset booking
						ICABBI.Controller.trigger('mapview:reset');
					}
			    }
			});	
		};
		
		var editAction = function(e){
			ICABBI.navController.close(options.parent.window,{animated:false});
		};
		
		var btnEdit = new ActionButton({title:'Edit',icon:'/images/buttons/icon_edit.png'});
		controlBar.add(btnEdit);
		btnEdit.addEventListener('click',editAction);

		var btnCancel = new ActionButton({title:'Cancel',icon:'/images/buttons/icon_cancel.png'});
		btnCancel.left = 16;
		controlBar.add(btnCancel);
		btnCancel.addEventListener('click',cancelAction);

		var btnConfirm = new ActionButton({title:'Confirm', icon:'/images/buttons/icon_save.png', blinkIcon:'/images/buttons/icon_save_inv.png'});
		btnConfirm.left = 16;
		controlBar.add(btnConfirm);
		btnConfirm.addEventListener('click',confirmAction);

			     
		return mainview;
	};


	var _registerBooking = function(order_id){
		var registerBookingSuccess = function(resBooking, isReturn){
			
			if(!isReturn) {
				booking = resBooking;
			}
			
			var BookingDetailView = require('/ui/common/BookingDetailView');
			var config = {
				header:{visible:true , controls : {submit : {visible: false}, close: {visible:true} }},
				content: BookingDetailView,
				title: 'Booking Confirmed',
				data: { booking: booking },
				notScrollable: true,
			};
			var modal = new ChildWindow(config);
			
			modal.addEventListener('open',function(){
				ICABBI.navController.close(options.parent.window,{animated:false});
				// reset booking from map and booking edit
				ICABBI.Controller.trigger('mapview:open:edit', {find:false});
				ICABBI.Controller.trigger('mapview:reset');
			});
			ICABBI.navController.open(modal);
		};
		
		var registerBookingError = function(resp) {
			$$.ActivityIndicator.stop();
			var msg = resp && resp.message ? resp.message : 'Server Unavailable. Please try again later.';
			if(resp)
			for (var i in resp.body) {
				msg += '\n' + i + ': ' + resp.body[i];
			}
			ICABBI.navController.showError({
			    title:'Error!',
			    message:msg,
			    close:true
			});
		};
		
		$$.ActivityIndicator.start('Getting Booking Confirmation');
		bookingData.booking.set({
			order_id:order_id,
			payment:booking.get('payment'),
			return_payment:booking.get('return_payment')
		});
		ICABBI.Controller.registerBooking(bookingData.booking, {
			success: function(resBooking) { registerBookingSuccess(resBooking, false); },
			error: registerBookingError
		});		
	};
	
	var self = createForm();
	return self;
};

module.exports = BookingReviewView;
