var	ICABBI = require('/core/icabbi'),
	ChildWindow = require('/ui/common/ChildWindow');

var Theme = require('/ui/Theme');

var SlidingView = require('/ui/common/SlidingView'),
	_ = require('/lib/lodash')._ ,
	$$ = require('/core/ThemeManager'),
	MapControl = require('/lib/MapControl'),
	Booking = require('/model/Booking'),
	Address = require('/model/Address'),
	moment = require('/lib/moment'),
	Events = require('/lib/Events');
	
ICABBI.getDateTime = function(now,objDateTime){
	var form = {};
	objDateTime=objDateTime||{};

	form.now = now||false;
	if(form.now){
		form.raw = objDateTime = {date:new Date,time:new Date};
	}else{
		form.raw = objDateTime = {date:objDateTime.date || new Date,time:objDateTime.time || new Date};
	}

	form.isoFormat = function() {
		var getTimezoneOffset = function(date){
				date===void 0 && (date=new Date);
				var offset_minutes = date.getTimezoneOffset(),
					// offset is positive if the local timezone is behind UTC and negative if it is ahead
					direction=offset_minutes<0?'+':'-', 
					hours = Math.abs(parseInt(offset_minutes/60))+'',
					minutes = Math.abs(parseInt(offset_minutes%60))+'';
				return direction+(hours[1]?hours:'0'+hours)+':'+(minutes[1]?minutes:'0'+minutes);
			};
			
			var formattedDate = (function(){
					var yyyy = this.getFullYear().toString(),
			  			mm = (this.getMonth()+1).toString(), // getMonth() is zero-based
			   			dd  = this.getDate().toString();
			   			
			   		return yyyy +'-'+ (mm[1]?mm:"0"+mm[0]) +'-'+ (dd[1]?dd:"0"+dd[0]); // padding
				}).call(this.raw.date);
			var formatedTime = (function(){
				var H = this.getHours().toString(),
					i = this.getMinutes().toString(),
					s = this.getSeconds().toString();
		
		   		return (H[1]?H:"0"+H[0]) +':'+ (i[1]?i:"0"+i[0]) +':'+ (s[1]?s:"0"+s[0]);
			}).call(this.raw.time);

			return formattedDate+'T'+formatedTime+getTimezoneOffset();
	};
	form.formatted = {date:'',time:''};
	form.formatted.date = (function(){
		return moment(this).format(ICABBI.Config.formatDate);
	}).call(objDateTime.date);
	
	form.formatted.time = (function(){
		return moment(this).format(ICABBI.Config.formatTime);
	}).call(objDateTime.time);

	return form;
};

var BookingView = function() {
	var resetToAddress = function() {
		
		bookingData.addressTo.reset();
		
		if(ICABBI.Config.behaviours && ICABBI.Config.behaviours.fields && ICABBI.Config.behaviours.fields.destination &&
					ICABBI.Config.behaviours.fields.destination['default'])
		{
			var destination = ICABBI.Controller.getMeta().destinations.get(ICABBI.Config.behaviours.fields.destination['default'].id);
			if(destination)
				bookingData.addressTo.set(destination.toJSON());
			else
				bookingData.addressTo.set({});
		}
	};

	var bookingData = { 
			pois : null,
			addressFrom : new Address.Model(),
			addressTo : new Address.Model(),
			booking : new Booking.Model(),
			bookingReview : {
				booking : null,
				estimate: null,
				price : null
			},
			objDateTime : ICABBI.getDateTime(true),
			objReturnDateTime : ICABBI.getDateTime(true),
			reset : function(){
				this.objDateTime = ICABBI.getDateTime(true);
				this.objReturnDateTime = ICABBI.getDateTime(true);
				this.booking = new Booking.Model();
				this.bookingReview = {
					booking : null,
					estimate: null,
					price : null
				};
				MapControl.pauseRegionChangedListener(false);
				MapControl.getCurrentPosition();
			
				resetToAddress();
				
				this.returnJourneyChecked = false;
				this.returnJourneyNotified = false;
				
				this.trigger('reset');
			},
			
			returnJourneyChecked: false,
			returnJourneyNotified: false,
	};
	
	_.extend(bookingData,Events);
	var currentETA = 30;
	
	// set start address 
	var a = ICABBI.Controller.DEVICE_LOCATION_ADDRESS;
	if(a)
	{
		bookingData.addressFrom.set({id:a.id,src:a.src,area:a.area,formatted:a.formatted,lat:a.lat,lng:a.lng});
	}
	
	var header,map,footer,map_overlay,mapControl;
	
	var footerHeight = headerHeight = 112,
		btnSlideMenu;
	var footerInnerHeight = 88;
	
	var createHeaderBg = function(){
		var headerBg = Ti.UI.createView({
			top: 0,
			left: 0,
			right:0,
			zIndex:10,
			//height: headerHeight,
			height: 48,
			backgroundColor: Theme.map.overlayBGColor,
			opacity:0.80,
		});	
		return headerBg;	
	};	

	var createHeader = function(){
		var header = Ti.UI.createView({
			top: 0,
			left: 0,
			right:0,
			zIndex:11,
			height: 48,
		});

		var headerBorder = Ti.UI.createView({
			bottom: 0,
			left: 0,
			right:0,
			height:1,
			backgroundColor:Theme.global.altBGColor,
		});
		header.add(headerBorder);
		
		if(ICABBI.Config.show_map)
		{
			btnSlideMenu = Ti.UI.createView({
				left:6,
				height:36,
				width:36,
				animateType:Theme.ANTIMATE_TYPE_OPACITY,
			});
			
			var menuIcon = Ti.UI.createImageView({
				height:16,
				width:21,
				touchEnabled:false,
				image:'/images/map/btn_menu.png',
			});
			btnSlideMenu.add(menuIcon);
			Theme.animateButton(btnSlideMenu);
	
			btnSlideMenu.addEventListener('click',function(e){
				ICABBI.Controller.trigger('slidingmenu:toggle');
			});
			header.add(btnSlideMenu);
			
			var lblNotification = Ti.UI.createLabel({
				right:0,
				top:0,
				color:Theme.global.fontColor,
				backgroundColor:Theme.global.highlightLightColor,
				width:20,
				height:18,
			    textAlign: Ti.UI.TEXT_ALIGNMENT_CENTER,
				borderRadius:6,
				borderWidth:1,
				borderColor:Theme.global.highlightColor,
				font:Theme.global.smallFont,
				touchEnabled:false,
				visible:false			
			});
			ICABBI.Controller.on('change:active_bookings',function(e){
				lblNotification.text = e.count;
				lblNotification.visible = !!e.count;
			});
			btnSlideMenu.add(lblNotification);
		}
		else {
			btnBackMenu = Ti.UI.createView({
				left:6,
				height:48,
				width:51,
				animateType:Theme.ANTIMATE_TYPE_OPACITY,
			});
			
			var backIcon = Ti.UI.createImageView({
				height:30,
				width:50,
				left:0,
				top:9,
				touchEnabled:false,
				image:'/images/headers/icon_back.png',
			});
			btnBackMenu.add(backIcon);
			var backLabel = Ti.UI.createLabel({
				text: 'Back',
				bottom:17,
			    left: 15,
				color:Theme.childwindow.buttonTextColor,
				font:Theme.childwindow.headerButtonFont,
			});
			btnBackMenu.add(backLabel);
			Theme.animateButton(btnBackMenu);
			btnBackMenu.addEventListener('click',function(e){
				ICABBI.SlidingMenu.fireEvent('openview',{index:1});
				
			});

			header.add(btnBackMenu);
		}
		var statuxBoxContainer = Ti.UI.createView({
			left:60,
			right:35,
			height:36,
			
		});
		header.add(statuxBoxContainer);
		
		var statuxBox = Ti.UI.createView({
			height:36,
			width:Ti.UI.SIZE,
			layout:'horizontal',
			horizontalWrap:false,
			center: {
		  		x: '50%',
			},
		});
		statuxBoxContainer.add(statuxBox);
		if(!ICABBI.Config.app_disable_eta) 
		{
			var lblEstimate = Ti.UI.createLabel({
				width:Ti.UI.SIZE,
				height:36,
				color:Theme.map.smallColor,
				font:Theme.map.smallFont,
				textAlign:'right',
				text:'Estimated\nArrival',
			});
			statuxBox.add(lblEstimate);
	
			var timeBox = Ti.UI.createView({
				left:4,
				height:36,
				width:Ti.UI.SIZE,
			});
			statuxBox.add(timeBox);
			
			var doTime = function() {
				if (timeBox.children && timeBox.children.length) {
					timeBox.removeAllChildren();
				}
				
				var etaArray = currentETA.toString().split('');
				var thisLeft = 0;
				
				for (var i in etaArray) {
					var box = Ti.UI.createView({
						height:34,
						width:31,
						left:thisLeft,
						backgroundImage:'/images/map/background_minute.png'
					});
					
					var digit = Ti.UI.createLabel({
						width:31,
						height:45,
						color:Theme.map.timeColor,
						font:Theme.map.timeFont,
						text:etaArray[i],
						textAlign:'center',
					});
					box.add(digit);
					
					timeBox.add(box);
					
					thisLeft += 32;
				}
			};
			
			doTime();
			
			var lblMins = Ti.UI.createLabel({
			    left:4,
				height:36,
			    width:Ti.UI.SIZE,
			    color:Theme.global.fontColor,
			    font:Theme.map.infoFont,
			    text:'Minutes',
			});
			statuxBox.add(lblMins);
			
			ICABBI.Controller.on('change:eta',function(e){
				if (currentETA !== e.eta) {
					currentETA=e.eta;
					doTime();
				}
			});
		}
		var btnCurrentLocation = Ti.UI.createView({
			right:5,
			height:36,
			width:25,
			animateType:Theme.ANTIMATE_TYPE_OPACITY,
		});
		
		var locationIcon = Ti.UI.createImageView({
			height:25,
			width:25,
			touchEnabled:false,
			image:'/images/map/btn_location.png',
		});
		btnCurrentLocation.add(locationIcon);
		Theme.animateButton(btnCurrentLocation);

		btnCurrentLocation.addEventListener('click',function(e){
			$$.ActivityIndicator.start('Locating Address...please wait');
			MapControl.getCurrentPosition();
			setTimeout(function(){
				$$.ActivityIndicator.stop(); 
				if(!bookingData.addressFrom.get('src')){
					ICABBI.navController.showAlert({
					    title:'No address found!',
					    message:'- Sorry, address not found',
					    close:true
					});
			}
			}, 1000);	
		});
		header.add(btnCurrentLocation);
	
		return header;
	};
	
	var createFooterBg = function(){
		var footerBg = Ti.UI.createView({
			bottom: 0,
			left: 0,
			right:0,
			zIndex:10,
			height:footerHeight,
			backgroundColor:'#ffffff',
			opacity:0.90,
		});
		var footerBorder = Ti.UI.createView({
			top: 0,
			left: 0,
			right:0,
			height:1,
			backgroundColor:Theme.global.altBGColor,
		});
		footerBg.add(footerBorder);

		return footerBg;
	};
	
	var createFooter = function(){
		var footer = Ti.UI.createView({
			bottom: 12,
			left: 0,
			right:0,
			zIndex:11,
			height: footerInnerHeight,
		});
		
		var addressView = Ti.UI.createView({
			left: 0,
			right: 90,
		    layout:'vertical',
		});
		var addressFromView = Ti.UI.createView({
			left: 0,
			right: 0,
		    height: footerInnerHeight/2,
		    horizontalWrap: false,
		});
		var footerBorderView = Ti.UI.createView({
			left: 0,
			right: 0,
		    height: 1,
		    horizontalWrap: false,
		});
		var addressToView = Ti.UI.createView({
			left: 0,
			right: 0,
		    height: footerInnerHeight/2,
		    horizontalWrap: false,
		});
		
		var lblFrom = Ti.UI.createLabel({
		    left:0,
		    width: 36,
		    wordWrap:false,
		    ellipsize:true,
		    color:Theme.map.smallColor,
	  	    font:Theme.map.smallFont,
	  	    minimumFontSize:Theme.map.smallFont.fontSize,
		    text:'From',
		    touchEnabled:false,
		    verticalAlign:Titanium.UI.TEXT_VERTICAL_ALIGNMENT_CENTER,
			textAlign : 'right',
		});
		addressFromView.add(lblFrom);
		
		var addressFromViewWrapper = Ti.UI.createView({
			left:42,
			right:0,
		});

		var pencilFrom = Ti.UI.createImageView({
		    left:0,
		    height:15,
		    width:15,
		    image:'/images/map/btn_pencil_noborder.png',
		    touchEnabled:false,
		});
		addressFromViewWrapper.add(pencilFrom);

		var lblFromAddress = Ti.UI.createLabel({
		    left:20,
		    right: 0,
		    wordWrap:false,
		    ellipsize:true,
		    color:Theme.global.fontColor,
	  	    font:Theme.map.addressFont,
	  	    minimumFontSize:Theme.map.addressFont.fontSize,
		    text:'Locating Address...',
		    touchEnabled:false,
		    verticalAlign:Titanium.UI.TEXT_VERTICAL_ALIGNMENT_CENTER,
		    width: Ti.UI.FILL,
		});
		addressFromViewWrapper.add(lblFromAddress);
		addressFromView.add(addressFromViewWrapper);
		addressView.add(addressFromView);
		Theme.animateButton(addressFromViewWrapper);
						
		var footerBorder = Ti.UI.createView({
			left: 42,
			right:0,
			height:1,
			backgroundColor:Theme.global.altBGColor,
		});
		footerBorderView.add(footerBorder);

		addressView.add(footerBorderView);
		
		// wire events
		addressFromView.addEventListener('click',function(e){
			if (ICABBI.navController.opening(self)) {
				return;
			}
			$$.ActivityIndicator.start('Load addresses!');

			if(ICABBI.Config.search_addresses)
			{
				var AddressSearchView = require('/ui/common/AddressSearchView');
				var config = {content:AddressSearchView,
						header:{visible:true , controls : {cancel : {visible: true} }},
						title : 'Address search',
						notScrollable : true,
						data:{destination: false, location : MapControl.currentLocation ? {lat : MapControl.currentLocation.latitude, lng : MapControl.currentLocation.longitude} : null},
					};
			    var win = new ChildWindow(config);
			    ICABBI.navController.open(win,undefined,self);
			    win.addEventListener('close',function(e){
					if(win.__retval && _.isObject( win.__retval) && win.__retval.address){
						bookingData.addressFrom.set(win.__retval.address);
						MapControl.setAddressEdited(bookingData.addressFrom, false);
					}					
				});
			}
			else
			{ 
				var EditAddressView = require('/ui/common/EditAddressView');
				var config = {content:EditAddressView,
						title:'Edit / Favorites',
						titleBtnClose:'Submit',
						header:{visible:true, controls:{cancel:{visible:true},submit:{visible:true}}},
						data:{address: bookingData.addressFrom, location : MapControl.currentLocation ? {lat : MapControl.currentLocation.latitude, lng : MapControl.currentLocation.longitude} : null},
						notScrollable : true,
					};
			    var win = new ChildWindow(config);
			    ICABBI.navController.open(win,undefined,self);
			}
		});
		
		var /*previousAddressId = '',*/
			possibleDestinations = {},
			selectPossibleDestination = function(address_id) {
				if(!address_id || !possibleDestinations[address_id]) {
					resetToAddress();
					addressToView.fireEvent('click');
				}
				bookingData.addressTo.set(possibleDestinations[address_id]);
			};
		
		bookingData.addressFrom.on('change', function(){
			Ti.API.info("FROM address changed ===============");
			Ti.API.info("FROM address changed ==============="+JSON.stringify(bookingData.addressFrom ));
			
			lblFromAddress.setText(bookingData.addressFrom.formatDisplay(false));
			//lblCity.setText(bookingData.addressFrom.get('area'));
			
			/*var addressId = bookingData.addressFrom.get('id');

			if (addressId && addressId != previousAddressId && ICABBI.Controller.getUser().isAuthenticated()) {

				IcabbiAPI.relatedAddresses(addressId, {
					success: function(data){
						
						if(data && data.length) {
							
							possibleDestinations = {};
							
							var pickerDestinations = [];
							
							_.each(data, function(adr){
								
								pickerDestinations.push({
									title: adr.formatted||'',
									id: adr.id
								});
								
								possibleDestinations[adr.id] = {
									success: true,
									id: adr.id,
									src: adr.source||'',
									formatted: adr.formatted||'',
									area: adr.area||'',
									lat: adr.lat,
									lng: adr.lng,
								};
							});
							pickerDestinations.push({
								title: 'Other',
								id: 0
							});
							
							if(data.length > 1) {
								ICABBI.navController.showPickerModal({
								    title:'Where to?',
								    data: pickerDestinations,
								    click : function(e){
							    		selectPossibleDestination(e.selected);
								    }
								});
							}
							else {
								selectPossibleDestination(pickerDestinations[0].id);
							}
						}
					},
					error: function(error){
						Ti.API.info('Error loading related addresses: ' + error); 
					}
				});
				
				previousAddressId = addressId;
			}*/
		});
		
		//Ti.App.addEventListener('locating',function(){
		//	lblFromAddress.setText('Locating Address...');
		//});


		var lblTo = Ti.UI.createLabel({
		    left:0,
		    width: 36,
		    wordWrap:false,
		    ellipsize:true,
			color:Theme.map.smallColor,
	  	    font:Theme.map.smallFont,
	  	    minimumFontSize:Theme.map.smallFont.fontSize,
		    text:'To',
		    verticalAlign:Titanium.UI.TEXT_VERTICAL_ALIGNMENT_CENTER,
		    touchEnabled:false,
		    textAlign:'right',
		});
		addressToView.add(lblTo);
		
		var addressToViewWrapper = Ti.UI.createView({
			left:42,
			right:0,
		});

		var pencilTo = Ti.UI.createImageView({
		    left:0,
		    height:15,
		    width:15,
		    image:'/images/map/btn_pencil_noborder.png',
		    touchEnabled:false,
		});
		addressToViewWrapper.add(pencilTo);
		
		var lblToAddress = Ti.UI.createLabel({
		    left:20,
		    right:0,
		    wordWrap:false,
		    ellipsize:true,
		    color:Theme.global.fontColor,
	  	    font:Theme.map.addressFont,
	  	    minimumFontSize:Theme.map.addressFont.fontSize,
		    text:'',
		    verticalAlign:Titanium.UI.TEXT_VERTICAL_ALIGNMENT_CENTER,
		    touchEnabled:false,
		    width: Ti.UI.FILL,
		});
		addressToViewWrapper.add(lblToAddress);
		addressToView.add(addressToViewWrapper);
		addressView.add(addressToView);
		Theme.animateButton(addressToViewWrapper);
		
		footer.add(addressView);
		// wire events
		addressToView.addEventListener('click',function(e){
			if (ICABBI.navController.opening(self)) {
				return;
			}

			$$.ActivityIndicator.start('Load addresses!');
			if(ICABBI.Config.search_addresses)
			{
				var AddressSearchView = require('/ui/common/AddressSearchView');
				var config = {content:AddressSearchView,
						header:{visible:true , controls : {cancel : {visible: true} }},
						title : 'Address search',
						notScrollable : true,
						data:{destination: true},
					};
			    var win = new ChildWindow(config);
			    ICABBI.navController.open(win,undefined,self);
			    win.addEventListener('close',function(e){
					if(win.__retval && _.isObject( win.__retval) && win.__retval.address){
						bookingData.addressTo.set(win.__retval.address);	
					}
					
				});
			}
			else
			{
				if(!bookingData.pois)
			 	{
			 		bookingData.pois = [];
					
					ICABBI.Controller.getMeta().destinations.each(function(destination){
						bookingData.pois.push({
							title:destination.get('formatted'),
							id:destination.id
						});
					});
			 	}
			 	$$.ActivityIndicator.stop();
			 	if(bookingData.pois.length == 0)
			 	{
			 		self.opening = false;
					return;
			 	}
			 	ICABBI.navController.showPickerModal({
				    title:'Destination',
				    data: bookingData.pois,
				    click : function(e){
				    	//if(e.selected) {
				    		var destination = null;
							ICABBI.Controller.getMeta().destinations.each(function(dest){
								if(dest && dest.id == e.selected){
									destination = dest;							
								}
							});
							if(destination)
				    			bookingData.addressTo.set(destination.toJSON());
				    	//}
				    }
				});
				self.opening = false;
			}
		});

		bookingData.addressTo.on('change',function(){
			Ti.API.info("to address changed ===============");
			lblToAddress.setText(bookingData.addressTo.formatDisplay(false));
		});
		
		resetToAddress();
		
		var btnBook = Ti.UI.createView({
			right:6,
			height:footerInnerHeight,
			width:78,
			//backgroundColor:Theme.global.buttonBGColor,
			borderColor:Theme.global.highlightColor,
			borderWidth:1,
			borderRadius:8,
		});
		
		var iconBook = Ti.UI.createImageView({
			top:16,
			height:34,
			width:38,
			image:'/images/map/btn_book.png',
			touchEnabled:false,
		});
		btnBook.add(iconBook);
		
		var labelBook = Ti.UI.createLabel({
			top:50,
			height:24,
			width:Ti.UI.FILL,
			color:Theme.global.fontColor,
			font:Theme.map.buttonFont,
			text:'Book',
			textAlign:'center',
			touchEnabled:false,
		});
		btnBook.add(labelBook);

		footer.add(btnBook);
		Theme.animateButton(btnBook);

		btnBook.addEventListener('click', function(e) {
			if(btnSlideMenu)
				btnSlideMenu.visible=true;
			ICABBI.SlidingMenu.fireEvent('openview',{index:1});
		});
		
		return footer;
	};	

	var createMap = function(){
		var view = Ti.UI.createView({
			top:0,
			//bottom:Ti.Android? footerHeight:0, // TODO: see if we should change this with V2
			bottom:0,
			left: 0,
			right:0,
			visible:true,
			backgroundColor:'#ffffff'
		});
		
		// if(ICABBI.Controller.DEVICE_LOCATION_LATLNG)

		MapControl.init(_.extend({
			mapType		: Titanium.Map.STANDARD_TYPE,
	    	latitude	: 0,
	    	longitude	: 0,
    	    animate		: true,
		    regionFit 	: true,
		    userLocation: false,
		    annotations	: [],
		},ICABBI.Controller.DEVICE_LOCATION_LATLNG),bookingData.addressFrom);
		mapControl = MapControl.getView();
		view.add(mapControl);
		
		map_overlay = Ti.UI.createImageView({
		    width:Ti.Android ? 45:30,
		    height:Ti.Android ? 51:34,
			image:'/images/map/pin_hail.png',
		    touchEnabled:false
		});
		
		view.add(map_overlay);
		
		var centerPin = function(){
			view.removeEventListener('postlayout',centerPin);
			map_overlay.top = (view.size.height/2)-map_overlay.height;
		};
		
		view.addEventListener('postlayout',centerPin);
		return view;
	};
	
	var create = function(reset){
		//create object instance, a parasitic subclass of Observable
	    var view = reset;
	    if(!view)
		    view = new SlidingView({
		        width:Ti.UI.FILL,
		        height:Ti.UI.FILL,
		        backgroundColor:'#fff',
		    });
		
		//do like this so that footer can register on events
		headerBg = createHeaderBg();
		header = createHeader();
		if(!reset)
			map = createMap();
		footerBg = createFooterBg();
		footer = createFooter();
		view.add(headerBg); 
		view.add(header);
		view.add(map);
		//view.mapControl = map;
		view.add(footerBg);
		view.add(footer);
		view.getBookingData = function(){
			return bookingData;
		};
		return view;
	};

	var self = create();
	
	
	ICABBI.Controller.on('mapview:reset',function(data){
		bookingData.reset();
	});
		ICABBI.Controller.on('mapview:open:edit',function(data) {
			if(data.find) {
				if(btnSlideMenu)
					btnSlideMenu.visible = false;
			} else {
				if(btnSlideMenu)
					btnSlideMenu.visible=true;
			}
		});
		
		
		self.addEventListener('slidingmenu:view:load',function(e){
			//MapControl.updateMapLocation({latitude:bookingData.addressFrom.get('lat'),longitude:bookingData.addressFrom.get('lng')},false);
			setTimeout(function(){MapControl.setAddressEdited(bookingData.addressFrom, false);}, 1000);
		}); 
	
	ICABBI.Controller.on('application:reset',function(data){
		
		self.remove(headerBg); 
		self.remove(header);
		self.remove(map);
		self.remove(footerBg);
		self.remove(footer);
		
		create(self);
		setTimeout(function(){
			bookingData.reset();
			$$.ActivityIndicator.stop();
		}, 1000);
	});
	
	return self;
};

module.exports = BookingView;
