var ICABBI = require('/core/icabbi');
var _ = require('/lib/lodash')._ ,
	$$ = require('/core/ThemeManager');

var Theme = require('/ui/Theme');
var ToggleButton = require('/ui/common/ToggleButton');
var FormPickerRow = require('/ui/common/FormPickerRow');

//BookingsView Component Constructor
var BookingsView = function(options) {
	//create object instance, a parasitic subclass of Observable
	var defaults = {
		title:'My Bookings',
	};
	options = _.extend(defaults,options.data||{});
	
	var bookingSelectedIndex = 0;
	
	var createSectionHeader = function(leftTitle,rightTitle) {
		var header = Ti.UI.createView({
			left:0,
			right:0,
			top:0,
			height:40,
			backgroundColor:Theme.global.highlightColor,
		});
		
		var leftLabel = Ti.UI.createLabel({
			left:15,
			height:26,
			width:Ti.UI.SIZE,
			color:Theme.menu.headerColor,
			font:Theme.menu.rowFont,
			text:leftTitle,
		});
		header.add(leftLabel);
		
		var rightLabel = Ti.UI.createLabel({
			right:12,
			height:26,
			width:Ti.UI.SIZE,
			color:Theme.menu.headerColor,
			font:Theme.menu.rowFont,
			text:rightTitle,
			textAlign:'right',
		});
		header.add(rightLabel);
		
		return header;
	};
	
	var createEmptyRow = function(bookingName) {
		return Ti.UI.createTableViewRow({
			height:44,
			color:Theme.global.fontColor,
			backgroundColor:Theme.global.bgColor,
			selectedColor: 'transparent',
			backgroundFocusedColor:'transparent',
			backgroundSelectedColor:'transparent',
			selectionStyle:Ti.Android?undefined:Ti.UI.iPhone.TableViewCellSelectionStyle.NONE,
			title:'No '+bookingName+' Bookings',
			touchEnabled:false,
			font:Theme.menu.rowFont,
			emptyRow:true,
		});
	};
	
	var bookingsActive = ICABBI.Controller.getUser().bookings,
		bookingsUpcoming = ICABBI.Controller.getUser().bookingsUpcoming,
		bookingsPrevious = ICABBI.Controller.getUser().bookingsPrevious;
	
	var self = Ti.UI.createView({
		backgroundColor:'#fff',
		top:0,
		bottom:0,
		left:0,
		right:0,
	});

	// whenever this view is focused, do this, starting the second time
	self.addEventListener('slidingmenu:view:load',function(e){
		ICABBI.Controller.getUser().syncBookingsActive(true);
		
		if (bookingSelectedIndex === 1) {
			ICABBI.Controller.getUser().syncBookingsUpcoming(true);
		}
		if (bookingSelectedIndex === 2) {
			ICABBI.Controller.getUser().syncBookingsPrevious(true);

		}
	});
	
	// add buttons
	var tabsBox = Ti.UI.createView({
		top:0,
		left:0,
		right:0,
		height: 40,
		backgroundColor:Theme.nav.backgroundColor,
		layout:'horizontal',
	});
	self.add(tabsBox);
	
	var rowLineBottom = Ti.UI.createView({
		left:0,
		right:0,
		top:39,
		height:'2px',
		backgroundColor:Theme.nav.seperatorColor,
	});
	//self.add(rowLineBottom);
	
	var activeButton = new ToggleButton({
		height:40,
		width:'33%',
		borderSide:'right',
		selected:bookingSelectedIndex===0,
		title:'Active'
	});
	tabsBox.add(activeButton);
	activeButton.addEventListener('change', function(e){
		if (e.selected) {
			upcomingButton.turnOff();
			previousButton.turnOff();
			bookingSelectedIndex = 0;
			switchView();
		}
	});
	
	var upcomingButton = new ToggleButton({
		height:40,
		width:'33%',
		borderSide:'right',
		selected:bookingSelectedIndex===1,
		title:'Upcoming'
	});
	tabsBox.add(upcomingButton);
	upcomingButton.addEventListener('change',function(e){
		if (e.selected) {
			activeButton.turnOff();
			previousButton.turnOff();
			bookingSelectedIndex = 1;
			switchView();
		}
	});
	
	var previousButton = new ToggleButton({
		height:40,
		width:Ti.UI.FILL,
		selected:bookingSelectedIndex===2,
		title:'Previous'
	});
	tabsBox.add(previousButton);
	previousButton.addEventListener('change',function(e){
		if (e.selected) {
			activeButton.turnOff();
			upcomingButton.turnOff();
			bookingSelectedIndex = 2;
			switchView();
		}
	});
	
	var switchView = function() {
		if (bookingSelectedIndex === 0) {
			ICABBI.Controller.getUser().syncBookingsActive(true);
			// show active bookings
			activeBox.visible = true;
			upcomingBox.visible = false;
			previousBox.visible = false;
		}
		if (bookingSelectedIndex === 1) {
			ICABBI.Controller.getUser().syncBookingsUpcoming(true);
			// show active bookings
			upcomingBox.visible = true;
			activeBox.visible = false;
			previousBox.visible = false;
		}
		if (bookingSelectedIndex === 2) {
			ICABBI.Controller.getUser().syncBookingsPrevious(true);
			// show active bookings
			previousBox.visible = true;
			activeBox.visible = false;
			upcomingBox.visible = false;
		}
		// TODO: this would be a great place to force a sync
	};
	
	// active header/table
	var activeBox = Ti.UI.createView({
		top:40,
		bottom:0,
		left:0,
		right:0,
		visible:bookingSelectedIndex===0,
	});
	self.add(activeBox);
	
	//activeBox.add(createSectionHeader('From','Status'));
	
	var activeTable = Ti.UI.createTableView({
		backgroundColor:'#fff',
		separatorColor:'transparent',
		separatorStyle:Ti.UI.Android ? undefined : Ti.UI.iPhone.TableViewSeparatorStyle.NONE,
		allowsSelection:Ti.Android,
		allowsSelection:Ti.Android,
		top:0,
		bottom:0,
		left:0,
		right:0,
		title:'Active',
	});
	activeBox.add(activeTable);
	
	// upcoming header/table
	var upcomingBox = Ti.UI.createView({
		top:40,
		bottom:0,
		left:0,
		right:0,
		visible:bookingSelectedIndex===1,
	});
	self.add(upcomingBox);
	
	//upcomingBox.add(createSectionHeader('From','Booked'));
	
	var upcomingTable = Ti.UI.createTableView({
		backgroundColor:'#fff',
		separatorColor:'transparent',
		separatorStyle:Ti.UI.Android ? undefined : Ti.UI.iPhone.TableViewSeparatorStyle.NONE,
		top:0,
		bottom:0,
		left:0,
		right:0,
		title:'Upcoming',
	});
	upcomingBox.add(upcomingTable);
	
	// previous header/table
	var previousBox = Ti.UI.createView({
		top:40,
		bottom:0,
		left:0,
		right:0,
		visible:bookingSelectedIndex===2,
	});
	self.add(previousBox);
	
	//previousBox.add(createSectionHeader('From','Booked'));
	
	var previousTable = Ti.UI.createTableView({
		backgroundColor:'#fff',
		separatorColor:'transparent',
		separatorStyle:Ti.UI.Android ? undefined : Ti.UI.iPhone.TableViewSeparatorStyle.NONE,
		allowsSelection:Ti.Android,
		top:0,
		bottom:0,
		left:0,
		right:0,
		title:'Previous',
	});
	previousBox.add(previousTable);
	
	var createRow = function(booking) {
		var address = booking.get('address');
		
		var row = Ti.UI.createTableViewRow({
			height:58,
			width:'100%',
			backgroundColor:'transparent',
			selectedColor: 'transparent',
			backgroundFocusedColor:'transparent',
			backgroundSelectedColor:'transparent',
			selectionStyle:Ti.Android?undefined:Ti.UI.iPhone.TableViewCellSelectionStyle.NONE,
			__booking:booking.get('perma_id'),
			emptyRow:false,
		});
		
		var container=Ti.UI.createView({
			right: booking.isFinished() ? 32 : 25,
			height:58,
			left:0,
			touchEnabled:true,
		});
		row.add(container);
		
		var title = Ti.UI.createLabel({
			color:Theme.global.fontColor,
			left:10,
			right:0,
			top:8,
			height:23,
			text:address.formatted,
			touchEnabled:false,
		    wordWrap:false,
		    ellipsize:true,
			width: Ti.UI.FILL,
			verticalAlign:'bottom',
			minimumFontSize:Theme.address.addressFont.fontSize,
			font:Theme.address.addressFont,
		});

		container.add(title);
		
		var dateText = timeText = '';
		var localeDateTime = booking.formatShortDateLocaleDisplay();
		dateText = localeDateTime.date.replace(' ','');
		timeText = localeDateTime.time;
			
		var subtitle = Ti.UI.createLabel({
			color:'#999999',
			left:10,
			right:0,
			top:28,
			height:17,
			text:address.area||''+' @'+dateText+' '+timeText,
			touchEnabled:false,
		    wordWrap:false,
		    ellipsize:true,
			font:Theme.address.subaddressFont,
		});
		container.add(subtitle);
		
		// if (booking.isActive()) {
			// var statusText = booking.get('status');
// 			
			// if(!!~(['NEW','ASSIGN','BACKTOBASE','DRIVER_CANCELLED'].indexOf(statusText)))
			// {
				// statusText='Assigning Driver';
			// }
			// else if(booking.get('status')==='ENROUTE'){
				// statusText='Driver On Way';
			// }
			// else if(booking.get('status')==='MADECONTACT' || booking.get('status')==='DROPPINGOFF'){
				// statusText='On Board';
			// }
// 			
			// statusText = statusText.toUpperCase();
// 
			// var statusLabel = Ti.UI.createLabel({
				// right:12,
				// height:75,
				// width:100,
				// color:Theme.nav.headerColor,
				// font:Theme.nav.headerFont,
				// text:statusText,
				// textAlign:'right',
				// touchEnabled:false,
			// });
			// container.add(statusLabel);
		// } else {
			// var dateText = timeText = '';
// 			
			// var localeDateTime = booking.formatShortDateLocaleDisplay();
			// dateText = localeDateTime.date;
			// timeText = localeDateTime.time;
// 
			// var dateLabel = Ti.UI.createLabel({
				// right:12,
				// top:15,
				// height:28,
				// width:100,
				// color:Theme.global.fontColor,
				// font:Theme.menu.rowFont,
				// minimumFontSize:Theme.menu.rowFont.fontSize,
				// wordWrap:false,
				// text:dateText,
				// textAlign:'right',
				// touchEnabled:false,
			// });
			// container.add(dateLabel);
// 			
			// var timeLabel = Ti.UI.createLabel({
				// right:12,
				// bottom:15,
				// height:21,
				// width:160,
				// color:Theme.form.smallColor,
				// font:Theme.form.smallFont,	
				// text:timeText,
				// textAlign:'right',
				// touchEnabled:false,
			// });
			// container.add(timeLabel);
		// }
		
		if (booking.isFinished()) {
			var removeContainer =Ti.UI.createView({
				top:0,
				width:32,
				bottom:0,
				right:0,
				touchEnabled:true,
			});
			row.add(removeContainer);
			var removeBtn = Ti.UI.createImageView({
			    height:22,
			    width: 22,
			    right: 5,
			    top:18,
			    image:'/images/buttons/icon_remove.png',
			    touchEnabled:true,
			});
			removeContainer.addEventListener('click', function(e) {
					ICABBI.navController.showYesNo({
					    title:'Remove this booking?',
					    message:'Are you sure you want to remove this booking?',
					    click : function(e){
					    	if(e.index == 1){
					    		ICABBI.Controller.deleteBooking(row.__booking,{
									success: function(data){										
										var content = [];
										if(data.error) {
											content.push(data.message || 'Error delete booking!');
										}
										else {
											ICABBI.Controller.getUser().syncBookingsPrevious(true);
										}
									}
								});
							};
					    }
					});
			});
			removeContainer.add(removeBtn);
		}
		else {
			var arrowIcon = Ti.UI.createImageView({
			    height:11,
			    width:11,
			    right: 7,
			    top:23,
			    image:'/images/map/arrow.png',
			    touchEnabled:false,
			});
			row.add(arrowIcon);		
		}
		var rowTopLine = Ti.UI.createView({
			left:0,
			right:0,
			top:0,
			height:'1px',
			backgroundColor:'magenta',
			touchEnabled:false,
		});
		//row.add(rowTopLine);
		var rowBottomLine = Ti.UI.createView({
			left:0,
			right:0,
			bottom:0,
			height:'1px',
			backgroundColor:'#cacaca',
			touchEnabled:false,
		});
		row.add(rowBottomLine);
		Theme.animateButton(container);		
		if (booking.isFinished()){
			container.addEventListener('click',function(e){
				e.row=row;
				clickHandler(e,previousTable);
			});
		}	
		return row;
	};
	
	var clickHandler = function(e,table){
		if (ICABBI.navController.opening(table)) {
			return;
		}
		var ChildWindow = require('/ui/common/ChildWindow');
		var BookingDetailView = require('/ui/common/BookingDetailView');
		
		var row = e.row;
		
		if (row.emptyRow) {
			return;
		}
		
		var bookings;
		var title;
		if (bookingSelectedIndex === 0) {
			bookings = bookingsActive;
			title = 'Active';
		}
		if (bookingSelectedIndex === 1) {
			bookings = bookingsUpcoming;
			title = 'Upcoming';
		}
		if (bookingSelectedIndex === 2) {
			bookings = bookingsPrevious;
			title = 'Previous';
		}
		
		var booking = bookings.get(row.__booking);
		
		setTimeout(function(){
			var config = {
				content:BookingDetailView,
				title:title,
				header:{visible:true,controls:{cancel:{visible:true},submit:{visible:false},close:{visible:true}}},
				data:{booking:booking},
				notScrollable: true,
			};
			
			var win = new ChildWindow(config);
			ICABBI.navController.open(win,undefined,table);
		},13);
	};
	
	(function(){
		//table click action handler
		activeTable.addEventListener('click',function(e){
			clickHandler(e,activeTable);
		});
		upcomingTable.addEventListener('click',function(e){
			clickHandler(e,upcomingTable);
		});
		
		var populate = function(bookings, table, wireEvents) {
			if (!table) {
				return;
			}
			
			var rows = [];
			
			bookings.each(function(booking){
				rows.push(createRow(booking));
			});
			
			if (!rows.length) {
				rows.push(createEmptyRow(table.title));
			}
			
			table.data = rows;
			
			if(wireEvents) {
				bookings.on('add',function(booking){
					Ti.API.info('###############bookingsview [ADD]: '+booking.get('perma_id'));
					if (table.data.length && table.data[0].rows && table.data[0].rows.length && table.data[0].rows[0].emptyRow) {
						table.data = [createRow(booking)];
					} else {
						table.insertRowBefore(0,createRow(booking));
					} 
					if (table === activeTable) {
						activeButton.fireEvent('click');
					}
				});		
				bookings.on('reset',function(e){
					var test = 1;
				});
				bookings.on('cleared',function(e){
					table.data = [createEmptyRow(table.title)];
				});
				bookings.on('remove',function(booking){
					booking.off();
					var id = booking.get('perma_id');
		
					// find the row
					for (var i in table.data[0].rows) {
						if (table.data[0].rows[i].__booking === id) {
							// remove row
							table.deleteRow(table.data[0].rows[i]);
							break;
						}
					}
		
					if (!table.data[0].rows.length) {
						table.data = [createEmptyRow(table.title)];
					}
				});
			}
		};

		/*ICABBI.Controller.on('change:active_bookings',function(e){
			populate(bookingsActive,activeTable, false);
		});
		
		bookingsUpcoming.on('sync:upcoming',function(e){
			populate(bookingsUpcoming,upcomingTable, false);		
		});*/
		
		//if bookingsActive are fetched before viewload
		bookingsActive.once('sync:active',function(e) {
			self.addEventListener('slidingmenu:view:load',function(e){
				self.removeEventListener('slidingmenu:view:load',arguments.callee);
				populate(bookingsActive,activeTable, true);
			});
		});
		bookingsUpcoming.once('sync:upcoming',function(e){
			self.addEventListener('slidingmenu:view:load',function(e){
				self.removeEventListener('slidingmenu:view:load',arguments.callee);
				populate(bookingsUpcoming,upcomingTable, true);
			});
		});
		bookingsPrevious.once('sync:previous',function(e){
			self.addEventListener('slidingmenu:view:load',function(e){
				self.removeEventListener('slidingmenu:view:load',arguments.callee);
				populate(bookingsPrevious,previousTable, true);
			});
		});
	})();

	return self;
};

module.exports = BookingsView;
