/**
 * @author Sonal 
 */
var ICABBI = require('/core/icabbi');
var tools = require('/lib/tools'),
	_ = require('/lib/lodash')._ ,
	$$ = require('/core/ThemeManager');

var Theme = require('/ui/Theme');
var ActionButton = require('/ui/common/ActionButton');
var FormPickerRow = require('/ui/common/FormPickerRow');
var FormValidationRow = require('/ui/common/FormValidationRow');

//ClientAccountPaymentView
var ClientAccountPaymentView = function(options) {
	options = options || {};
	var currClientAccount,
		booking = options.data.booking,
		clientaccounts = options.data.clientaccounts;
	
	var createForm = function(){
		var form=[];
		//create object instance, a parasitic subclass of Observable
		var view = Titanium.UI.createView({
	        height:options.height || Ti.UI.SIZE,
	        width:options.width || Ti.UI.FILL,
			layout:'vertical',
			top: 10,
	    });
	    
		/*
		 *******************************************
		 * Section Info
		 *******************************************
		 */	
		view.add(Ti.UI.createLabel({
			top:10,
			left:16,
			right:16,
			height:50,
			color:Theme.global.fontColor,
			font:Theme.global.font,
			text:'Please choose an account to charge this journey to:',
		}));

   		/*
   		 * ClientAccount
   		 */
   		var accountRow = new FormPickerRow({
   			label:'Acc.',
   			title:'',
   			subTitle:'',
   			firstItem:true,
   		});
   		view.add(accountRow);

		/*
		 *******************************************
		 * Section ClientAccountFields
		 *******************************************
		 */	
		var sectionClientAccountFields = Titanium.UI.createView({
	        left:0,
	        right:0,
	        top:5,
	        height: Ti.UI.SIZE,
			layout:'vertical',
	    });
		view.add(sectionClientAccountFields);

   		/*
		 * add accounts
   		 */
		var clientAccountData = [];
		clientaccounts.each(function(clientaccount){
			clientAccountData.push({
				title:clientaccount.get('name'),
				id:clientaccount.get('id')
			});
		}); //each
		var row = clientAccountData[0];
		currClientAccount = clientaccounts.get(row.id);
								
		var accountRowUpdate = function(e){		
			// reset form fields
			form = [];
    		accountRow.putText({
	   			label:'',
	   			title:currClientAccount.get('name'),
	   			subTitle:'',
	   		});
	   			   		
	   		for (var i = sectionClientAccountFields.children.length-1;i>-1;i--) {
	   			sectionClientAccountFields.children[i].doCleanup();
	   			sectionClientAccountFields.remove(sectionClientAccountFields.children[i]);
	   		}

			if(currClientAccount.get('type')==='CASH'){
				return;
			}
			
			_.each(currClientAccount.get('fields'),function(clientaccountfield){
				if(!clientaccountfield.active) return;
				
				var fld,description;
				var	flddata={
						id:clientaccountfield.id,
						key:clientaccountfield.key,
						title:clientaccountfield.title,
						type:clientaccountfield.type,
						regex:clientaccountfield.regex,
						required:clientaccountfield.required
				};
				fld = new FormValidationRow({
					label:clientaccountfield.title,
					hint:clientaccountfield.description,
					values:JSON.parse(clientaccountfield.values),
					parentWindow:options.parent.window,
					autocapitalization:false,
					autocorrect:true,
					returnKeyType:Ti.UI.RETURNKEY_DONE,
				});
				flddata.getValue = function(){fld.doBlur(); return fld.value;};
				form.push(flddata);

				sectionClientAccountFields.add(fld);
			});
		};
		
		accountRow.addEventListener('click',function(e){
				ICABBI.navController.showPickerModal({
						    title:'Account Select',
						    data: clientAccountData,
						    click : function(e){	
						    	if(e.selected || e.selected === 0) 
						    	{
						    		currClientAccount=clientaccounts.get(e.selected);
						    		accountRowUpdate();							   		
						    	}
						    }
						});			
			});	
			
		accountRowUpdate();
		
		/*
		 * Control button
		 */		
		var controlBar = Titanium.UI.createView({
			top:10,
			bottom:10,
	        height:70,
	        width:Ti.UI.SIZE,
	    });
	    view.add(controlBar);
		
		var btnSubmit = new ActionButton({title:'CONFIRM',icon:'/images/buttons/icon_account.png'});
		controlBar.add(btnSubmit);
		
		btnSubmit.addEventListener('click',function(e){
			var fields={};
			var error='';
			var NoError=true;
			if(currClientAccount.get('type')!=='CASH'){
				for(var i=0,count=form.length;i<count;i++){
					if(!NoError) break;
					var field=form[i];
					fields[field.id]=field.getValue();
					
					// check- required 
					if(field.required && field.getValue().toString().length==0){
						error +='- ' +field.title+' is required!\n';
						NoError=false;
					}
					// check- pattern 
					if(field.type==='text' && field.regex.length){
						var pattern = field.regex.match(/^\/([^\/]*)\/([a-z]*)$/i);
						if(pattern && pattern.length===3){
							pattern=new RegExp(pattern[1],pattern[2]);
							if(!pattern.test(field.getValue())){
								error +='- '+ field.title+' value is invalid!\n';
								NoError=false;
							}
						}
					}
				}
			}
			if(!NoError) {
				ICABBI.navController.showError({
				    title:'Error!',
				    message:error,
				    close:true
				});
				return;
			}
			//send fields
			Ti.API.info('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>account: '+JSON.stringify(fields));
			// {"6":"ss2222","182":"test1"}
			if(booking){
				booking.set({account_id:currClientAccount.get('id')});
				booking.setPayment(currClientAccount.get('type'),fields);
			}
			setTimeout(function(){
				ICABBI.navController.close(options.parent.window);
			},13);
		});

		return view;
	};
	
	var self = createForm();
	return self;
};

module.exports = ClientAccountPaymentView;
