/**
 * @author Sonal 
 */
var ICABBI = require('/core/icabbi');
var tools = require('/lib/tools'),
	_ = require('/lib/lodash')._ ,
	$$ = require('/core/ThemeManager');

var Theme = require('/ui/Theme');
var ActionButton = require('/ui/common/ActionButton');
var FormPickerRow = require('/ui/common/FormPickerRow');
var PickerControl = require('/ui/common/PickerControl');

var createRow = function(args){
		
		var self = Ti.UI.createView({
			left:8,
			right:8,
			height:54,
			putText:function(args){
				input.value = args.value;
			},
			value:args.value,
		});
		
		var label = Ti.UI.createLabel({
			left:0,
			height:21,
			width:45,
			color:Theme.form.labelColor,
			font:Theme.form.labelFont,
			text:args.label,
			textAlign:'right',
			touchEnabled:false,
		});
		self.add(label);
		
		var input = Ti.UI.createTextField({
			left:50,
			right:10,
			paddingLeft:0,
			backgroundColor:'transparent',
			borderColor:'transparent',
			color:Theme.global.fontColor,
			font:Theme.menu.rowFont,
			hintText:args.hint,
			value:self.value,
			// keyboard stuff
			clearButtonMode:args.clearButtonMode,
			clearOnEdit:args.clearOnEdit,
			keyboardType:args.keyboardType,
			keyboardToolbar:args.keyboardToolbar,
			passwordMask:args.passwordMask,
			returnKeyType:args.returnKeyType,
			autocapitalization:args.autocapitalization,
			autocorrect:args.autocorrect,
			maxLength:args.maxLength,
		});
		input.addEventListener('change',function(e){
			self.value = e.value;
		});
		self.add(input);
		
		self.done = function(){
			input.blur();
		};
		
		self.getValue = function() {
			return input.getValue();
		};
				
		if(!args.readonly && !args.inline)
			Theme.animateButton(self);
		
		return self;
	};

//CreditCardPaymentView Component Constructor
var CreditCardPaymentView = function(options) {
	options = options || {};
	var ICABBI = require('/core/icabbi');

	var currCreditCard,
		booking = options.data.booking,
		creditcards = options.data.creditcards,
		requiresCVV = options.data.requiresCVV || false;

	var createForm = function(){
		//create object instance, a parasitic subclass of Observable
		var view = Titanium.UI.createView({
	        height:Ti.UI.FILL,
	        width:Ti.UI.FILL,
			layout:'vertical'
	    });
	    
		/*
		 *******************************************
		 * Section Info
		 *******************************************
		 */	
		view.add(Ti.UI.createLabel({
			top:10,
			left:16,
			right:16,
			height:Ti.UI.SIZE,
			color:Theme.global.fontColor,
			font:Theme.global.font,
			text:'Please choose your saved card to charge this journey to. You will also need your CVV security code.',
		}));

   		/*
   		 * Card
   		 */
   		var cardRow = new FormPickerRow({
   			label:'Card',
   			title:'',
   			subTitle:'',
   			lastItem: !requiresCVV,
   		});
   		view.add(cardRow);

   		/*
		 * add cards
   		 */
		var creditCardData = [];
		creditcards.each(function(creditcard){ 
			creditCardData.push({
	   			title:creditcard.get('nickname'),
				id:creditcard.get('id')
			});
		}); //each
		
		
		var cardid = options.data.creditcard_id || creditCardData[0].id;
		currCreditCard = creditcards.get(cardid);		
		
		cardRow.putText({
   			label:'Card',
   			title:currCreditCard.get('nickname'),
   			subTitle:'',
   		});
		
		cardRow.addEventListener('click',function(e){
			if(options.data.creditcard_id)
				ICABBI.navController.showPickerModal({
						    title:'Card Select',
						    data: creditCardData,
						    click : function(e){
						    	if(e.selected || e.selected === 0) 
						    	{
						    		currCreditCard = creditcards.get(e.selected);
						    		cardRow.putText({
						   			label:'Card',
						   			title:currCreditCard.get('nickname'),
						   			subTitle:'',
						   		});
						    	}
						    }
						});			
			});	

   		if(requiresCVV) {
	   		cvvRow = createRow({
	   			label:'CVV',
	   			hint: 'Required for payment',
	   			inline : true,
				autocapitalization:false,
				autocorrect:false,
				returnKeyType:Ti.UI.RETURNKEY_NEXT,
				keyboardType:Titanium.UI.KEYBOARD_NUMBER_PAD,
				maxLength:4,
				value:''
		    });
	   		view.add(cvvRow);
   		}

		/*
		 * Control button
		 */		
		var controlBar = Titanium.UI.createView({
			top:10,
	        height:70,
	        width:Ti.UI.SIZE,
	    });
	    view.add(controlBar);
		
		var btnSubmit = new ActionButton({title:'CONFIRM',icon:'/images/buttons/icon_card.png'});
		controlBar.add(btnSubmit);
		
		btnSubmit.addEventListener('click',function(e){
			var error = '';
			if(!currCreditCard) {
				error += '- Choose a credit card!\r\n';
			}			
			if(requiresCVV && !cvvRow.getValue().length) {
				error += '- CVV is required!\r\n';
			}
			
			if(error){
				ICABBI.navController.showError({
				    title:'Error!',
				    message:error,
				    close:true
				});
				return;
			};
					
			booking.set('creditcard_id', currCreditCard.get('id'));
			if(requiresCVV)
				booking.set('card_cvv', cvvRow.getValue());

			booking.setPayment(booking.PAYMENT_MODES.CREDITCARD);
			
			setTimeout(function(){
				options.parent.window.__retval = true;
				ICABBI.navController.close(options.parent.window);
			}, 13);
		});

		return view;
	};
	
	var self = createForm();
	return self;
};

module.exports = CreditCardPaymentView;
