/**
 * @author Sonal
 */
var ICABBI = require('/core/icabbi');
var tools = require('/lib/tools'),
	_ = require('/lib/lodash')._ ,
	$$ = require('/core/ThemeManager'),
	CreditCard = require('/model/CreditCard'),
	ConnectPayAPI = require('/core/ConnectPayAPI');

var Theme = require('/ui/Theme');
var FormPickerRow = require('/ui/common/FormPickerRow');
var FormTextfieldRow = require('/ui/common/FormTextfieldRow');
var FormCustomRow = require('/ui/common/FormCustomRow');
var ActionButton = require('/ui/common/ActionButton');

var luhn10 = function(a,b,c,d,e) {
  for(d = +a[b = a.length-1], e=0; b--;)
    c = +a[b], d += ++e % 2 ? 2 * c % 10 + (c > 4) : c;
  return !(d%10);
};

//CreditCardRegistrationView Component Constructor
var CreditCardRegistrationView = function(options) {
	options = options || {};
	var ICABBI = require('/core/icabbi');
	var booking = options.data.booking ? options.data.booking : null;
	var creditcard = options.data.creditcard ? options.data.creditcard.clone() : (new CreditCard.Model());
	
	var hasFliends = ICABBI.Config.card_config && ICABBI.Config.card_config.fields;
	
	var showCardHolder = hasFliends &&  'card_holder' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.card_holder.visible;
	var requiredCardHolder = hasFliends &&  'card_holder' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.card_holder.required;
	
	var showAccountCard = hasFliends &&  'accountcard' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.accountcard.visible;
	var requiredAccountCard = hasFliends &&  'accountcard' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.accountcard.required;
	
	var showSurname = hasFliends &&  'surname' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.surname.visible;
	var requiredSurname = hasFliends &&  'surname' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.surname.required;
	
	var showFirstname = hasFliends &&  'firstname' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.firstname.visible;
	var requiredFirstname = hasFliends &&  'firstname' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.firstname.required;
	
	var showStreet = hasFliends &&  'street' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.street.visible;
	var requiredStreet = hasFliends &&  'street' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.street.required;
	
	var showHouseNo = hasFliends &&  'houseno' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.houseno.visible;
	var requiredHouseNo = hasFliends &&  'houseno' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.houseno.required;
	
	var showPostcode = hasFliends &&  'postcode' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.postcode.visible;
	var requiredPostcode = hasFliends &&  'postcode' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.postcode.required;
	
	var showCity = hasFliends &&  'city' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.city.visible;
	var requiredCity = hasFliends &&  'city' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.city.required;
	
	var showCountry = hasFliends &&  'country' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.country.visible;
	var requiredCountry = hasFliends &&  'country' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.country.required;
	
	var requiredState = hasFliends &&  'country' in ICABBI.Config.card_config.fields && ICABBI.Config.card_config.fields.state.required;
	
	var createForm = function(){
		var form={fields:{}};
	
		//create object instance, a parasitic subclass of Observable
		var view = Titanium.UI.createView({
			top:0,
			left:0,
			right:0,
	        height:Ti.UI.SIZE,
			layout:'vertical',
			zIndex:10,
	    });
	    
		/*
		 *******************************************
		 * Section Logos
		 *******************************************
		 */	
		var sectionLogos = Titanium.UI.createView({
	        height: 42,
	        width:Ti.UI.SIZE,
			layout:'horizontal',
			top:10,
	    });
		view.add(sectionLogos);
		_.each(Theme.form.creditCards,function(imagePath){
			sectionLogos.add(Ti.UI.createImageView({
			  image:imagePath,
			  left:1,
			  width:67,
			  height:42,
			}));
		});

		/*
		 *******************************************
		 * Section CardDetails
		 *******************************************
		 */	
	    /*
	     * Fullname
	     */
	    var nameRow = new FormTextfieldRow({
   			label:'Name',
   			hint:'eg. "Work Card", "My Debit"',
			autocapitalization:true,
			autocorrect:true,
			returnKeyType:Ti.UI.RETURNKEY_NEXT,
			value:creditcard.get('nickname'),
	    });
		form.fields.name = {
			getValue: function(){nameRow.doBlur();return nameRow.value;}
		};
		view.add(nameRow);

		form.fields.ownername = {
			getValue: function(){ return ''; }
		};
		var ownerNameRow = null;
		if(showCardHolder)
		{
		    /*
		     * Owner's name
		     */
		    ownerNameRow = new FormTextfieldRow({
	   			label:'Full Name',
	   			hint:'Name written on the card',
				autocapitalization:true,
				autocorrect:true,
				returnKeyType:Ti.UI.RETURNKEY_NEXT,
				value:creditcard.get('ownername'),
		    });
			form.fields.ownername = {
				getValue: function(){ownerNameRow.doBlur(); return ownerNameRow.value;}
			};
			view.add(ownerNameRow);
		}
		
		var accountcard_id = options.data.accountcard_id;
		var accountCardRow = null;
		if(showAccountCard)
		{
			var card_types = [];
			
			var chooseAccountCard = function() {
				if(options.data.accountcard_id)
					return;
				var accountCards = [];
				
			 	_.each(card_types, function(card_type){
					accountCards.push({
							title: card_type.name,
							id: card_type.accountcard_id,
						});
				});
					
				ICABBI.navController.showPickerModal({
				    title:'Card Type',
				    data: accountCards,
				    click : function(e){
				    	if(e.selected) {
				    		var account = null;
							_.each(card_types, function(card_type){
								if(card_type.accountcard_id == e.selected){
									account = card_type;							
								}
							});
				    		accountCardRow.putValue(account.name);
				    		accountcard_id = e.selected;
				    	}
				    	else {
				    		accountCardRow.putValue('');
				    		accountcard_id = 0;
				    	}
				    }
				});
			};		
		    /*
		     * Card type
		     */
		    accountCardRow = new FormTextfieldRow({
	   			label:'Card Type',
	   			hint:'Choose ...',
				autocapitalization:true,
				autocorrect:true,
				returnKeyType:Ti.UI.RETURNKEY_NEXT,
				editable: false,
				click: options.data.accountcard_id ? null : chooseAccountCard
		    });
			view.add(accountCardRow);
			
			form.fields.accountcard_id = {
				getValue: function(){ return accountcard_id; }
			};
			
			ICABBI.Controller.getCardTypes({
				success: function(data){
					card_types = data;
					
					var id = options.data.accountcard_id || creditcard.get('accountcard_id');
					if(id) {
						var account = null;
						_.each(card_types, function(card_type){
							if(card_type.accountcard_id == id){
								account = card_type;							
							}
						});
						// set name
						accountCardRow.putValue(account.name);
					}
				}
			});
		}


		var cardHolderEdit, accountCardPiker,
			monthEdit, yearEdit, cvvEdit,
			surnameRow, firstnameRow, streetEdit, houseNoEdit, postCodeEdit,
			cityEdit, countryEdit, stateEdit;
	
	    /*
	     * CardNumber
	     */
	    var cardNumberRow = new FormCustomRow({
   			label:'Card #',
			prevField:nameRow,
	    });
		
		var cardNumber = Ti.UI.createTextField({
			left:0,
			right:0,
			height:38,
			backgroundColor:Theme.form.inputBGColor,
			borderRadius:10,
			color:Theme.global.fontColor,
			font:Theme.menu.rowFont,
			minimumFontSize:Theme.menu.rowFont.fontSize,
			paddingLeft:10,
			paddingRight:10,
			hintText:'',
			value:creditcard.get('cardno'),
			// keyboard stuff
			keyboardType:Titanium.UI.KEYBOARD_NUMBER_PAD,
			returnKeyType:Ti.UI.RETURNKEY_NEXT,
			autocapitalization:false,
			autocorrect:false,
			verticalAlign: Ti.UI.TEXT_VERTICAL_ALIGNMENT_CENTER,
			maxLength:19,
			suppressChange : false
		});
		form.fields.number={
			getValue: function(){
				var cardNo = cardNumber.getValue() || ''; 
				return cardNo.replace(/[^0-9]/g,'');
			}
		};
		cardNumberRow.addControl(cardNumber);
		var keyDel = false;
		cardNumber.addEventListener('keypressed', function(e){
			if (e.keyCode == KeyEvent.KEYCODE_DEL ||
				e.keyCode == KeyEvent.KEYCODE_FORWARD_DEL)
                keyDel = true;
		});
		
		cardNumber.addEventListener('change', function(e){
			/*
			if( ! e.source.suppressChange )
			{
				var value = e.value;
				if(value.length < 20)
				{
					value = value.replace(/ /g, "");
					var index = 0;
					var newValue = '';
					do{
						newValue += value.substr(index, 4) + " ";
						index+=4;
					}while(index < value.length);
					
					newValue = newValue.trim();
					if(e.value !== newValue) {
						e.source.suppressChange = true;
					
						e.source.value = newValue;
						var len = e.source.value.length;
	        			e.source.setSelection(len,len);
					}
				}
				
				if(cardNumber.getValue().length == 19)
				{
					monthEdit.focus();
				}
			}
			else {
				e.source.suppressChange = false;
			}
			*/
			if(cardNumber.getValue().length == 19)
			{
				monthEdit.focus();
			}
		});
		view.add(cardNumberRow);

   		/*
   		 * ExpiryDate
   		 */
		var expRow = new FormCustomRow({
   			label:'Expiry',
			prevField:nameRow,
   			firstItem:false,
   			layout:'horizontal'
	    });
	    //creditcard.get('expmonth'),
	    //returnKeyType:Ti.UI.RETURNKEY_DONE,
	    monthEdit = Ti.UI.createTextField({
			width:50,
			height:38,
			backgroundColor:Theme.form.inputBGColor,
			borderRadius:10,
			color:Theme.global.fontColor,
			font:Theme.menu.rowFont,
			paddingLeft:10,
			paddingRight:10,
			hintText:'MM',
			value:'',
			maxLength:2,
			// keyboard stuff
			keyboardType:Titanium.UI.KEYBOARD_NUMBER_PAD,
			returnKeyType:Ti.UI.RETURNKEY_NEXT,
		});
		form.fields.expmonth={
			getValue: function(){ return parseInt(monthEdit.getValue().replace(/[^0-9]/g,'') || '0');}
		};
		var lastMonth = '';
		monthEdit.addEventListener('change', function(e){
			if(e.value && e.value.length > 0){
				var month = parseInt(e.value);

				if(!(month > 0 && month <= 12)) {
					monthEdit.setValue(lastMonth);
					monthEdit.setSelection(lastMonth.length, lastMonth.length);
				}
				else
					lastMonth = e.value;
			}
			else
				lastMonth = ''; 
			
			if(monthEdit.getValue()> 1)	
				yearEdit.focus();
		});
		expRow.addControl(monthEdit);
		
		var lblSeparator = Ti.UI.createLabel({
			left:10,
			width:10,
			height:38,
		    color:Theme.global.fontColor,
			font:Theme.menu.rowFont,
		    text:'/'
		});
		expRow.addControl(lblSeparator);
		
		yearEdit = Ti.UI.createTextField({
			left:10,
			width:50,
			height:38,
			backgroundColor:Theme.form.inputBGColor,
			borderRadius:10,
			color:Theme.global.fontColor,
			font:Theme.menu.rowFont,
			paddingLeft:10,
			paddingRight:10,
			hintText:'YY',
			value:'',
			maxLength:2,
			// keyboard stuff
			keyboardType:Titanium.UI.KEYBOARD_NUMBER_PAD,
			returnKeyType:Ti.UI.RETURNKEY_NEXT,
		});
		form.fields.expyear={
			getValue: function(){
				return (new Date()).getFullYear().toString().substring(0, 2) + (yearEdit.getValue().replace(/[^0-9]/g,'') || '');
			}
		};
		yearEdit.addEventListener('change', function(){
			if(yearEdit.getValue().length == 2)	
				cvvEdit.focus();
		});
		expRow.addControl(yearEdit);
		
		cvvEdit = Ti.UI.createTextField({
			left:10,
			width:60,
			height:38,
			backgroundColor:Theme.form.inputBGColor,
			borderRadius:10,
			color:Theme.global.fontColor,
			font:Theme.menu.rowFont,
			paddingLeft:10,
			paddingRight:10,
			hintText:'CVV',
			value:'',
			maxLength:4,
			// keyboard stuff
			keyboardType:Titanium.UI.KEYBOARD_NUMBER_PAD,
			returnKeyType:Ti.UI.RETURNKEY_DONE,
		});
		form.fields.cvv={
			getValue: function(){ return cvvEdit.getValue();}
		};
		cvvEdit.addEventListener('change', function(){
			//if(cvvEdit.getValue().length == 4)	
				
		});
		expRow.addControl(cvvEdit);
		
   		view.add(expRow);
   		
   		if(showSurname)
		{
		    /*
		     * Owner's name
		     */
		    surnameRow = new FormTextfieldRow({
	   			label:'Surname',
	   			hint:'Family name',
				autocapitalization:true,
				autocorrect:true,
				returnKeyType:Ti.UI.RETURNKEY_NEXT,
				value:creditcard.get('surname'),
		    });
			form.fields.surname = {
				getValue: function(){surnameRow.doBlur(); return surnameRow.value;}
			};
			view.add(surnameRow);
		}
		
		if(showFirstname)
		{
		    /*
		     * Owner's name
		     */
		    firstnameRow = new FormTextfieldRow({
	   			label:'Firstname',
	   			hint:'First name',
				autocapitalization:true,
				autocorrect:true,
				returnKeyType:Ti.UI.RETURNKEY_NEXT,
				value:creditcard.get('firstname'),
		    });
			form.fields.firstname = {
				getValue: function(){firstnameRow.doBlur(); return firstnameRow.value;}
			};
			view.add(firstnameRow);
		}
		
   		/*
   		 * Address validation
   		 */
		var addressRow = new FormCustomRow({
   			label:'Address',
			prevField:nameRow,
   			firstItem:false,
   			layout:'horizontal'
	    });
	    var lines = 1;
	    //creditcard.get('expmonth'),
	    //returnKeyType:Ti.UI.RETURNKEY_DONE,
	    if(showStreet) {
		    streetEdit = Ti.UI.createTextField({
				width:'100%',
				bottom : 15,
				height:38,
				backgroundColor:Theme.form.inputBGColor,
				borderRadius:10,
				color:Theme.global.fontColor,
				font:Theme.menu.rowFont,
				paddingLeft:10,
				paddingRight:10,
				hintText:'Street',
				maxLength:100,
				value:creditcard.get('street'),
				// keyboard stuff
				keyboardType:Titanium.UI.KEYBOARD_DEFAULT,
				returnKeyType:Ti.UI.RETURNKEY_NEXT,
			});
			form.fields.street={
				getValue: function(){ return streetEdit.getValue();}
			};
			lines++;
			addressRow.addControl(streetEdit);
		}
		var controls = 0;
		if(showHouseNo) {
		    houseNoEdit = Ti.UI.createTextField({
				width:'50%',
				bottom : 15,
				height:38,
				backgroundColor:Theme.form.inputBGColor,
				borderRadius:10,
				color:Theme.global.fontColor,
				font:Theme.menu.rowFont,
				paddingLeft:10,
				paddingRight:10,
				hintText:'House No',
				maxLength:20,
				value:creditcard.get('houseno'),
				// keyboard stuff
				keyboardType:Titanium.UI.KEYBOARD_DEFAULT,
				returnKeyType:Ti.UI.RETURNKEY_NEXT,
			});
			form.fields.houseno={
				getValue: function(){ return houseNoEdit.getValue();}
			};
			controls = (controls + 1) % 2;
			lines += controls;
			addressRow.addControl(houseNoEdit);
		}

		if(showPostcode) {
			
			postCodeEdit = Ti.UI.createTextField({
				left:controls == 1 ? '2%' : 0,
				bottom : 15,
				width:controls == 1 ? '48%' : '50%',
				height:38,
				backgroundColor:Theme.form.inputBGColor,
				borderRadius:10,
				color:Theme.global.fontColor,
				font:Theme.menu.rowFont,
				paddingLeft:10,
				paddingRight:10,
				hintText:'Post Code',
				value:creditcard.get('postcode'),
				maxLength:10,
				// keyboard stuff
				keyboardType:Titanium.UI.KEYBOARD_DEFAULT,
				returnKeyType:Ti.UI.RETURNKEY_NEXT,
			});
			form.fields.postcode={
				getValue: function(){
					return postCodeEdit.getValue();
				}
			};
			controls = (controls + 1) % 2;
			lines += controls;
			addressRow.addControl(postCodeEdit);
		}

		if(showCity) {
			cityEdit = Ti.UI.createTextField({
				left:controls == 1 ? '2%' : 0,
				bottom : 15,
				width:controls == 1 ? '48%' : '50%',
				height:38,
				backgroundColor:Theme.form.inputBGColor,
				borderRadius:10,
				color:Theme.global.fontColor,
				font:Theme.menu.rowFont,
				paddingLeft:10,
				paddingRight:10,
				hintText:'City',
				value:creditcard.get('city'),
				maxLength:40,
				// keyboard stuff
				keyboardType:Titanium.UI.KEYBOARD_DEFAULT,
				returnKeyType:Ti.UI.RETURNKEY_NEXT,
			});
			form.fields.city={
				getValue: function(){
					return cityEdit.getValue();
				}
			};
			controls = (controls + 1) % 2;
			lines += controls;
			addressRow.addControl(cityEdit);
		}

		var country = '';
		var states = [];
		if(showCountry) {

			countryEdit = Ti.UI.createTextField({
				left:controls == 1 ? '2%' : 0,
				bottom : 15,
				width:controls == 1 ? '48%' : '50%',
				height:38,
				backgroundColor:Theme.form.inputBGColor,
				borderRadius:10,
				color:Theme.global.fontColor,
				font:Theme.menu.rowFont,
				paddingLeft:10,
				paddingRight:10,
				hintText:'Country',
				value:creditcard.get('country') in ICABBI.Config.card_config.fields.country.values ? ICABBI.Config.card_config.fields.country.values[creditcard.get('country')] : '',
				maxLength:100,
				editable : false,
				touchEnabled: true
			});
			var countries = [];
			for(var countryId in ICABBI.Config.card_config.fields.country.values){
				countries.push({
					title: ICABBI.Config.card_config.fields.country.values[countryId],
					id: countryId
				});
			}
			
			var chooseCountry = function() {
				ICABBI.navController.showPickerModal({
				    title:'Country',
				    data: countries,
				    //simple : true,
				    click : function(e){
				    	states = [];
				    	stateEdit.setValue('');
				    	if(e.selected) {
				    		countryEdit.setValue(ICABBI.Config.card_config.fields.country.values[e.selected]);
				    		country = e.selected;
				    		if(ICABBI.Config.card_config.fields.country.values[country]){
				    			
				    			for(var stateId in ICABBI.Config.card_config.fields.state.values[country]){
				    				Ti.API.error(stateId + '-' + ICABBI.Config.card_config.fields.state.values[country][stateId]);
									states.push({
										title: ICABBI.Config.card_config.fields.state.values[country][stateId],
										id: stateId
									});
								}
								if(states.length > 0)
									stateEdit.visible = true;
				    		}
				    	}
				    	else {
				    		stateEdit.visible = false;
				    		countryEdit.setValue('');
				    		country = '';
				    	}
				    }
				});				
			};		
		    
			//countryEdit.addEventListener('click', chooseCountry);
			countryEdit.addEventListener('touchend', chooseCountry);
			
			form.fields.country={
				getValue: function(){
					return country;
				}
			};
			controls = (controls + 1) % 2;
			lines += controls;
			addressRow.addControl(countryEdit);
		}
		
		
		var state = '';
		if(showCountry) {

			stateEdit = Ti.UI.createTextField({
				left:controls == 1 ? '2%' : 0,
				bottom : 15,
				width:controls == 1 ? '48%' : '50%',
				height:38,
				backgroundColor:Theme.form.inputBGColor,
				borderRadius:10,
				color:Theme.global.fontColor,
				font:Theme.menu.rowFont,
				paddingLeft:10,
				paddingRight:10,
				hintText:'State',
				value:creditcard.get('state') in ICABBI.Config.card_config.fields.state.values ? ICABBI.Config.card_config.fields.state.values[creditcard.get('country')][creditcard.get('state')] : '',
				maxLength:100,
				// keyboard stuff
				editable : false,
				touchEnabled: true,
				visible : false
			});
			
			
			var chooseState = function() {
				if(states && states.length > 0)
					ICABBI.navController.showPickerModal({
					    title:'State',
					    data: states,
					    //simple : true,
					    click : function(e){
					    	if(e.selected) {
					    		stateEdit.setValue(ICABBI.Config.card_config.fields.state.values[country][e.selected]);
					    		state = e.selected;
					    	}
					    	else {
					    		stateEdit.setValue('');
					    		state = '';
					    	}
					    }
					});				
			};		
		    
			stateEdit.addEventListener('touchend', chooseState);
			form.fields.state={
				getValue: function(){
					return state;
				}
			};
			controls = (controls + 1) % 2;
			lines += controls;
			addressRow.addControl(stateEdit);
		}
		
		if(showStreet || showHouseNo || showPostcode || showCity || showCountry)
		{
			addressRow.setHeight(43 * lines);
   			view.add(addressRow);
   		}

		/*
		 * Control button
		 */
		// var filler = Ti.UI.createView({
	        // bottom:15,
   			// height:Ti.UI.FILL,
   			// left:0,
   			// right:0,
   		// });
   		// view.add(filler);
// 
		// var controlBar = Titanium.UI.createView({
	        // height:Ti.UI.SIZE,
	        // width:Ti.UI.SIZE,
	        // layout:'horizontal',
	    // });
	    // filler.add(controlBar);
// 
		// var btnSubmit = new ActionButton({title:'SAVE',icon:'/images/buttons/icon_save.png'});
		// controlBar.add(btnSubmit);
		
		var btnSubmit = Ti.UI.createButton({
			top:10,
			right:16,
			width:110,
			height:42,
			backgroundColor:Theme.global.buttonBGColor,
			borderColor:Theme.global.highlightColor,
			backgroundFocusedColor:Theme.global.highlightColor,
			backgroundSelectedColor:Theme.global.highlightColor,
			borderWidth:1,
			borderRadius:8,
			color:Theme.global.fontColor,
			font:Theme.form.buttonFont,
			title: 'Save',
	  	    style:Ti.UI.iPhone.SystemButtonStyle.PLAIN,
		});
		Theme.animateButton(btnSubmit);
		view.add(btnSubmit);
		
		btnSubmit.addEventListener('click',function(e){
			nameRow.doBlur();
			if(showCardHolder)
				ownerNameRow.doBlur();
			cardNumber.blur();
			if(showAccountCard)
				accountCardRow.doBlur();
			monthEdit.blur();
			yearEdit.blur();
			cvvEdit.blur();
			if(showSurname)
				surnameRow.doBlur();
			if(showFirstname)
				firstnameRow.doBlur();
			if(showStreet)
				streetEdit.blur();
			if(showHouseNo)
				houseNoEdit.blur();
			if(showPostcode)
				postCodeEdit.blur();
			if(showCity)
				cityEdit.blur();
			if(showCountry)
				countryEdit.blur();
									
			var now = new Date;
			var error='';

			var fields={
				cardno: form.fields.number.getValue(),
				nickname: form.fields.name.getValue(),
				ownername: showCardHolder ? form.fields.ownername.getValue() : null,
				accountcard_id: showCardHolder ? form.fields.accountcard_id.getValue() : null,
				expmonth: form.fields.expmonth.getValue(),
				expyear: form.fields.expyear.getValue(),
				cvv: form.fields.cvv.getValue(),
				surname: showSurname ? form.fields.surname.getValue() : null,
				firstname: showFirstname ? form.fields.firstname.getValue() : null,
				street: showStreet ? form.fields.street.getValue() : null,
				houseno: showHouseNo ? form.fields.houseno.getValue() : null,
				postcode: showPostcode ? form.fields.postcode.getValue() : null,
				city: showCity ? form.fields.city.getValue() : null,
				country: showCountry ? form.fields.country.getValue() : null,
				state: showCountry ? form.fields.state.getValue() : null,
			};
			
			if(!fields.cardno.length || !luhn10(fields.cardno)){
				error += '- Invalid Credit Card Number!\n';
			}
			if(!fields.nickname.length){
				error += '- Name is required!\n';
			}
			if(showCardHolder && requiredCardHolder && !fields.ownername.length){
				error += '- Full Name is required!\n';
			}
			if(showAccountCard && requiredAccountCard && !fields.accountcard_id){
				error += '- Card Type is required!\n';
			}
			if(!fields.cvv.length){
				error += '- CVV is required!\n';
			}			
			if(fields.expyear==now.getFullYear() && (fields.expmonth-1)<now.getMonth()){
				error += '- Expiry Date must be in future!\n';
			}
			
			if(fields.expyear<now.getFullYear()){
				error += '- Invalid Expiry Year!\n';
			}
			if(fields.expmonth > 12 || fields.expmonth < 1){
				error += '- Invalid Expiry Month!\n';
			}
			
			if(showSurname && requiredSurname && !fields.surname) {
				error += '- Please provide a family name.!\n';
			}
			if(showFirstname && requiredFirstname && !fields.firstname) {
				error += '- Please provide a first name.!\n';
			}
			if(showStreet && requiredStreet && !fields.street) {
				error += '- Please provide a surname.!\n';
			}
			
			if((showHouseNo && requiredHouseNo && !fields.houseno) ||
					(showHouseNo && fields.postcode && !fields.houseno)) {
				error += '- Please provide a house number.!\n';
			}
			
			if((showPostcode && requiredPostcode && !fields.postcode) ||
					(showPostcode && fields.houseno && !fields.postcode)) {
				error += '- Please provide a post code.!\n';
			}
			
			if(showCity && requiredCity && !fields.city) {
				error += '- Please provide a city.!\n';
			}
			if(showCountry && requiredCountry && !fields.country) {
				error += '- Please provide a country.!\n';
			}
			
			if(showCountry && requiredState && ICABBI.Config.card_config.fields.state.values[fields.country] && !fields.state) {
				error += '- Please provide a state.!\n';
			}
			
			fields.nickname += ' ('+fields.cardno.replace(/^(\d{12})/g,'*')+')';
			
			if(error){
				ICABBI.navController.showError({
				    title:'Error!',
				    message:error,
				    close:true
				});
				return;
			};
			
			if(fields.expyear)
				fields.expyear = fields.expyear.toString().substring(2, 4);
			
			$$.ActivityIndicator.start('Adding Your Card');
			ICABBI.Controller.registerCreditCard(
				fields,
				{
					success: function(e){
						$$.ActivityIndicator.stop();
						
						if(booking){
							booking.set({creditcard_id: e.get('id')});
							booking.setPayment(booking.PAYMENT_MODES.CREDITCARD);
						}
						
						ICABBI.navController.showYesNo({
						    title:'Registration Successful!',
						    message:'Your new account has been successfully registered. Would you like to add another card?',
						    click : function(e){
						    	if(e.index == 1) {
						    		creditcard = new CreditCard.Model();
									nameRow.putValue('');
									if(ownerNameRow)
										ownerNameRow.putValue('');
									cardNumber.setValue('');
									accountcard_id = 0;
									if(accountCardRow)
										accountCardRow.putValue('');
									monthEdit.setValue('');
									yearEdit.setValue('');
									cvvEdit.setValue('');
									if(surnameRow)
										surnameRow.putValue('');
									if(firstnameRow)
										firstnameRow.putValue('');
									if(streetEdit)
										streetEdit.setValue('');
									if(houseNoEdit)
										houseNoEdit.setValue('');
									if(postCodeEdit)
										postCodeEdit.setValue('');
									if(cityEdit)
										cityEdit.setValue('');
									country = '';
									if(countryEdit)
										countryEdit.setValue('');
									state = '';
									if(stateEdit)
										stateEdit.setValue('');
								}
								else
								{
									options.parent.window.__retval = true;
									ICABBI.navController.close(options.parent.window);
								}
						    }
						});
					}, 
					
					error: function(e){
						$$.ActivityIndicator.stop();
						ICABBI.navController.showError({
						    title:'Error!',
						    message:e.message,
						    close:true
						});
						return;
					}
			});
		});

/////////////
	var doDefaults = function() {
		if(!creditcard.isNew()){
			
		} else {
			
		}
	};
////////////	
		
		options.parent.window.addEventListener('open',doDefaults);

		return view;
	}; 

	
	var self = createForm();
	return self;
};

module.exports = CreditCardRegistrationView;
