/**
 * @author Sonal
 * DateTimeView.js
 */
var ICABBI = require('/core/icabbi');
var tools = require('/lib/tools'),
	_ = require('/lib/lodash')._ ,
	mds = require('/model/MemoryDataStore'),
	$$ = require('/core/ThemeManager');

var Theme = require('/ui/Theme');
var ActionButton = require('/ui/common/ActionButton');

//DateTimeView Component Constructor
var DateTimeView = function(options) {
	options = options || {};
	var datechanged=timechanged=false;
	var objDateTime = options.data.objDateTime || {date:false,time:false};

	var createDateTimePicker = function(objDateTime){
		var view = Titanium.UI.createView({
			top:10,
	        left:0,
	        right:0,
	        height:Ti.UI.SIZE,
			layout:'vertical'
	    });
		
		if (Ti.Android) {
			// Set picker data
			var picker_date = Ti.UI.createPicker({
			  type:Ti.UI.PICKER_TYPE_DATE,
			  selectionIndicator:true,
			  minDate:new Date(),
			  maxDate:new Date(2020,12,31),
			  value:new Date(),
			  locale:Titanium.Locale,
			});
			
			var picker_time = Ti.UI.createPicker({
			  type:Ti.UI.PICKER_TYPE_TIME,
			  selectionIndicator:true,
			  value:new Date(),
			  locale:Titanium.Locale,
			  format24 : true,
			});
			
			// turn on the selection indicator (off by default)
			picker_date.selectionIndicator = true;
			picker_time.selectionIndicator = true;
		
			picker_date.addEventListener('change',function(e){
				datechanged=true;
				//e.value.toLocaleString() = Wed Jan 23 2013 00:00:00 GMT+0530 (IST)
				objDateTime.date=e.value;
			    if(!timechanged){
			    	var now = new Date;
					objDateTime.time=now;
			    }
			});
			picker_time.addEventListener('change',function(e){
				timechanged=true;
				var time = e.value,
					now = new Date;
				// objDateTime.date>today then dont care abt time
				var selDate = new Date(objDateTime.date),
					today = new Date;
				selDate.setHours(0,0,0,0);
				today.setHours(0,0,0,0);
				if((selDate-today<=0) &&  time<now) time=now;
				//Wed Jan 23 2013 08:49:38 GMT+0530 (IST)
				objDateTime.time=time;
			});
			view.add(picker_date);
			view.add(picker_time);
		} else {
			// Set picker data
			var picker_date = Ti.UI.createPicker({
			  type:Ti.UI.PICKER_TYPE_DATE_AND_TIME,
			  selectionIndicator:true,
			  minDate:new Date(),
			  maxDate:new Date(2020,12,31),
			  value:objDateTime.time,
			});
			picker_date.addEventListener('change',function(e){
				datechanged=true;
				timechanged=true;
				//e.value.toLocaleString() = Wed Jan 23 2013 00:00:00 GMT+0530 (IST)
				objDateTime.date=e.value;
				objDateTime.time=e.value;
			});
			view.add(picker_date);
		}
		
		return view;
	};

	var createForm = function(){
		//create object instance, a parasitic subclass of Observable
		var view = Titanium.UI.createView({
	        height:Ti.UI.SIZE,
			layout:'vertical'
	    });

		var picker_view = createDateTimePicker(objDateTime);
		view.add(picker_view);

		var controlBar = Titanium.UI.createView({
			top:10,
	        bottom:15,
	        height:70,
	        width:Ti.UI.SIZE,
	        layout:'horizontal',
	    });
	    view.add(controlBar);
		
		var btnNow = new ActionButton({title:'NOW',icon:'/images/buttons/icon_clock.png'});
		controlBar.add(btnNow);

		var btnSubmit = new ActionButton({title:'SUBMIT',icon:'/images/buttons/icon_save.png'});
		btnSubmit.left = 8;
		controlBar.add(btnSubmit);

		var _submitForm = function(now){
			var form = ICABBI.getDateTime(now, objDateTime);
			options.parent.window.__retval = form;
			ICABBI.navController.close(options.parent.window);
		};
		
		btnNow.addEventListener('click',function(e){
			_submitForm(true);
		});
		
		btnSubmit.addEventListener('click',function(e){
			_submitForm();
		});

		return view;
	}; 
	
	var self = createForm();
	return self;
};

module.exports = DateTimeView;
