var tools = require('/lib/tools'),
	_ = require('/lib/lodash')._ ,
	Geolocation = require('/lib/Geolocation'),
	$$ = require('/core/ThemeManager');

var Theme = require('/ui/Theme');
var FormTextfieldRow = require('/ui/common/FormTextfieldRow');
var FormCheckboxRow = require('/ui/common/FormCheckboxRow');

//EditAddressView Component Constructor
var EditAddressView = function(options) {
	options = options || {};

	var address = options.data.address;
	
	var tableView;
	var buildingRow, streetRow;
	
	var fillTable = function(addresses) {
		Ti.API.info('fill address');
		var tableData = [];
		if(addresses) {
			_.each(addresses, function(address){
				tableData.push(createRecentAddressRow({
				  	title: address.formatted||'',
				  	subtitle:address.area||'',
				  	data:address
				}));
			});
		}
		tableView.setData([]);
		tableView.setData(tableData);
	};
	
	var createRecentAddressRow = function(data) {
		var row = Ti.UI.createTableViewRow({
			height:48,
			width:'100%',
			backgroundColor:'transparent',
			backgroundFocusedColor:Theme.global.altBGColor,
			backgroundSelectedColor:Theme.global.altBGColor,
			focusable :true,
			selectionStyle:Ti.Android?undefined:Ti.UI.iPhone.TableViewCellSelectionStyle.GRAY,
		});
		row.data = data.data;
		row.className = 'datarow';
		row.clickName = 'row';
		
		var title = Ti.UI.createLabel({
			color:Theme.global.fontColor,
			left:10,
			right:36,
			top:3,
			height:23,
			text:data.title,
			touchEnabled:false,
		    wordWrap:false,
		    ellipsize:true,
			width: Ti.UI.FILL,
			verticalAlign:'bottom',
			minimumFontSize:Theme.address.addressFont.fontSize,
			font:Theme.address.addressFont,
		});
		row.add(title);
		
		var subtitle = Ti.UI.createLabel({
			color:'#999999',
			left:10,
			right:36,
			top:23,
			height:17,
			text:data.subtitle,
			touchEnabled:false,
		    wordWrap:false,
		    ellipsize:true,
			font:Theme.address.subaddressFont,
		});
		row.add(subtitle);
		
		
		var arrowIcon = Ti.UI.createImageView({
		    height:11,
		    width:11,
		    right: 7,
		    top:11,
		    image:'/images/map/arrow.png',
		    touchEnabled:false,
		});
		row.add(arrowIcon);
		
		var rowTopLine = Ti.UI.createView({
			left:0,
			right:0,
			top:0,
			height:'1px',
			backgroundColor:'magenta',
			touchEnabled:false,
		});
		//row.add(rowTopLine);
		var rowBottomLine = Ti.UI.createView({
			left:0,
			right:0,
			bottom:0,
			height:'1px',
			backgroundColor:'#cacaca',
			touchEnabled:false,
		});
		row.add(rowBottomLine);
	
		return row;
		  
		  
		  		
	};

	var createForm = function(){
		//create object instance, a parasitic subclass of Observable
		var view = Titanium.UI.createView({
			top:0,
			left:0,
			right:0,
	        height:Ti.UI.SIZE,
			layout:'vertical'
	    });

		var arrAddress = address.get('formatted').split(', ');
		var building = arrAddress.shift();
		var street = arrAddress.join(', ');
		
	    buildingRow = new FormTextfieldRow({
   			label:'Building',
			autocapitalization:true,
			autocorrect:true,
			clearButtonMode:Ti.UI.INPUT_BUTTONMODE_ONFOCUS,
			returnKeyType:Ti.UI.RETURNKEY_NEXT,
			value:building,
	    });
	    view.add(buildingRow);

	    streetRow = new FormTextfieldRow({
   			label:'Street',
			autocapitalization:true,
			autocorrect:true,
			returnKeyType:Ti.UI.RETURNKEY_DONE,
			value:street,
			prevField:buildingRow,
	    });
	    view.add(streetRow);

		// previously used addresses
		var section = Ti.UI.createView({
			left:0,
			right:0,
			bottom:10,
			top:15,
			height: 46,
			backgroundColor:Theme.nav.backgroundColor,
		});
		
		var sectionLineTop = Ti.UI.createView({
			left:0,
			right:0,
			top:0,
			height:1,
			backgroundColor:Theme.nav.seperatorColor,
		});
		section.add(sectionLineTop);

		var sectionLineTopWhite = Ti.UI.createView({
			left:0,
			right:0,
			top:1,
			height:1,
			backgroundColor:Theme.global.bgColor,
		});
		section.add(sectionLineTopWhite);
		
		var sectionLineBottomWhite = Ti.UI.createView({
			left:0,
			right:0,
			bottom:1,
			height:1,
			backgroundColor:Theme.global.bgColor,
		});
		section.add(sectionLineBottomWhite);

		var sectionLineBottom = Ti.UI.createView({
			left:0,
			right:0,
			bottom:0,
			height:1,
			backgroundColor:Theme.nav.seperatorColor,
		});
		section.add(sectionLineBottom);

	
		var previouslyLabel = Ti.UI.createLabel({
			text:'Previously used addresses',
			left:12,
			right:12,
			height:42,
			color:Theme.section.titleColor,
			font:Theme.section.titleFont,
			textAlign:'left',
		});
		section.add(previouslyLabel);
   		view.add(section);
   				
		var tableData = [];
		
		tableView = Ti.UI.createTableView({
		  backgroundColor:'white',
		  data:tableData,
		  separatorStyle:Ti.UI.Android ? undefined : Ti.UI.iPhone.TableViewSeparatorStyle.NONE,
		  allowsSelection:Ti.Android,
		  top: 0,
		  bottom: 1,
		  left: 0,
		  right:0,
		  separatorColor: '#ffffff',
		});
		
		tableView.addEventListener('click', function(e) {
			address.set(e.row.data);
			ICABBI.navController.close(options.parent.window);
		});
		view.add(tableView);

		Geolocation.searchAddressBook('', false, options.data.location, function(addresses){
			fillTable(addresses);
		});
				
		return view;
	}; 
	
	var self = createForm();
	
	self.onSubmit = function() {
		var oldFormatted = address.get('formatted'),
			newFormatted = (buildingRow.value && buildingRow.value.length > 0 ? buildingRow.value + ', ' : '') + streetRow.value;

		if(oldFormatted != newFormatted) {
			address.set({
				id: '',
				area: '',
				formatted: newFormatted
			});
		}
		else {
			address.set('area', '');
		}
		return false;
	};
	
	return self;
};

module.exports = EditAddressView;
