/**
 * @author Sonal
 * FavouritesView.js
 */
var ICABBI = require('/core/icabbi');
var _ = require('/lib/lodash')._ ,
	$$ = require('/core/ThemeManager');


//FavouritesView Component Constructor
var FavouritesView = function(options) {
	//create object instance, a parasitic subclass of Observable
	var defaults = {
			title:'My Favourites',
		};
	options = _.extend(defaults,options.data||{});
	
	var platformWidth = ICABBI.width;
	var headerHeight = 30;
	var table;

	var createLabel = function(options){
		var label = Ti.UI.createLabel($$.styl('label',{
		    height:Ti.UI.FILL,
		    left:0,
		    width:80,//Ti.UI.FILL
	  	    font:{fontSize:14,fontWeight:'bold'},
		},options)
		);
		return label;
	};
	
	var createNRFRow = function(){
		var row = Titanium.UI.createTableViewRow({
			top:4,
	        height: Ti.UI.SIZE,//40,
	        width: Ti.UI.FILL,
			backgroundColor:'transparent',
			selectionStyle:Ti.Android?undefined:Ti.UI.iPhone.TableViewCellSelectionStyle.GRAY,
			__booking: 'nrf',
			visible: false,
	    });
		var cell = Titanium.UI.createView({
			height: Ti.UI.SIZE,
			width: Ti.UI.FILL,
			top:0,
		});
		var lblContent = createLabel({
			text:'No Favourite Bookings!',
//		 	color:'#000',
			left:0,
			height:Ti.UI.SIZE, 
			width:Ti.UI.FILL,
			textAlign: Ti.UI.TEXT_ALIGNMENT_CENTER,
			font: {
				fontSize:14,
				// fontWeight:'bold'
			}
		});
		cell.add(lblContent);
		row.add(cell);
	
		return row;
	};

	var createTableViewRow = function(booking){
	    var createCell = function(data,options){
			var cell = Titanium.UI.createView(_.extend({
		        height: Ti.UI.SIZE,
		        width: '40%',
				layout:'vertical'
		    },options));
			cell.add(createLabel({
			 	text:data.heading+':',
			 	color:'#ccc',
			 	left:0,
			 	width:Ti.UI.FILL,
			 	height:Ti.UI.SIZE, 
		  	    font:{fontSize:12,fontWeight:'bold'}
		 	}));
		 	var lblContent = createLabel({
			 	text:data.content,
			 	left:0,
			 	height:Ti.UI.SIZE, 
			 	width:Ti.UI.FILL,
		  	    font: data.font || {fontSize:14,fontWeight:'bold'}
		 	});
			cell.add(lblContent);
		    return {cell:cell,label:lblContent};
	 	};
		var row = Titanium.UI.createTableViewRow({
			top:4,
	        height: Ti.UI.SIZE,//40,
	        width: Ti.UI.FILL,
			backgroundColor:'transparent',
			selectionStyle:Ti.Android?undefined:Ti.UI.iPhone.TableViewCellSelectionStyle.GRAY,
			//rowData: {},
			__booking: booking.get('perma_id')
	    });

		var data = {heading:'',content:''};
		
		//column1 
		data.heading='From';
		data.content=booking.formatAddressDisplay('address');
		var col1 = createCell(data,{left:5}); 
		row.add(col1.cell);
		
		(function(col,booking){
		 	booking.on('change:address',function(){
		 		col.label.setText(booking.formatAddressDisplay('address'));
		 	});
		})(col1,booking);

		//column2 
		data.heading='To';
		data.content=booking.formatAddressDisplay('destination');
		var col2 = createCell(data,{right:15}); 
		row.add(col2.cell);
		(function(col,booking){
		 	booking.on('change:destination',function(){
		 		col.label.setText(booking.formatAddressDisplay('destination'));
		 	});
		})(col2,booking);
		
		var btnDelete = Ti.UI.createButton({
			title: 'X',
			color:'#000',
			right: 0,
			height: Ti.UI.SIZE,
			width: Ti.UI.SIZE,
			backgroundColor : '#fff000',
			borderColor:'#fff000',
			borderRadius: 10,
			borderWidth: 1
		});
		row.add(btnDelete);
		btnDelete.addEventListener('click',function(e){
			booking.set('favourite',false);
			booking.manageFavourite();
			//ICABBI.Controller.updateBooking(booking);
		});
	

	    return row;
	};

	var createTableView = function(title){
		var table = Ti.UI.createTableView({
			//backgroundImage:'/images/table_bg.png',
			separatorStyle:Ti.Android ? 0 : Ti.UI.iPhone.TableViewSeparatorStyle.NONE,
			allowsSelection:Ti.Android,
			top:10,
			left:0,
			width: Ti.UI.FILL,
			height: Ti.UI.SIZE,
			//separatorColor: '#FFFFFFF',
		   // backgroundColor: '#FFFFFF'
		});
		return table;
	};

	var create = function(){
		var content = Ti.UI.createView({
			backgroundColor: 'transparent',
		    borderColor:'#CCC',
		    borderWidth:0,
			width:'98%',
			//height: '99%',
			layout:'vertical'
		});
		var scroll = Ti.UI.createScrollView({
		    contentWidth:'auto',//Ti.UI.FILL,
		    contentHeight:'auto',//Ti.UI.FILL,
		    showVerticalScrollIndicator:false,
		    showHorizontalScrollIndicator:false,
		    top:0,
		    left:0,
		    scrollType: 'vertical',
		    layout:'vertical'
		});
		
		table = createTableView({
			allowsSelection:Ti.Android,
		});
		scroll.add(table);
		
		content.add(scroll);
		return content;
	};

	var self = create();
	
	(function(){
		var	bookingsActive = ICABBI.Controller.getUser().bookings, 
			bookingsUpcoming = ICABBI.Controller.getUser().bookingsUpcoming, 
			bookingsPrevious = ICABBI.Controller.getUser().bookingsPrevious; 

		var appendRow = function(booking){
			var row = createTableViewRow(booking);
			table.appendRow(row);
		};

		var favourites = _.union(
							bookingsActive.fetchFavourites(),
							bookingsUpcoming.fetchFavourites(),
							bookingsPrevious.fetchFavourites()
						);
		if(!favourites.length){
			table.appendRow(createNRFRow());
			return;
		}
		//on dialog load..get all the booking from sqlite and get addresses from 3booking bucket
		_.each(favourites,function(booking){ 
				appendRow(booking);
				booking.on('change:favourite',function(){
					if(!booking.get('favourite')){
						var id = booking.get('perma_id');
						var section = table.data[0];
						for(var i=0;i<section.rowCount;i++){
							var row = section.rows[i];
							if(row.__booking==id){
								table.deleteRow(i);
								break;
							}
						}
					}
				})
		}); //each
		$$.ActivityIndicator.stop();



	})();

	return self;
};

module.exports = FavouritesView;
