var Theme = require('/ui/Theme');

var FormPickerRow = function(args){
	args.firstItem = args.firstItem || false; 
	args.onRemoveClick = args.onRemoveClick || false;
	
	var controlHeight = 54;
	
	var self = Ti.UI.createView({
		left:8,
		right:8,
		height:controlHeight,
		putText:function(args){
			if(args.label !== undefined && args.label !== null){
				label.text = args.label;
			}
			if(args.title !== undefined && args.title !== null){
				title.text = args.title;
			}
			if(args.subTitle !== undefined && args.subTitle !== null) {
				title.top = args.subTitle.length ? 4 : undefined;
				subTitle.text = args.subTitle;
			}
		},
		addEventListener: function(ev, fn) {
			titleView.addEventListener(ev, fn);
		},
	});
	
	
	var label = Ti.UI.createLabel({
		left:0,
		height:21,
		width:45,
		color:Theme.form.labelColor,
		font:Theme.form.labelFont,
		text:args.label,
		textAlign:'right',
		touchEnabled:false,
	});
	self.add(label);
	
	var titleView = Ti.UI.createView({
		right: args.onRemoveClick ? 32 : 0,
		left:args.label.length?60:12,
		height:54,
	});
	
	var title = Ti.UI.createLabel({
		left:1,
		right:25,
		top:args.subTitle && args.subTitle.length ? 4:undefined,
		height:28,
		color:Theme.global.fontColor,
		font:Theme.menu.rowFont,
		minimumFontSize:Theme.menu.rowFont.fontSize,
		wordWrap:false,
		ellipsize:true,
		text:args.title,
		touchEnabled:false,
		width: Ti.UI.FILL,
	});
	titleView.add(title);
	
	var subTitle = Ti.UI.createLabel({
		// left:args.label.length?60:12,
		left:1,
		right:25,
		bottom:4,
		height:24,
		color:Theme.form.smallColor,
		font:Theme.form.smallFont,
		text:args.subTitle,
		touchEnabled:false,
	});
	titleView.add(subTitle);
	
	if(!args.onRemoveClick) {
		var detail = Ti.UI.createImageView({
			right:7,
			width:11,
			height:11,
			image:'/images/map/arrow.png',
			touchEnabled:false,
		});
		titleView.add(detail);
	}
	
	self.add(titleView);
	
	if(args.onRemoveClick) {
		var removeView = Ti.UI.createView({
			right: 0,
			width: 32,
			height: 54,
		});
		self.add(removeView);
		var removeImg = Ti.UI.createImageView({
			right: 5,
			width: 22,
			height: 22,
			image: '/images/buttons/icon_remove.png',
			touchEnabled: true,
		});
		removeView.add(removeImg);
		
		if(typeof(args.onRemoveClick) == 'function') {
			removeView.addEventListener('click', function(e) {
					args.onRemoveClick(e);
			});
		}
	}
	
	var rowLine = Ti.UI.createView({
		left:args.label.length?60:0,
		right:0,
		bottom:0,
		height:1,
		backgroundColor:Theme.global.listBottomLineColor,
		visible:!args.lastItem,
		touchEnabled:false,
	});
	self.add(rowLine);
	
	var rowLineTop = Ti.UI.createView({
		left:args.label.length?60:0,
		right:0,
		top:0,
		height:1,
		backgroundColor:Theme.global.listTopLineColor,
		visible:!args.firstItem,
		touchEnabled:false,
	});
	self.add(rowLineTop);
	
	Theme.animateButton(titleView);
	
	self.titleView = titleView;
	
	var highlightBGColor = Theme.makeTranslucentColor(Theme.global.highlightRowColor),
		animationOngoing = false,
		doNext = null;
	
	self.showRow = function() {
		if(Ti.Android)
		{
			if(animationOngoing) {
				doNext = 'show';
				return;
			}

			doNext = null;
			if (self.getHeight() > 0) {
				return;
			}

			animationOngoing = true;
			self.animate({
				height : controlHeight,
				duration : 300
			}, function() {
				titleView.animate({
					backgroundColor : highlightBGColor,
					duration : 2000,
					autoreverse : true
				});
				animationOngoing = false;
				if (doNext == 'hide')
					self.hideRow();
			}); 

		}
		else
		{
			self.height = controlHeight;
			setTimeout(function(){
				var animation = Titanium.UI.createAnimation();
				animation.backgroundColor = highlightBGColor;
				animation.duration = 2000;
				animation.autoreverse = true;

				titleView.animate(animation);
			}, 1000);
		}
	};
	
	self.hideRow = function() {
		
		if (Ti.Android) {
			if (animationOngoing) {
				doNext = 'hide';
				return;
			}

			doNext = null;
			if (self.getHeight() == 0) {
				return;
			}

			animationOngoing = true;
			self.animate({
				height : 0,
				duration : 300
			}, function() {
				animationOngoing = false;
				if (doNext == 'show')
					self.showRow();
			});
		} else {
			self.height = 0;
		}

	};
	
	return self;
};

module.exports = FormPickerRow;
