var _ = require('/lib/lodash')._;

var Theme = require('/ui/Theme');

var FormValidationRow = function(args){
	args.value = args.value||'';
	args.hint = args.hint||'';
	args.passwordMask = args.passwordMask||false;
	args.keyboardType = args.keyboardType||Ti.UI.KEYBOARD_DEFAULT;
	args.autocapitalization = args.autocapitalization||false;
	args.autocorrect = args.autocorrect||false;
	args.returnKeyType = args.returnKeyType||Ti.UI.RETURNKEY_DEFAULT;
	
	var self = Ti.UI.createView({
		left: 0,
		right: 0,
		top: 7,
		bottom: 0,
		value: args.value,
        height:Ti.UI.SIZE,
		layout:'vertical',
		height:Ti.UI.SIZE,
	});
	
	var rowView = Ti.UI.createView({
		left: 0,
		right: 0,
		top: 0,
		bottom: 0,
		height: 33,
	});
	
	self.add(rowView);
	
	var label = Ti.UI.createLabel({
		left: 0,
		top: 6,
		height: 21,
		width: 90,
		color: Theme.form.labelColor,
		font: Theme.form.labelFont,
		text: args.label,
		textAlign: 'right',
		touchEnabled: false,
	});
	rowView.add(label);
	
	var input;
	
	if (!args.values) {
		// text input
		input = Ti.UI.createTextField({
			top:0,
			left:100,
			right:12,
			height:33,
			backgroundColor:Theme.form.inputBGColor,
			borderRadius:10,
			color:Theme.global.fontColor,
			font:Theme.form.smallFont,
			paddingLeft:10,
			paddingRight:10,
			value:self.value,
			// keyboard stuff
			clearButtonMode:args.clearButtonMode,
			clearOnEdit:args.clearOnEdit,
			keyboardType:args.keyboardType,
			passwordMask:args.passwordMask,
			returnKeyType:args.returnKeyType,
			autocapitalization:args.autocapitalization,
			autocorrect:args.autocorrect,
		});
		rowView.add(input);

		input.addEventListener('change',function(e){
			self.value = e.value;
		});
		
		// interaction handlers
		if (args.prevField) {
			args.prevField.addEventListener('return',function(){
				input.focus();
			});
		}
		
		self.doCleanup = function() {
			var test = 0;
		};
		self.doFocus = function() {
			input.focus();
		};
		self.doBlur = function() {
			input.blur();
		};
	} else {
		// combobox input
		input = Ti.UI.createLabel({
			top:0,
			left:100,
			right:12,
			height:33,
			color:Theme.global.fontColor,
			font:Theme.menu.rowFont,
			minimumFontSize:Theme.menu.rowFont.fontSize,
			wordWrap:false,
			text:'',
			touchEnabled:false,
		});
		rowView.add(input);
		
		var detail = Ti.UI.createImageView({
			top:9,
			right:12,
			width:9,
			height:15,
			image:'/images/menu/arrow.png',
			touchEnabled:false,
		});
		rowView.add(detail);
		
		Theme.animateButton(self);
		
		self.doCleanup = function() {
			var test = 0;
		};
		self.doFocus = function() {
			var test = 0;
		};
		self.doBlur = function() {
			var test = 0;
		};
		
   		/*
		 * row values
   		 */
		var combovalues = [];
		var idx = 0;
		var currSelected;
		_.each(args.values,function(data){ 
			combovalues.push({title:data.key,id: idx, fldvalue:data.value});
			idx = idx +1;
		}); //each
		
		currSelected = combovalues[0];
		var rowUpdate = function(e){
			self.value = currSelected.fldvalue;
			input.text = currSelected.title;
		};
		
		rowUpdate();
		
		self.addEventListener('click',function(){
			ICABBI.navController.showPickerModal({
			    title:'Select',
			    data: combovalues,
			    click : function(e){	
			    	if(e.selected || e.selected===0) 
			    	{
			    		currSelected= combovalues[e.selected];
			    		rowUpdate();				   		
			    	}
			    }
				});		
		});
	}
	
	var hintLabel = Ti.UI.createLabel({
		left:0,
		bottom:0,
		left:12,
		right:12,
		color:Theme.form.smallColor,
		font:Theme.global.buttonFont,
		text:args.hint,
		textAlign:'right',
		touchEnabled:false,
	});
	self.add(hintLabel);

	var rowLine = Ti.UI.createView({
		top: 7,
		left:0,
		right:0,
		bottom:0,
		height:'1px',
		backgroundColor:Theme.global.altBGColor,
		visible:!args.lastItem,
		touchEnabled:false,
	});
	self.add(rowLine);
	
	return self;
};

module.exports = FormValidationRow;
