var _ = require('/lib/lodash')._,
	ICABBI = require('/core/icabbi'),
	Theme = require('/ui/Theme'),
	Address = require('/model/Address'),
	ChildWindow = require('/ui/common/ChildWindow'),
	FormPickerRow = require('/ui/common/FormPickerRow');

var FormViaRow = function(options) {
	
	options = options || {};

	var booking = null;
	
	var self = Ti.UI.createView({
		top:0,
        left:0,
        right:0,
        height:Ti.UI.SIZE,
		layout:'vertical',
	});
	
	var viaRowsView = Ti.UI.createView({
		top:0,
        left:0,
        right:0,
        height:Ti.UI.SIZE,
		layout:'vertical',
	}); 	
	self.add(viaRowsView);
	
	var vias = [], 
		viaRows = []; 
	
	var addViaRow = new FormPickerRow({
		label: ' ',
		title: '+ Via',
		subTitle: '',
	});
	self.add(addViaRow);
	
	var addAddressRow = function(adr) {
		var address = new Address.Model();
		address.set(adr);
		
		var newIdx = vias.length;
		vias[newIdx] = address;
		
		address.set('via_idx', newIdx);

		var viaRow = new FormPickerRow({
			label: 'Via ' + (newIdx + 1),
			title: address.formatDisplay(),
			subTitle: address.get('area'),
			onRemoveClick: function(evt) {
				ICABBI.navController.showYesNo({
					    title:'Remove this Via?',
					    message:'Are you sure you want to remove this via?',
					    click : function(e){
					    	if(e.index == 1)
					    		removeAddress(evt, address.get('via_idx'));
					    }
					});
			},
		});
		viaRowsView.add(viaRow);
		viaRows[newIdx] = viaRow;
		
		viaRow.titleView.addEventListener('click',function(evt) {
			addEditAddress(evt, address.get('via_idx'));
		});
	};
	
	var addEditAddress = function(e, editIdx) {
		
		if(ICABBI.Config.search_addresses)
		{
			if (ICABBI.navController.opening(e.source)) {
				return;
			}
			var AddressSearchView = require('/ui/common/AddressSearchView');
			var config = {
				content: AddressSearchView,
				header: {
					visible: true, 
					controls: { cancel: { visible: true } }
				},
				title : 'VIA address search',
				notScrollable : true,
				data:{destination: true},
			};
		    var win = new ChildWindow(config);
		    win.addEventListener('close',function(e){
				if(win.__retval && _.isObject( win.__retval) && win.__retval.address) {
					
					if(editIdx < 0) {
						addAddressRow(win.__retval.address);						
					}
					else {
						vias[editIdx].set(win.__retval.address);
						var address = vias[editIdx];
						
						viaRows[editIdx].putText({
	   						title: address.formatDisplay(),
	   						subTitle: address.get('area'),
				  		});						
					}
				}
			});
		    ICABBI.navController.open(win, undefined, e.source);
		}
		else
		{
			if(!bookingData.pois)
		 	{
		 		bookingData.pois = [];
				
				ICABBI.Controller.getMeta().destinations.each(function(destination){
					bookingData.pois.push({
						title:destination.get('formatted'),
						id:destination.id
					});
				});
		 	}
		 	if(bookingData.pois.length == 0)
		 	{
		 		e.source.opening = false;
				return;
		 	}
		 	ICABBI.navController.showPickerModal({
			    title:'Destination',
			    data: bookingData.pois,
			    click : function(e){
			    	//if(e.selected) {
			    		var destination = null;
						ICABBI.Controller.getMeta().destinations.each(function(dest){
								if(dest && dest.id == e.selected){
									destination = dest;							
								}
							});
						if(!destination)
							return;
			    		if(editIdx < 0) {
							addAddressRow(destination.toJSON());						
						}
						else {
							vias[editIdx].set(destination.toJSON());
							var address = vias[editIdx];
							
							viaRows[editIdx].putText({
		   						title: address.formatDisplay(),
		   						subTitle: address.get('area'),
					  		});						
						}
			    	//}
			    }
			});
			e.source.opening = false;
		}
	};
	
	var removeAddress = function(e, idx) {
		viaRowsView.remove(viaRows[idx]);
		viaRows.splice(idx, 1);
		vias.splice(idx, 1);
		
		for(var i in viaRows) {
			viaRows[i].putText({
				label: 'Via ' + (parseInt(i) + 1),
	  		});
	  		vias[i].set('via_idx', i);
		}
	};
	
	addViaRow.titleView.addEventListener('click',function(e) {
		addEditAddress(e, -1);
	});
	
	var clearViaRow = function() {
		viaRowsView.removeAllChildren();
		viaRows = [];
		vias = [];
	};
		
	// Functions
	self.setBooking = function(bk) {
		booking = bk;
		
		clearViaRow();
		
		var bookingVias = bk.get('vias');
		
		if(bookingVias) {
			for(var idx in bookingVias) {
				addAddressRow(bookingVias[idx]);
			}
		}	
	};
	
	self.getVias = function() {
		var res = [];
		for(var idx in vias) {
			res.push(vias[idx].toJSON());
		};
		return res;
	};
	
	return self;
};

module.exports = FormViaRow;