var ICABBI = require('/core/icabbi'),
	$$ = require('/core/ThemeManager');

var Theme = require('/ui/Theme');
var HeaderButton = require('/ui/common/HeaderButton');

var headerHeight = Theme.childwindow.headerHeight;

var createTableViewRow = function(rowData){
	var row = Ti.UI.createTableViewRow({
		height:44,
		navView: rowData.view,
		backgroundColor:Theme.global.bgColor,
		selectedColor: 'transparent',
		backgroundFocusedColor:'transparent',
		backgroundSelectedColor:'transparent',
		selectionStyle:Ti.Android?undefined:Ti.UI.iPhone.TableViewCellSelectionStyle.NONE,
		data:rowData.data || {},
	});
	
	var icon = Ti.UI.createImageView({
		left:10,
		width:24,
		height:24,
		image:rowData.image,
		touchEnabled:false,
	});
	row.add(icon);
	
	var container=Ti.UI.createView({
		right:0,
		height:44,
		left:44,
	});
	row.add(container);
	var title = Ti.UI.createLabel({
		color:Theme.global.fontColor,
		left:0,
		right:25,
		height:30,
		text:rowData.title,
		font:Theme.menu.rowFont,
		touchEnabled:false,
	});
	container.add(title);
	row.setTitle = function(t){
		title.setText(t);
	};
	row.setImage = function(i){
		icon.setImage(i);
	};
	
	if(rowData.hasNotification){
		var lblNotification = Ti.UI.createLabel({
			right:30,
			color:Theme.global.fontColor,
			backgroundColor:Theme.global.highlightLightColor,
			width:20,
			height:18,
		    textAlign: Ti.UI.TEXT_ALIGNMENT_CENTER,
			borderRadius:6,
			borderWidth:1,
			borderColor:Theme.global.highlightColor,
			font:Theme.global.smallFont,
			touchEnabled:false,
			visible:false,
		});
		ICABBI.Controller.on('change:active_bookings',function(e){
			lblNotification.text = e.count;
			lblNotification.visible = !!e.count;
		});

		container.add(lblNotification);
	}

	if(rowData.hasDetail){
		var detail = Ti.UI.createImageView({
			right:12,
			width:9,
			height:15,
			image:'/images/map/arrow.png',
			touchEnabled:false,
		});
		container.add(detail);
	}
	
	var rowLine = Ti.UI.createView({
		left:44,
		right:0,
		bottom:0,
		height:'1px',
		backgroundColor:Theme.global.listBottomLineColor,
		touchEnabled:false,
		visible:!rowData.lastItem,
	});
	row.add(rowLine);
	Theme.animateButton(container);	
	return row;
};
var createHeadingView = function(title, closeAction) {
	var view = Ti.UI.createView({
		height: headerHeight,
		left:0,
		right:0,
		backgroundColor:Theme.nav.backgroundColor,
	});
	
	var section = Ti.UI.createView({
			left:0,
			right:0,
			height: headerHeight,
			backgroundColor:headerHeight,
		});
		
		if(!closeAction){
			var sectionLineTop = Ti.UI.createView({
				left:0,
				right:0,
				top:0,
				height:1,
				backgroundColor:Theme.nav.seperatorColor,
			});
		section.add(sectionLineTop);
	
		var sectionLineTopWhite = Ti.UI.createView({
			left:0,
			right:0,
			top:1,
			height:1,
			backgroundColor:Theme.global.bgColor,
		});
		section.add(sectionLineTopWhite);
		
		var sectionLineBottomWhite = Ti.UI.createView({
			left:0,
			right:0,
			bottom:1,
			height:1,
			backgroundColor:Theme.global.bgColor,
		});
		section.add(sectionLineBottomWhite);
		}
		var sectionLineBottom = Ti.UI.createView({
			left:0,
			right:0,
		bottom:0,
			height:1,
			backgroundColor: closeAction ? Theme.global.altBGColor : Theme.nav.seperatorColor,
	});
	section.add(sectionLineBottom);


	var text = Ti.UI.createLabel({
		text: title,
		left: 12,
		color:Theme.nav.headerColor,
		font:Theme.nav.headerFont,
		textAlign: Ti.UI.TEXT_ALIGNMENT_CENTER,
	});
		view.add(text);
   		view.add(section);
	
	if(closeAction){
		var controls = Ti.UI.createView({
			width:Ti.UI.SIZE,
			height: headerHeight,
			right:0,
			layout:'horizontal',
		});
		view.add(controls);
		
		var btnClose = Ti.UI.createView({
			right:5,
			height:48,
			width:50,
			animateType:Theme.ANTIMATE_TYPE_OPACITY,
		});
		
		var closeIcon = Ti.UI.createImageView({
			height:30,
			width:50,
			left:0,
			touchEnabled:false,
			image:'/images/headers/icon_close_button.png',
		});
		btnClose.add(closeIcon);
		var closeLabel = Ti.UI.createLabel({
			text: 'Close',
			bottom:17,
		    left: 10,
			color:Theme.childwindow.buttonTextColor,
			font:Theme.childwindow.headerButtonFont,
		});
		btnClose.add(closeLabel);
		btnClose.addEventListener('click', closeAction);
		Theme.animateButton(btnClose);
		controls.add(btnClose);
	}

	return view;
};

var MenuView = function(args){

	var logoutButton = null;
	var logoutIndex = null;
	var closeAction = function(e){			
		ICABBI.SlidingMenu.fireEvent('openview',{ index:ICABBI.Config.show_map ? 0 : 1 });			
	};
	
	var createRows = function(rowData){
		var sections = [];
		var rows = [];
		var index = 0;
		var rowIndex = 0;
		for(var i=0,j=rowData.length; i<j; i++) {
			var data = rowData[i];
			var section= Ti.UI.createTableViewSection({
				headerView : createHeadingView(data.heading, i == 0 ? closeAction : false),
			});
			for(var k=0,l=data.rows.length; k<l; k++) {
				var row = data.rows[k];
				
				var rowView = createTableViewRow(row);
				
				if(row.data && row.data.logout) {
					logoutButton = rowView;
					logoutIndex = rowIndex;
				}
					
				rowView.viewIndex = index;
				rows.push(rowView);
				if(!('visible' in row && ((typeof row.visible === 'function' && !row.visible()) || !row.visible)))
				{
					section.add(rowView);
					rowIndex++;
				}
				index++;
			}
			sections.push(section);
		}
		
		return [sections, rows];
	};
	
	var cri = createRows(args.sectionData);
	
	var self = Ti.UI.createTableView({
		backgroundColor:'#fff',
		separatorColor:'transparent',
		separatorStyle:Ti.UI.Android ? undefined : Ti.UI.iPhone.TableViewSeparatorStyle.NONE,
		allowsSelection:Ti.Android,
		top:0,
		bottom:0,
		left:0,
		right:0,
		data:cri[0],
		rows:cri[1],
		
		onBackPressed: closeAction
	});
	ICABBI.Controller.getUser().on('change:authenticated',function(){
		if(logoutButton){
			logoutButton.setTitle(ICABBI.Controller.getUser().isAuthenticated() ? 'Logout' : 'Login');
			logoutButton.setImage(ICABBI.Controller.getUser().isAuthenticated() ? '/images/menu/icon_logout.png' : '/images/menu/icon_login.png');
			self.updateRow(logoutIndex,logoutButton);
		}
	},this);
	
	self.sectionData = args.sectionData;
	self.restart = function(){
		var cr = createRows(self.sectionData);
		var sections = cr[0];
 
		self.setData(sections);
		self.rows = cr[1];
	};
	return self;
};

module.exports = MenuView;
