var tools = require('/lib/tools'),
	_ = require('/lib/lodash')._ ,
	$$ = require('/core/ThemeManager'),
	Theme = require('/ui/Theme');

var PickerView = function(options) {
	options = options || {};

	var headerHeight = footerHeight = 48;
	var tableView;

	var fillTable = function(rows) {
		var tableData = [];
		if(rows) {
			_.each(rows, function(row){
				tableData.push(createRow({
				  	title : row.title||'',
				  	data : row.data,
				}));
			});
		}
		tableView.setData(tableData);
	};
	
	var createContent = function(){
		var content = Titanium.UI.createView({
			top:0,
			bottom:options.data.footerControls ? footerHeight : 0,
			left:0,
			right:0,
	    });

		var contentTopBorder = Ti.UI.createView({
			top: 0,
			left: 0,
			right:0,
			height:1,
			backgroundColor:Theme.picker.backgroundColor,
		});
		content.add(contentTopBorder);
		var contentBottomBorder = Ti.UI.createView({
			bottom: 0,
			left: 0,
			right:0,
			height:1,
			backgroundColor:Theme.global.altBGColor,
		});
		content.add(contentBottomBorder);

		
		tableView = Ti.UI.createTableView({
		  backgroundColor:'white',
		  data:[],
		  top: 0,
		  bottom: 1,
		  left: 0,
		  right:0,
		  separatorColor: Theme.picker.backgroundColor,
		});
		
		tableView.addEventListener('click', function(e) {
			options.parent.window.__retval = {selected:e.row.data};
			ICABBI.navController.close(options.parent.window);
		});
		content.add(tableView);

		return content;
	}; 

	var createRow = function(data) {
		var row = Ti.UI.createTableViewRow({
			height:44,
			backgroundColor:Theme.picker.rowBackgroundColor,
			selectedColor: 'transparent',
			backgroundFocusedColor:'transparent',
			backgroundSelectedColor:'transparent',
			selectionStyle:Ti.Android?undefined:Ti.UI.iPhone.TableViewCellSelectionStyle.NONE,
		});
		row.data = data.data;
		
		row.className = 'datarow';
		row.clickName = 'row';
		
		var container=Ti.UI.createView({
			right:0,
			height:44,
			left:Ti.UI.SIZE,
		});
		row.add(container);
		var title = Ti.UI.createLabel({
			color:Theme.global.fontColor,
			left:7,
			right:36,
			text:data.title,
			touchEnabled:false,
		    wordWrap:false,
		    ellipsize:true,
			width: Ti.UI.FILL,
			font:Theme.picker.font,
		});
		container.add(title);
		
		var arrowIcon = Ti.UI.createImageView({
		    height:11,
		    width:11,
		    right: 7,
		    image:'/images/map/arrow.png',
		    touchEnabled:false,
		});
		container.add(arrowIcon);
		
		var rowBottomLine = Ti.UI.createView({
			left:0,
			right:0,
			bottom:0,
			height:1,
			backgroundColor:Theme.picker.rowSeparator,
			touchEnabled:false,
		});
		container.add(rowBottomLine);
		Theme.animateButton(container);	
		return row;

	};
	var createFooter = function(controls){
		var footer = Ti.UI.createView({
			bottom: 0,
			left: 0,
			right:0,
			height: footerHeight,
			backgroundColor: Theme.picker.footerBackgroundColor,
		});
		
		var footerBorder = Ti.UI.createView({
			top: 0,
			left: 0,
			right:0,
			height:1,
			backgroundColor:Theme.picker.backgroundColor,
		});
		footer.add(footerBorder);
		
		footer.add(controls);
		
		return footer;
	};
	
	var create = function(){
		var view = Titanium.UI.createView({
			top:0,
			left:0,
			right:0,
	        height:Ti.UI.SIZE,
			//layout:'vertical',
	    });
		
		content = createContent();
		var footer = null;
		if(options.data.footerControls)
			footer = createFooter(options.data.footerControls());
			
		view.add(content);
		
		if(footer)
			view.add(footer);

		return view;
	};
	options.parent.window.addEventListener('open',function(){
		setTimeout(function(){
			if(options.data.callback)
			{
				options.callback(function(data){
					fillTable(data);
				});
			}
			else
			{
				fillTable(options.data.rows || []);
			}
		}, 100);
	});
	var self = create();
	return self;
};

module.exports = PickerView;
