var _ = require('/lib/lodash')._ ,
	ICABBI = require('/core/icabbi');

var Theme = require('/ui/Theme');
var ActionButton = require('/ui/common/ActionButton');
var HeaderButton = require('/ui/common/HeaderButton');
var ui = require('ui');

var RateDriverView = function(options) {
	options = options || {};
	
	var booking = options.data && options.data.booking ? options.data.booking : null;
	var self = Ti.UI.createView({
 		width:Ti.UI.FILL,
        height:Ti.UI.FILL,
		layout:'vertical',
	});
	// close button
	var controls = Ti.UI.createView({
		width:Ti.UI.SIZE,
		height: 44,
		top:1,
		right:0,
		layout:'horizontal',
	});
	self.add(controls);
	
	if(!ICABBI.Config.rating_mandatory) {
		var btnClose = new HeaderButton({
			title:'Close',
			icon:'/images/headers/icon_close.png',
		});
		
		btnClose.addEventListener('click',function(e) {
			if(ICABBI.Config.rating_stars_enabled || ICABBI.Config.rating_comment_enabled) {
				ICABBI.navController.showYesNo({
				    title:'Feedback!',
				    message:'Your feedback would help. Are you sure you want to close without adding your feedback?',
				    click : function(e){
				    	if(e.index == 1)
				    		ICABBI.navController.close(options.parent.window);
				    }
				});
			}
			else {
				ICABBI.navController.close(options.parent.window);
			}
		});
		
		controls.add(btnClose);
	}

	// logo
	var logo = Ti.UI.createImageView({
		top:0,
		height:  Math.round(Ti.Platform.displayCaps.platformWidth*0.20),//Ti.Platform.displayCaps.platformHeight<350?65:120,//320,
		width:  Math.round(Ti.Platform.displayCaps.platformWidth*0.20),//Ti.Platform.displayCaps.platformHeight<350?65:120,//320,
		image:'/images/logo.png',
	});
	self.add(logo);

	// thank you note
	var thanks = Ti.UI.createLabel({
		text: 'Job completed.Thank you for travelling with '+ICABBI.Config.Company.name,
		top:5,
		left:12,
		right:12,
		color:Theme.form.smallColor,
		font:Theme.form.smallFont,
		textAlign:'center',		
	});
	self.add(thanks);

	var controlBar = null,
		ratingStarsView = null,
		ratingCommentView = null;
	if(ICABBI.Config.rating_stars_enabled || ICABBI.Config.rating_comment_enabled) {
		
		// title
		var title = Ti.UI.createLabel({
			text:'Rate Your Driver',
			top:10,
			left:12,
			right:12,
			color:Theme.section.titleColor,
			font:Theme.section.titleFont,
			textAlign:'center',		
		});
		self.add(title);
		
		// rating stars
		if(ICABBI.Config.rating_stars_enabled) {
			
			var ratingStarsView = new ui.RatingView(0, 5);
			ratingStarsView.top = 5;
			ratingStarsView.width = Ti.UI.SIZE;
			self.add(ratingStarsView);
		}
		
		// rating comment
		if(ICABBI.Config.rating_comment_enabled) {
			
			var ratingCommentView = new ui.RatingComment();
			ratingCommentView.top = 7;
			ratingCommentView.height = Ti.Platform.displayCaps.platformHeight<350?60:80;
			self.add(ratingCommentView);
		}
		
		controlBar = Titanium.UI.createView({
			top:7,
	        height:70,
	        width:Ti.UI.SIZE,
	        layout:'horizontal',
	    });
	    self.add(controlBar);
		
		// submit button
		var btnSubmit = new ActionButton({title:'SUBMIT',icon:'/images/buttons/icon_save.png'});
		controlBar.add(btnSubmit);
		
		btnSubmit.addEventListener('click',function(){
			var error='';
			var noError=true;	
			if(!booking) {
				error+='- No booking to rate!\n';
				ICABBI.navController.showError({
				    title:'Error!',
				    message:error,
				    close:true
				});
				return;
			}
						
			if(ICABBI.Config.rating_mandatory && ratingStarsView && !ratingStarsView.rating) {
				error+='- Please provide your rating!';
				noError=true;
			}
						
			if(ICABBI.Config.rating_mandatory && !ratingStarsView && !ratingCommentView.textArea.value) {
				error+='- Please provide a comment!\n';
				noError=true;
			}
			if(!noError){
				ICABBI.navController.showError({
				    title:'Error!',
				    message:error,
				    close:true
				});
				return;
			}
			
			var driver = booking.get('driver') || {id: null};
			
			var ratingData = {
				booking_id: booking.get('perma_id'),
				rating: ratingStarsView && ratingStarsView.rating,
				comment: ratingCommentView && ratingCommentView.textArea.value,
				driver_id: driver.id
			};
			
			$$.ActivityIndicator.start('Saving Your Rating');
			ICABBI.Controller.rateDriver(ratingData, {
				success: function(rating){
					$$.ActivityIndicator.stop();
					
					ICABBI.navController.close(options.parent.window);
					
					ICABBI.navController.showAlert({
					    title:'Thank you!',
					    message:'Thank you for your rating!',
					    close:true
					});
				},
				error: function(resp) {
					$$.ActivityIndicator.stop();
					var msg = resp && resp.message ? resp.message : 'Server Unavailable. Please try again later!';
					if(resp)
					for (var i in resp.body) {
						msg += '\n' + i + ': ' + resp.body[i];
					}

					ICABBI.navController.showError({
					    title:'Error!',
					    message:msg,
					    close:true
					});				
				}
			});
			

		});	
	}
	
	var btnPhone = new ActionButton({title:'PHONE',icon:'/images/buttons/icon_phone.png'});
	btnPhone.left = 8;

	if(!controlBar) {
		btnPhone.left = 0;
		
		controlBar = Titanium.UI.createView({
			top:20,
	        height:70,
	        width:Ti.UI.SIZE,
	        layout:'horizontal',
	    });
	    self.add(controlBar);
	}
	controlBar.add(btnPhone);

	btnPhone.addEventListener('click',function(){
		Ti.Platform.openURL('tel:'+ICABBI.Config.phone);
	});

	// powered by
	var poweredView = Ti.UI.createView({
 		width:Ti.UI.FILL,
        height:Ti.UI.FILL,
	});
	self.add(poweredView);
	
	var powered = Ti.UI.createImageView({
		bottom:2,
		height:28,
		width:128,
		image:'/images/powered.png',
	});
	poweredView.add(powered);
	
	return self;
};

module.exports = RateDriverView;
