var tools = require('/lib/tools'),
	_ = require('/lib/lodash')._ ,
	$$ = require('/core/ThemeManager'),
	Theme = require('/ui/Theme'),
	FormTextfieldRow = require('/ui/common/FormTextfieldRow'),
	FormCheckboxRow = require('/ui/common/FormCheckboxRow'),
	Geolocation = require('/lib/Geolocation'),
	RemoteImage = require('/lib/RemoteImage'),
	ICABBI = require('/core/icabbi'),
	HeaderButton = require('/ui/common/HeaderButton');

var TourView = function(options) {
	options = options || {};

	var headerHeight = footerHeight = 48;
	var btnNextMenu, btnBackMenu, stepsView,scrollVView;
	
	var currentIndex = 0;
	var tourImages = [];
	
	var tourAllImages = ICABBI.Config.tour;
	
	if(Ti.Android){
		for(var i in tourAllImages){
			if(tourAllImages[i].indexOf('android')!=-1){
				
				// Cache method --- need to be fixed crash for android
				// var image = RemoteImage.createRemoteImageView({
					// touchEnabled:false,
					// width:'100%',
					// height:'100%',
					// image:tourAllImages[i],				
				// });
				
				var image = Ti.UI.createImageView({
					touchEnabled:false,
					width:Ti.UI.FILL,
					height:Ti.UI.FILL,
					image:tourAllImages[i],				
				});
				tourImages.push(image);
			}
		}
	}
	else{
		for(var i in tourAllImages){
			if(tourAllImages[i].indexOf('ios')!=-1){
				var image = RemoteImage.createRemoteImageView({
					touchEnabled:false,
					width:Ti.UI.FILL,
					height:Ti.UI.FILL,
					image:tourAllImages[i],				
				});
				tourImages.push(image);
			}
		}
	}
	
	var createHeader = function(){
		var header = Ti.UI.createView({
			top: 0,
			left: 0,
			right:0,
			height: 45,
			backgroundColor: Theme.childwindow.bgColor,
		});

		var headerBorder = Ti.UI.createView({
			bottom: 0,
			left: 0,
			right:0,
			height:1,
			backgroundColor:Theme.global.altBGColor,
		});
		header.add(headerBorder);
		
		var lblInfo = Ti.UI.createLabel({
		    text:'Tour',
		    height:headerHeight,
			color:Theme.nav.headerColor,
			font:Theme.nav.headerFont,
		    left:8,
		});
		header.add(lblInfo);

		var controls = Ti.UI.createView({
			width:Ti.UI.SIZE,
			height: 44,
			right:0,
			layout:'horizontal',
		});
		header.add(controls);
		
		var btnClose = new HeaderButton({
			title:'Close',
			icon:'/images/headers/icon_close.png',
		});
		btnClose.addEventListener('click',function(e){
			close();
		});
		
		controls.add(btnClose);
	
		return header;
	};

	var createContent = function(){
		var content = Titanium.UI.createView({
			top:headerHeight,
			bottom:'15%',
			left:0,
			right:0,
			backgroundColor: '#ffffff',
	    });

		var contentTopBorder = Ti.UI.createView({
			top: 0,
			left: 0,
			right:0,
			height:1,
		});
		content.add(contentTopBorder);
				
		scrollVView =  Ti.UI.createView({
		        top:0,
		        bottom:0,
		        layout:'vertical',
			});
			
	    content.add(scrollVView);
		
		return content;
	}; 

	var loadTour = function() {
		var tour = tourImages[currentIndex];
		if(!tour)
			return;
		//var a=Ti.Platform.displayCaps.platformHeight-48-Ti.Platform.displayCaps.platformHeight*0.15;
		// var contentWrapper =  Ti.UI.createView({
				// height: '100%',
			    // left: 0,
			    // right: 0,
		        // layout:'vertical',      
			// });
		
		var topImageWrapper = Titanium.UI.createView({
			width: Ti.UI.FILL,
		    bottom: 10,
		    top: 10,
		});
		
		
		topImageWrapper.add(tour);
		
		//contentWrapper.add(topImageWrapper);

		scrollVView.removeAllChildren();
		scrollVView.add(topImageWrapper);
	};
	var createFooter = function(){
		var footer = Ti.UI.createView({
			bottom: 0,
			left: 0,
			right:0,
			height: '15%',
			backgroundColor: '#dddddd',
		});
		
		var footerBorder = Ti.UI.createView({
			top: 0,
			left: 0,
			right:0,
			height:1,
			backgroundColor:"#000000",
		});
		footer.add(footerBorder);
			
		btnBackMenu = Ti.UI.createView({
			left:5,
			height:'60%',
			width: Ti.UI.SIZE,
			animateType:Theme.ANTIMATE_TYPE_OPACITY,
			visible : true,
		});
		
		var backIcon = Ti.UI.createImageView({
			left:5,
			touchEnabled:false,
			image:'/images/buttons/left_navigation.png',
		});

		btnBackMenu.add(backIcon);
		Theme.animateButton(btnBackMenu);
		btnBackMenu.addEventListener('click',function(e){
			if(currentIndex > 0)
			{
				currentIndex--;
				updateStep();
				loadTour();
			}
		});

		footer.add(btnBackMenu);

		stepsView = Ti.UI.createView({
			width:Ti.UI.SIZE,
			height: '70%',
			layout:'horizontal',
		});
		footer.add(stepsView);
		for( i=0; i<tourImages.length; i++){
			var bullet = Ti.UI.createImageView({
			touchEnabled:false,
			height:'100%',
			image: (i==currentIndex)?'/images/buttons/bullet_on.png':'/images/buttons/bullet_off.png',
			});
			Ti.API.info('SIZE'+JSON.stringify(bullet.getSize()));
			stepsView.add(bullet);
		}
						
		btnNextMenu = Ti.UI.createView({
			right:5,
			height:'60%',
			width: Ti.UI.SIZE,
			animateType:Theme.ANTIMATE_TYPE_OPACITY,
		});
		
		var nextIcon = Ti.UI.createImageView({
			right:5,
			touchEnabled:false,
			image:'/images/buttons/right_navigation.png',
		});

		btnNextMenu.add(nextIcon);
		Theme.animateButton(btnNextMenu);
		btnNextMenu.addEventListener('click',function(e){			
			if(currentIndex < tourImages.length - 1)
			{
				currentIndex++;
				updateStep();
				loadTour();
			}
			else {
				currentIndex = 0;
				close();
				return;
			}			
		});
		footer.add(btnNextMenu);

		return footer;
	};
	
	var updateStep = function () {
		var children = stepsView.children.slice(0);
 
	    for (var i = 0; i < children.length; ++i) {
	        children[i].image= (i==currentIndex)?'/images/buttons/bullet_on.png':'/images/buttons/bullet_off.png';
	    }
	};
	
	var close = function(){
		currentIndex = 0;
		loadTour();
		if(!Ti.App.Properties.getBool("TourTaken", false)) {
			Ti.App.Properties.setBool("TourTaken", true);
			ICABBI.SlidingMenu.fireEvent('openview',{index:ICABBI.Config.show_map?0:1});
		}
		else
			ICABBI.Controller.trigger('slidingmenu:toggle');
	};
	var create = function(){
		var view = Titanium.UI.createView({
			width:Ti.UI.FILL,
	        height:Ti.UI.FILL,
			//layout:'vertical',
	    });

		header = createHeader();
		content = createContent();
		footer = createFooter();
		view.add(header);		
		view.add(content);
		view.add(footer);

		loadTour();
		
		return view;
	};
	
	var self = create();
	
	self.addEventListener('slidingmenu:view:load',function(e){
			loadTour();
			updateStep();
		});
	return self;
};

module.exports = TourView;
