/**
 * @author Sonal
 */
var ICABBI = require('/core/icabbi');
var tools = require('/lib/tools'),
	_ = require('/lib/lodash')._ ,
	$$ = require('/core/ThemeManager');

var Theme = require('/ui/Theme');
var FormPickerRow = require('/ui/common/FormPickerRow');
var FormTextfieldRow = require('/ui/common/FormTextfieldRow');
var FormCustomRow = require('/ui/common/FormCustomRow');
var ActionButton = require('/ui/common/ActionButton');

var UserProfileView = function(options) {
	options = options || {};
	
	var user = ICABBI.Controller.getUser(),
		userprofile = user.get('userprofile') || {};
	
	var createForm = function(){

		var view = Titanium.UI.createView({
			top:10,
			left:0,
			right:0,
	        height:Ti.UI.SIZE,
			layout:'vertical'
	    });
	    
	    var form={fields:{}};
	    
	    /*
	     * Street
	     */
	    streetRow = new FormTextfieldRow({
   			label:'Street',
			returnKeyType:Ti.UI.RETURNKEY_NEXT,
			value: userprofile.street || '',
	    });
		form.fields.street = {
			title:'Street',
			required:true,
			getValue: function(){streetRow.doBlur();return streetRow.value;}
		};
		view.add(streetRow);
	    
	    /*
	     * Postcode
	     */
	    postcodeRow = new FormTextfieldRow({
   			label:'Post Code',
			returnKeyType:Ti.UI.RETURNKEY_NEXT,
			value: userprofile.postcode || '',
	    });
		form.fields.postcode = {
			title:'Post Code',
			required:true,
			getValue: function(){postcodeRow.doBlur();return postcodeRow.value;}
		};
		view.add(postcodeRow);
	    
	    /*
	     * City
	     */
	    cityRow = new FormTextfieldRow({
   			label:'City',
			returnKeyType:Ti.UI.RETURNKEY_NEXT,
			value: userprofile.city || '',
	    });
		form.fields.city = {
			title:'City',
			required:true,
			getValue: function(){cityRow.doBlur();return cityRow.value;}
		};
		view.add(cityRow);
	    
	    /*
	     * County
	     */
	    countyRow = new FormTextfieldRow({
   			label:'County',
			returnKeyType:Ti.UI.RETURNKEY_NEXT,
			value: userprofile.county || '',
	    });
		form.fields.county = {
			title:'County',
			required:false,
			getValue: function(){countyRow.doBlur();return countyRow.value;}
		};
		view.add(countyRow);
	    
		var btnSubmit = Ti.UI.createButton({
			top:10,
			right:16,
			width:110,
			height:42,
			backgroundColor:Theme.global.buttonBGColor,
			borderColor:Theme.global.highlightColor,
			backgroundFocusedColor:Theme.global.highlightColor,
			backgroundSelectedColor:Theme.global.highlightColor,
			borderWidth:1,
			borderRadius:8,
			color:Theme.global.fontColor,
			font:Theme.form.buttonFont,
			title: 'Save',
	  	    style:Ti.UI.iPhone.SystemButtonStyle.PLAIN,
		});
		Theme.animateButton(btnSubmit);
		view.add(btnSubmit);
		
		btnSubmit.addEventListener('click',function(e){
			streetRow.doBlur();
			postcodeRow.doBlur();
			cityRow.doBlur();
			countyRow.doBlur();
		
			var error='';

			var fields={
				street: form.fields.street.getValue(),
				postcode: form.fields.postcode.getValue(),
				city: form.fields.city.getValue(),
				county: form.fields.county.getValue(),
				
				// Use same values for billing info
				billing_street: form.fields.street.getValue(),
				billing_postcode: form.fields.postcode.getValue(),
				billing_city: form.fields.city.getValue(),
				billing_county: form.fields.county.getValue(),
			};
			
			if(!fields.street.length){
				error += '- Street is required!\n';
			}
			if(!fields.postcode.length){
				error += '- Post Code is required!\n';
			}
			if(!fields.city.length){
				error += '- City is required!\n';
			}

			if(error){
				ICABBI.navController.showError({
				    title:'Error!',
				    message:error,
				    close:true
				});
				return;
			};

			$$.ActivityIndicator.start('Updating your profile');
			ICABBI.Controller.updateUserProfile(
				fields, {
					success: function(e){
						$$.ActivityIndicator.stop();
						
						if(e && e.userprofile)
							user.set('userprofile', e.userprofile);
						
						ICABBI.navController.close(options.parent.window);
					},					
					error: function(e){
						$$.ActivityIndicator.stop();
						
						ICABBI.navController.showError({
						    title:'Error!',
						    message:e,
						    close:true
						});
						return;
					}
				});
		});

		return view;
	}; 
	
	var self = createForm();
	return self;
};

module.exports = UserProfileView;

