/**
 * @author Sonal
 * VehicleType.js
 */

var ICABBI = require('/core/icabbi');

var tools = require('/lib/tools'),
	_ = require('/lib/lodash')._ ,
	$$ = require('/core/ThemeManager'),
	Booking = require('/model/Booking'),
	RemoteImage = require('/lib/RemoteImage');

var Theme = require('/ui/Theme');

//VehicleType Component Constructor
var VehicleType = function(options) {
	options = options || {};
	var vehicleType = options.data.vehicleType,
		vehicleTypes = ICABBI.Controller.getMeta().vehicleTypes;
		
	var createTableViewRow = function(vehicleType,checked){
		checked = checked || false;
		var row = Ti.UI.createTableViewRow({
			height:63,
			backgroundColor:Theme.global.bgColor,
			selectedColor: 'transparent',
			backgroundFocusedColor:'transparent',
			backgroundSelectedColor:'transparent',
			selectionStyle:Ti.Android?undefined:Ti.UI.iPhone.TableViewCellSelectionStyle.NONE,
			__vehicleType : vehicleType.get('key'),
		});
	
		var iconContainer=Ti.UI.createView({
			left:0,
			width:70,
			height:62,
		});
		row.add(iconContainer);
		var icon = RemoteImage.createRemoteImageView({
			width:58,
			height:'46',
			defaultImage:'',
			image:vehicleType.get('image'),
			touchEnabled:false,
		});
		iconContainer.add(icon);
				
		var container=Ti.UI.createView({
			right:0,
			height:62,
			left:iconContainer.width,
		});
		row.add(container);
		
		var attributes = Ti.UI.createView({
			left:0,
			width:88,
			top:12,
			height:38,
		});
		if(vehicleType.get('luggage')) {
			var luggages = Ti.UI.createView({
				left:0,
				top:0,
				height:19,
				width:58,
				layout:'horizontal',
			});
	
			var luggageNo = Ti.UI.createLabel({
				color:'#6a6a6a',
				font:Theme.vehicleType.attrFont,
				left:12,
				top:0,
				bottom:0,
				width: 11,
				text:vehicleType.get('luggage'),
				touchEnabled:false,
			});
			luggages.add(luggageNo);
	
			var times = Ti.UI.createLabel({
				color:'#6a6a6a',
				font:Theme.vehicleType.timesFont,
				left:0,
				bottom:0,
				text:'×',
				touchEnabled:false,
				height: '85%',
			});
			luggages.add(times);
	
			var luggageicon = Ti.UI.createImageView({
			    height:21,
			    width:18,
			    image:'/images/vehicles/icon_luggage.png',
			    touchEnabled:false,
			});
			luggages.add(luggageicon);
							
			attributes.add(luggages);
		}
		
		if(vehicleType.get('seats')) {
			var persons = Ti.UI.createView({
				left:0,
				bottom:0,
				height:19,
				width:58,
				layout:'horizontal',
			});
			
			var personNo = Ti.UI.createLabel({
				color:'#6a6a6a',
				font:Theme.vehicleType.attrFont,
				left:12,
				top:0,
				bottom:0,
				width:11,
				text:vehicleType.get('seats'),
				touchEnabled:false,
			});
			persons.add(personNo);
	
			var times = Ti.UI.createLabel({
				color:'#6a6a6a',
				font:Theme.vehicleType.timesFont,
				left:0,
				bottom:0,
				text:'×',
				touchEnabled:false,
				height:'85%',
			});
			persons.add(times);
	
			var personicon = Ti.UI.createImageView({
			    height:21,
			    width:18,
			    image:'/images/vehicles/icon_person.png',
			    touchEnabled:false,
			});
			persons.add(personicon);
			
			attributes.add(persons);
		}
		
		if (vehicleType.get('wheelchair')) {
			var wheelchair = Ti.UI.createView({
				left:58,
				top:0,
				right:0,
				bottom:0,
			});
			var wheelchairicon = Ti.UI.createImageView({
			    height:21,
			    width:18,
			    image:'/images/vehicles/icon_wheelchair.png',
			    touchEnabled:false,
			});
			wheelchair.add(wheelchairicon);
			attributes.add(wheelchair);
		}

		container.add(attributes);
					
		var title = Ti.UI.createLabel({
			left:88,
			right:34,
			height:64,
			color:Theme.global.fontColor,
			font:Theme.vehicleType.titleFont,
			text:vehicleType.get('title'),
			touchEnabled:false,
			textAlign:Ti.UI.TEXT_ALIGNMENT_RIGHT,
		});
		container.add(title);
		
		var arrowIcon = Ti.UI.createImageView({
		    height:11,
		    width:11,
		    right: 13,
		    image:'/images/map/arrow.png',
		    touchEnabled:false,
		});
		container.add(arrowIcon);

		var rowBottomLine = Ti.UI.createView({
			left:0,
			right:0,
			bottom:0,
			height:'1px',
			backgroundColor:'#cacaca',
			touchEnabled:false,
		});
		row.add(rowBottomLine);
		Theme.animateButton(container);	
		return row;
	};

	var createForm = function(){
		var table = Ti.UI.createTableView({
			top:0,
	        bottom:0,
			left:13,
			right:0,
			separatorColor:'#ffffff',
			separatorStyle:Ti.Android ? 0 : Ti.UI.iPhone.TableViewSeparatorStyle.NONE,
			allowsSelection:Ti.Android,
		});

		return table;
	}; 
	var self = createForm();
	
	//set default to 4-seater
	// if(!vehicleType.get('id')){
		// vehicleType.set(vehicleTypes.get(1).toJSON());
	// }
	var rows = [];
	vehicleTypes.each(function(type){ 
		var row = createTableViewRow(type,(type.get('key')==vehicleType.get('key')));
		rows.push(row);
	}); //each
	self.data = rows;

	//table click action handler
	self.addEventListener('click',function(e){
		var row = selectedRow = e.row;
		var rowIndex = e.index;
		var id =  row.__vehicleType;
		vehicleType.set(vehicleTypes.get(id).toJSON());

		self.data = rows;
		ICABBI.navController.close(options.parent.window,{animated:true});
	});
	
	return self;
};

module.exports = VehicleType;

